@@ -65,7 +65,7 @@ static void   channels_new_callback             (GtkWidget     *dialog,
                                                  GimpChannel   *channel,
                                                  GimpContext   *context,
                                                  const gchar   *channel_name,
-                                                 const GimpRGB *channel_color,
+                                                 GeglColor     *channel_color,
                                                  gboolean       save_selection,
                                                  gboolean       channel_visible,
                                                  GimpColorTag   channel_color_tag,
@@ -78,7 +78,7 @@ static void   channels_edit_attributes_callback (GtkWidget     *dialog,
                                                  GimpChannel   *channel,
                                                  GimpContext   *context,
                                                  const gchar   *channel_name,
-                                                 const GimpRGB *channel_color,
+                                                 GeglColor     *channel_color,
                                                  gboolean       save_selection,
                                                  gboolean       channel_visible,
                                                  GimpColorTag   channel_color_tag,
@@ -127,7 +127,7 @@ channels_edit_attributes_cmd_callback (GimpAction *action,
                                            _("_Fill opacity:"),
                                            FALSE,
                                            gimp_object_get_name (channel),
-                                           &channel->color,
+                                           channel->color,
                                            gimp_item_get_visible (item),
                                            gimp_item_get_color_tag (item),
                                            gimp_item_get_lock_content (item),
@@ -173,7 +173,7 @@ channels_new_cmd_callback (GimpAction *action,
                                            _("_Fill opacity:"),
                                            TRUE,
                                            config->channel_new_name,
-                                           &config->channel_new_color,
+                                           config->channel_new_color,
                                            TRUE,
                                            GIMP_COLOR_TAG_NONE,
                                            FALSE,
@@ -204,7 +204,7 @@ channels_new_last_vals_cmd_callback (GimpAction *action,
                               gimp_image_get_width (image),
                               gimp_image_get_height (image),
                               config->channel_new_name,
-                              &config->channel_new_color);
+                              config->channel_new_color);
 
   gimp_drawable_fill (GIMP_DRAWABLE (channel),
                       action_data_get_context (data),
@@ -613,7 +613,7 @@ channels_new_callback (GtkWidget     *dialog,
                        GimpChannel   *channel,
                        GimpContext   *context,
                        const gchar   *channel_name,
-                       const GimpRGB *channel_color,
+                       GeglColor     *channel_color,
                        gboolean       save_selection,
                        gboolean       channel_visible,
                        GimpColorTag   channel_color_tag,
@@ -638,7 +638,7 @@ channels_new_callback (GtkWidget     *dialog,
 
       gimp_object_set_name (GIMP_OBJECT (channel),
                             config->channel_new_name);
-      gimp_channel_set_color (channel, &config->channel_new_color, FALSE);
+      gimp_channel_set_color (channel, config->channel_new_color, FALSE);
     }
   else
     {
@@ -646,7 +646,7 @@ channels_new_callback (GtkWidget     *dialog,
                                   gimp_image_get_width  (image),
                                   gimp_image_get_height (image),
                                   config->channel_new_name,
-                                  &config->channel_new_color);
+                                  config->channel_new_color);
 
       gimp_drawable_fill (GIMP_DRAWABLE (channel), context,
                           GIMP_FILL_TRANSPARENT);
@@ -671,7 +671,7 @@ channels_edit_attributes_callback (GtkWidget     *dialog,
                                    GimpChannel   *channel,
                                    GimpContext   *context,
                                    const gchar   *channel_name,
-                                   const GimpRGB *channel_color,
+                                   GeglColor     *channel_color,
                                    gboolean       save_selection,
                                    gboolean       channel_visible,
                                    GimpColorTag   channel_color_tag,
@@ -682,12 +682,12 @@ channels_edit_attributes_callback (GtkWidget     *dialog,
 {
   GimpItem *item = GIMP_ITEM (channel);
 
-  if (strcmp (channel_name, gimp_object_get_name (channel))              ||
-      gimp_rgba_distance (channel_color, &channel->color) > RGBA_EPSILON ||
-      channel_visible         != gimp_item_get_visible (item)            ||
-      channel_color_tag       != gimp_item_get_color_tag (item)          ||
-      channel_lock_content    != gimp_item_get_lock_content (item)       ||
-      channel_lock_position   != gimp_item_get_lock_position (item)      ||
+  if (strcmp (channel_name, gimp_object_get_name (channel))                   ||
+      ! gimp_color_is_perceptually_identical (channel_color, channel->color) ||
+      channel_visible         != gimp_item_get_visible (item)                 ||
+      channel_color_tag       != gimp_item_get_color_tag (item)               ||
+      channel_lock_content    != gimp_item_get_lock_content (item)            ||
+      channel_lock_position   != gimp_item_get_lock_position (item)           ||
       channel_lock_visibility != gimp_item_get_lock_visibility (item))
     {
       gimp_image_undo_group_start (image,
@@ -697,7 +697,7 @@ channels_edit_attributes_callback (GtkWidget     *dialog,
       if (strcmp (channel_name, gimp_object_get_name (channel)))
         gimp_item_rename (GIMP_ITEM (channel), channel_name, NULL);
 
-      if (gimp_rgba_distance (channel_color, &channel->color) > RGBA_EPSILON)
+      if (! gimp_color_is_perceptually_identical (channel_color, channel->color))
         gimp_channel_set_color (channel, channel_color, TRUE);
 
       if (channel_visible != gimp_item_get_visible (item))
