@@ -41,6 +41,7 @@
 #include "core/gimpcontext.h"
 #include "core/gimpdisplay.h"
 #include "core/gimpdrawable-operation.h"
+#include "core/gimpdrawablefilter.h"
 #include "core/gimpimage.h"
 #include "core/gimplayermask.h"
 #include "core/gimpparamspecs.h"
@@ -366,6 +367,7 @@ gimp_gegl_procedure_execute_async (GimpProcedure  *procedure,
       if (! strcmp (tool_name, "gimp-operation-tool"))
         {
           gimp_operation_tool_set_operation (GIMP_OPERATION_TOOL (active_tool),
+                                             gegl_procedure->filter,
                                              gegl_procedure->operation,
                                              gimp_procedure_get_label (procedure),
                                              gimp_procedure_get_label (procedure),
@@ -376,6 +378,74 @@ gimp_gegl_procedure_execute_async (GimpProcedure  *procedure,
 
       tool_manager_initialize_active (gimp, display);
 
+      /* For GIMP-specific GEGL operations, we need to copy over the
+       * config object stored in the GeglNode */
+      if (gegl_procedure->filter)
+        {
+          GeglNode *node;
+
+          GIMP_FILTER_TOOL (active_tool)->existing_filter = gegl_procedure->filter;
+          gimp_filter_set_active (GIMP_FILTER (gegl_procedure->filter), FALSE);
+
+          node = gimp_drawable_filter_get_operation (gegl_procedure->filter);
+
+          if (gimp_operation_config_is_custom (gimp, gegl_procedure->operation))
+            {
+              gegl_node_get (node,
+                             "config", &settings,
+                             NULL);
+            }
+          else
+            {
+              GParamSpec       **pspecs;
+              guint              n_pspecs;
+              gdouble            opacity;
+              GimpLayerMode      paint_mode;
+              GimpFilterRegion   region;
+
+              opacity    = gimp_drawable_filter_get_opacity (gegl_procedure->filter);
+              paint_mode = gimp_drawable_filter_get_paint_mode (gegl_procedure->filter);
+              region     = gimp_drawable_filter_get_region (gegl_procedure->filter);
+
+              settings =
+                g_object_new (gimp_operation_config_get_type (active_tool->tool_info->gimp,
+                                                              gegl_procedure->operation,
+                                                              gimp_tool_get_icon_name (active_tool),
+                                                              GIMP_TYPE_OPERATION_SETTINGS),
+                              NULL);
+
+              pspecs = gegl_operation_list_properties (gegl_procedure->operation, &n_pspecs);
+
+              for (gint i = 0; i < n_pspecs; i++)
+                {
+                  GValue      value      = G_VALUE_INIT;
+                  GParamSpec *pspec      = pspecs[i];
+                  GParamSpec *gimp_pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (settings),
+                                                                         pspec->name);
+
+                  g_value_init (&value, pspec->value_type);
+                  gegl_node_get_property (node, pspec->name,
+                                          &value);
+
+                  if (gimp_pspec)
+                    g_object_set_property (G_OBJECT (settings), gimp_pspec->name,
+                                           &value);
+                  else
+                    g_critical ("%s: property '%s' of operation '%s' doesn't exist in config %s",
+                                G_STRFUNC, pspec->name, gegl_procedure->operation,
+                                g_type_name (G_TYPE_FROM_INSTANCE (settings)));
+                  g_value_unset (&value);
+                }
+              g_free (pspecs);
+
+              g_object_set (settings,
+                            "gimp-opacity", opacity,
+                            "gimp-mode",    paint_mode,
+                            "gimp-region",  region,
+                            NULL);
+            }
+        }
+
       if (settings)
         gimp_filter_tool_set_config (GIMP_FILTER_TOOL (active_tool),
                                      GIMP_CONFIG (settings));
@@ -386,15 +456,16 @@ gimp_gegl_procedure_execute_async (GimpProcedure  *procedure,
 /*  public functions  */
 
 GimpProcedure *
-gimp_gegl_procedure_new (Gimp        *gimp,
-                         GimpRunMode  default_run_mode,
-                         GimpObject  *default_settings,
-                         const gchar *operation,
-                         const gchar *name,
-                         const gchar *menu_label,
-                         const gchar *tooltip,
-                         const gchar *icon_name,
-                         const gchar *help_id)
+gimp_gegl_procedure_new (Gimp               *gimp,
+                         GimpDrawableFilter *filter,
+                         GimpRunMode         default_run_mode,
+                         GimpObject         *default_settings,
+                         const gchar        *operation,
+                         const gchar        *name,
+                         const gchar        *menu_label,
+                         const gchar        *tooltip,
+                         const gchar        *icon_name,
+                         const gchar        *help_id)
 {
   GimpProcedure     *procedure;
   GimpGeglProcedure *gegl_procedure;
@@ -412,6 +483,7 @@ gimp_gegl_procedure_new (Gimp        *gimp,
 
   gegl_procedure = GIMP_GEGL_PROCEDURE (procedure);
 
+  gegl_procedure->filter           = filter;
   gegl_procedure->operation        = g_strdup (operation);
   gegl_procedure->default_run_mode = default_run_mode;
   gegl_procedure->menu_label       = g_strdup (menu_label);
@@ -442,7 +514,7 @@ gimp_gegl_procedure_new (Gimp        *gimp,
                                                       FALSE,
                                                       GIMP_PARAM_READWRITE));
   gimp_procedure_add_argument (procedure,
-                               g_param_spec_int ("n-drawables",
+                               g_param_spec_int ("num-drawables",
                                                  "N drawables",
                                                  "The number of drawables",
                                                  0, G_MAXINT32, 0,
