@@ -461,23 +461,22 @@ void
 gradient_editor_actions_update (GimpActionGroup *group,
                                 gpointer         data)
 {
-  GimpGradientEditor  *editor         = GIMP_GRADIENT_EDITOR (data);
-  GimpDataEditor      *data_editor    = GIMP_DATA_EDITOR (data);
+  GimpGradientEditor  *editor          = GIMP_GRADIENT_EDITOR (data);
+  GimpDataEditor      *data_editor     = GIMP_DATA_EDITOR (data);
   GimpGradient        *gradient;
-  gboolean             editable       = FALSE;
-  GimpRGB              left_color;
-  GimpRGB              right_color;
-  GimpRGB              left_seg_color;
-  GimpRGB              right_seg_color;
-  GimpRGB              fg;
-  GimpRGB              bg;
-  gboolean             blending_equal = TRUE;
-  gboolean             coloring_equal = TRUE;
-  gboolean             left_editable  = TRUE;
-  gboolean             right_editable = TRUE;
-  gboolean             selection      = FALSE;
-  gboolean             delete         = FALSE;
-  gboolean             edit_active    = FALSE;
+  gboolean             editable        = FALSE;
+  GeglColor           *color;
+  GeglColor           *left_color      = NULL;
+  GeglColor           *right_color     = NULL;
+  GeglColor           *left_seg_color  = NULL;
+  GeglColor           *right_seg_color = NULL;
+  gboolean             blending_equal  = TRUE;
+  gboolean             coloring_equal  = TRUE;
+  gboolean             left_editable   = TRUE;
+  gboolean             right_editable  = TRUE;
+  gboolean             selection       = FALSE;
+  gboolean             delete          = FALSE;
+  gboolean             edit_active     = FALSE;
 
   gradient = GIMP_GRADIENT (data_editor->data);
 
@@ -492,35 +491,31 @@ gradient_editor_actions_update (GimpActionGroup *group,
       if (data_editor->data_editable)
         editable = TRUE;
 
-      gimp_gradient_segment_get_left_flat_color (gradient,
-                                                 data_editor->context,
-                                                 editor->control_sel_l,
-                                                 &left_color);
+      left_color = gimp_gradient_segment_get_left_flat_color (gradient,
+                                                              data_editor->context,
+                                                              editor->control_sel_l);
 
       if (editor->control_sel_l->prev)
         left_seg = editor->control_sel_l->prev;
       else
         left_seg = gimp_gradient_segment_get_last (editor->control_sel_l);
 
-      gimp_gradient_segment_get_right_flat_color (gradient,
-                                                  data_editor->context,
-                                                  left_seg,
-                                                  &left_seg_color);
+      left_seg_color = gimp_gradient_segment_get_right_flat_color (gradient,
+                                                                   data_editor->context,
+                                                                   left_seg);
 
-      gimp_gradient_segment_get_right_flat_color (gradient,
-                                                  data_editor->context,
-                                                  editor->control_sel_r,
-                                                  &right_color);
+      right_color = gimp_gradient_segment_get_right_flat_color (gradient,
+                                                                data_editor->context,
+                                                                editor->control_sel_r);
 
       if (editor->control_sel_r->next)
         right_seg = editor->control_sel_r->next;
       else
         right_seg = gimp_gradient_segment_get_first (editor->control_sel_r);
 
-      gimp_gradient_segment_get_left_flat_color (gradient,
-                                                 data_editor->context,
-                                                 right_seg,
-                                                 &right_seg_color);
+      right_seg_color = gimp_gradient_segment_get_left_flat_color (gradient,
+                                                                   data_editor->context,
+                                                                   right_seg);
 
       left_editable  = (editor->control_sel_l->left_color_type ==
                         GIMP_GRADIENT_COLOR_FIXED);
@@ -546,12 +541,6 @@ gradient_editor_actions_update (GimpActionGroup *group,
       delete    = (editor->control_sel_l->prev || editor->control_sel_r->next);
     }
 
-  if (data_editor->context)
-    {
-      gimp_context_get_foreground (data_editor->context, &fg);
-      gimp_context_get_background (data_editor->context, &bg);
-    }
-
   /*  pretend the gradient not being editable while the dialog is
    *  insensitive. prevents the gradient from being modified while a
    *  dialog is running. bug #161411 --mitch
@@ -613,23 +602,22 @@ gradient_editor_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("gradient-editor-load-left-left-neighbor",  editable);
   SET_SENSITIVE ("gradient-editor-load-left-right-endpoint", editable);
 
+  color = gegl_color_new (NULL);
+
   if (gradient)
     {
-      SET_COLOR ("gradient-editor-left-color",
-                 &left_color, FALSE);
-      SET_COLOR ("gradient-editor-load-left-left-neighbor",
-                 &left_seg_color, FALSE);
-      SET_COLOR ("gradient-editor-load-left-right-endpoint",
-                 &right_color, FALSE);
+      SET_COLOR ("gradient-editor-left-color", left_color, FALSE);
+      SET_COLOR ("gradient-editor-load-left-left-neighbor", left_seg_color, FALSE);
+      SET_COLOR ("gradient-editor-load-left-right-endpoint", right_color, FALSE);
     }
 
   SET_SENSITIVE ("gradient-editor-load-left-fg", left_editable);
   SET_SENSITIVE ("gradient-editor-load-left-bg", left_editable);
 
   SET_COLOR ("gradient-editor-load-left-fg",
-             data_editor->context ? &fg : NULL, FALSE);
+             data_editor->context ? gimp_context_get_foreground (data_editor->context) : NULL, FALSE);
   SET_COLOR ("gradient-editor-load-left-bg",
-             data_editor->context ? &bg : NULL, FALSE);
+             data_editor->context ? gimp_context_get_background (data_editor->context) : NULL, FALSE);
 
   SET_SENSITIVE ("gradient-editor-load-left-01", left_editable);
   SET_SENSITIVE ("gradient-editor-load-left-02", left_editable);
@@ -642,16 +630,26 @@ gradient_editor_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("gradient-editor-load-left-09", left_editable);
   SET_SENSITIVE ("gradient-editor-load-left-10", left_editable);
 
-  SET_COLOR ("gradient-editor-load-left-01", &editor->saved_colors[0], TRUE);
-  SET_COLOR ("gradient-editor-load-left-02", &editor->saved_colors[1], TRUE);
-  SET_COLOR ("gradient-editor-load-left-03", &editor->saved_colors[2], TRUE);
-  SET_COLOR ("gradient-editor-load-left-04", &editor->saved_colors[3], TRUE);
-  SET_COLOR ("gradient-editor-load-left-05", &editor->saved_colors[4], TRUE);
-  SET_COLOR ("gradient-editor-load-left-06", &editor->saved_colors[5], TRUE);
-  SET_COLOR ("gradient-editor-load-left-07", &editor->saved_colors[6], TRUE);
-  SET_COLOR ("gradient-editor-load-left-08", &editor->saved_colors[7], TRUE);
-  SET_COLOR ("gradient-editor-load-left-09", &editor->saved_colors[8], TRUE);
-  SET_COLOR ("gradient-editor-load-left-10", &editor->saved_colors[9], TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[0]);
+  SET_COLOR ("gradient-editor-load-left-01", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[1]);
+  SET_COLOR ("gradient-editor-load-left-02", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[2]);
+  SET_COLOR ("gradient-editor-load-left-03", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[3]);
+  SET_COLOR ("gradient-editor-load-left-04", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[4]);
+  SET_COLOR ("gradient-editor-load-left-05", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[5]);
+  SET_COLOR ("gradient-editor-load-left-06", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[6]);
+  SET_COLOR ("gradient-editor-load-left-07", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[7]);
+  SET_COLOR ("gradient-editor-load-left-08", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[8]);
+  SET_COLOR ("gradient-editor-load-left-09", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[9]);
+  SET_COLOR ("gradient-editor-load-left-10", color, TRUE);
 
   SET_SENSITIVE ("gradient-editor-save-left-01", gradient);
   SET_SENSITIVE ("gradient-editor-save-left-02", gradient);
@@ -664,16 +662,26 @@ gradient_editor_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("gradient-editor-save-left-09", gradient);
   SET_SENSITIVE ("gradient-editor-save-left-10", gradient);
 
-  SET_COLOR ("gradient-editor-save-left-01", &editor->saved_colors[0], TRUE);
-  SET_COLOR ("gradient-editor-save-left-02", &editor->saved_colors[1], TRUE);
-  SET_COLOR ("gradient-editor-save-left-03", &editor->saved_colors[2], TRUE);
-  SET_COLOR ("gradient-editor-save-left-04", &editor->saved_colors[3], TRUE);
-  SET_COLOR ("gradient-editor-save-left-05", &editor->saved_colors[4], TRUE);
-  SET_COLOR ("gradient-editor-save-left-06", &editor->saved_colors[5], TRUE);
-  SET_COLOR ("gradient-editor-save-left-07", &editor->saved_colors[6], TRUE);
-  SET_COLOR ("gradient-editor-save-left-08", &editor->saved_colors[7], TRUE);
-  SET_COLOR ("gradient-editor-save-left-09", &editor->saved_colors[8], TRUE);
-  SET_COLOR ("gradient-editor-save-left-10", &editor->saved_colors[9], TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[0]);
+  SET_COLOR ("gradient-editor-save-left-01", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[1]);
+  SET_COLOR ("gradient-editor-save-left-02", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[2]);
+  SET_COLOR ("gradient-editor-save-left-03", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[3]);
+  SET_COLOR ("gradient-editor-save-left-04", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[4]);
+  SET_COLOR ("gradient-editor-save-left-05", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[5]);
+  SET_COLOR ("gradient-editor-save-left-06", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[6]);
+  SET_COLOR ("gradient-editor-save-left-07", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[7]);
+  SET_COLOR ("gradient-editor-save-left-08", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[8]);
+  SET_COLOR ("gradient-editor-save-left-09", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[9]);
+  SET_COLOR ("gradient-editor-save-left-10", color, TRUE);
 
   SET_SENSITIVE ("gradient-editor-right-color-fixed",                  editable);
   SET_SENSITIVE ("gradient-editor-right-color-foreground",             editable);
@@ -709,21 +717,18 @@ gradient_editor_actions_update (GimpActionGroup *group,
 
   if (gradient)
     {
-      SET_COLOR ("gradient-editor-right-color",
-                 &right_color, FALSE);
-      SET_COLOR ("gradient-editor-load-right-right-neighbor",
-                 &right_seg_color, FALSE);
-      SET_COLOR ("gradient-editor-load-right-left-endpoint",
-                 &left_color, FALSE);
+      SET_COLOR ("gradient-editor-right-color", right_color, FALSE);
+      SET_COLOR ("gradient-editor-load-right-right-neighbor", right_seg_color, FALSE);
+      SET_COLOR ("gradient-editor-load-right-left-endpoint", left_color, FALSE);
     }
 
   SET_SENSITIVE ("gradient-editor-load-right-fg", right_editable);
   SET_SENSITIVE ("gradient-editor-load-right-bg", right_editable);
 
   SET_COLOR ("gradient-editor-load-right-fg",
-             data_editor->context ? &fg : NULL, FALSE);
+             data_editor->context ? gimp_context_get_foreground (data_editor->context) : NULL, FALSE);
   SET_COLOR ("gradient-editor-load-right-bg",
-             data_editor->context ? &bg : NULL, FALSE);
+             data_editor->context ? gimp_context_get_background (data_editor->context) : NULL, FALSE);
 
   SET_SENSITIVE ("gradient-editor-load-right-01", right_editable);
   SET_SENSITIVE ("gradient-editor-load-right-02", right_editable);
@@ -736,16 +741,26 @@ gradient_editor_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("gradient-editor-load-right-09", right_editable);
   SET_SENSITIVE ("gradient-editor-load-right-10", right_editable);
 
-  SET_COLOR ("gradient-editor-load-right-01", &editor->saved_colors[0], TRUE);
-  SET_COLOR ("gradient-editor-load-right-02", &editor->saved_colors[1], TRUE);
-  SET_COLOR ("gradient-editor-load-right-03", &editor->saved_colors[2], TRUE);
-  SET_COLOR ("gradient-editor-load-right-04", &editor->saved_colors[3], TRUE);
-  SET_COLOR ("gradient-editor-load-right-05", &editor->saved_colors[4], TRUE);
-  SET_COLOR ("gradient-editor-load-right-06", &editor->saved_colors[5], TRUE);
-  SET_COLOR ("gradient-editor-load-right-07", &editor->saved_colors[6], TRUE);
-  SET_COLOR ("gradient-editor-load-right-08", &editor->saved_colors[7], TRUE);
-  SET_COLOR ("gradient-editor-load-right-09", &editor->saved_colors[8], TRUE);
-  SET_COLOR ("gradient-editor-load-right-10", &editor->saved_colors[9], TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[0]);
+  SET_COLOR ("gradient-editor-load-right-01", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[1]);
+  SET_COLOR ("gradient-editor-load-right-02", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[2]);
+  SET_COLOR ("gradient-editor-load-right-03", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[3]);
+  SET_COLOR ("gradient-editor-load-right-04", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[4]);
+  SET_COLOR ("gradient-editor-load-right-05", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[5]);
+  SET_COLOR ("gradient-editor-load-right-06", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[6]);
+  SET_COLOR ("gradient-editor-load-right-07", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[7]);
+  SET_COLOR ("gradient-editor-load-right-08", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[8]);
+  SET_COLOR ("gradient-editor-load-right-09", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[9]);
+  SET_COLOR ("gradient-editor-load-right-10", color, TRUE);
 
   SET_SENSITIVE ("gradient-editor-save-right-01", gradient);
   SET_SENSITIVE ("gradient-editor-save-right-02", gradient);
@@ -758,16 +773,26 @@ gradient_editor_actions_update (GimpActionGroup *group,
   SET_SENSITIVE ("gradient-editor-save-right-09", gradient);
   SET_SENSITIVE ("gradient-editor-save-right-10", gradient);
 
-  SET_COLOR ("gradient-editor-save-right-01", &editor->saved_colors[0], TRUE);
-  SET_COLOR ("gradient-editor-save-right-02", &editor->saved_colors[1], TRUE);
-  SET_COLOR ("gradient-editor-save-right-03", &editor->saved_colors[2], TRUE);
-  SET_COLOR ("gradient-editor-save-right-04", &editor->saved_colors[3], TRUE);
-  SET_COLOR ("gradient-editor-save-right-05", &editor->saved_colors[4], TRUE);
-  SET_COLOR ("gradient-editor-save-right-06", &editor->saved_colors[5], TRUE);
-  SET_COLOR ("gradient-editor-save-right-07", &editor->saved_colors[6], TRUE);
-  SET_COLOR ("gradient-editor-save-right-08", &editor->saved_colors[7], TRUE);
-  SET_COLOR ("gradient-editor-save-right-09", &editor->saved_colors[8], TRUE);
-  SET_COLOR ("gradient-editor-save-right-10", &editor->saved_colors[9], TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[0]);
+  SET_COLOR ("gradient-editor-save-right-01", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[1]);
+  SET_COLOR ("gradient-editor-save-right-02", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[2]);
+  SET_COLOR ("gradient-editor-save-right-03", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[3]);
+  SET_COLOR ("gradient-editor-save-right-04", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[4]);
+  SET_COLOR ("gradient-editor-save-right-05", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[5]);
+  SET_COLOR ("gradient-editor-save-right-06", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[6]);
+  SET_COLOR ("gradient-editor-save-right-07", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[7]);
+  SET_COLOR ("gradient-editor-save-right-08", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[8]);
+  SET_COLOR ("gradient-editor-save-right-09", color, TRUE);
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &editor->saved_colors[9]);
+  SET_COLOR ("gradient-editor-save-right-10", color, TRUE);
 
   SET_SENSITIVE ("gradient-editor-flip",           editable);
   SET_SENSITIVE ("gradient-editor-replicate",      editable);
@@ -902,4 +927,10 @@ gradient_editor_actions_update (GimpActionGroup *group,
 #undef SET_LABEL
 #undef SET_SENSITIVE
 #undef SET_VISIBLE
+
+  g_object_unref (color);
+  g_clear_object (&left_color);
+  g_clear_object (&right_color);
+  g_clear_object (&left_seg_color);
+  g_clear_object (&right_seg_color);
 }
