@@ -48,6 +48,7 @@
 #include "gimpimage-undo.h"
 #include "gimpimage-undo-push.h"
 #include "gimpobjectqueue.h"
+#include "gimppalette.h"
 #include "gimpprogress.h"
 
 #include "gimp-intl.h"
@@ -815,20 +816,6 @@ gimp_image_get_color_transform_to_srgb_u8 (GimpImage *image)
   return private->transform_to_srgb_u8;
 }
 
-GimpColorTransform *
-gimp_image_get_color_transform_from_srgb_u8 (GimpImage *image)
-{
-  GimpImagePrivate *private;
-
-  g_return_val_if_fail (GIMP_IS_IMAGE (image), NULL);
-
-  private = GIMP_IMAGE_GET_PRIVATE (image);
-
-  gimp_image_create_color_transforms (image);
-
-  return private->transform_from_srgb_u8;
-}
-
 GimpColorTransform *
 gimp_image_get_color_transform_to_srgb_double (GimpImage *image)
 {
@@ -857,65 +844,6 @@ gimp_image_get_color_transform_from_srgb_double (GimpImage *image)
   return private->transform_from_srgb_double;
 }
 
-void
-gimp_image_color_profile_pixel_to_rgb (GimpImage  *image,
-                                       const Babl *pixel_format,
-                                       gpointer    pixel,
-                                       GimpRGB    *color)
-{
-  GimpColorProfile   *profile = NULL;
-  const Babl         *space   = NULL;
-
-  g_return_if_fail (GIMP_IS_IMAGE (image));
-
-  profile = gimp_image_get_color_profile (image);
-
-  if (profile)
-    space = gimp_color_profile_get_space (profile,
-                                          GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
-                                          NULL);
-  if (profile)
-    {
-      babl_process (babl_fish (pixel_format,
-                               babl_format_with_space ("R'G'B'A double", space)),
-                    pixel, color, 1);
-    }
-  else
-    {
-      gimp_rgba_set_pixel (color, pixel_format, pixel);
-    }
-}
-
-void
-gimp_image_color_profile_rgb_to_pixel (GimpImage     *image,
-                                       const GimpRGB *color,
-                                       const Babl    *pixel_format,
-                                       gpointer       pixel)
-{
-  GimpColorProfile   *profile = NULL;
-  const Babl         *space   = NULL;
-
-  g_return_if_fail (GIMP_IS_IMAGE (image));
-
-  profile = gimp_image_get_color_profile (image);
-
-  if (profile)
-    space = gimp_color_profile_get_space (profile,
-                                          GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
-                                          NULL);
-
-  if (profile)
-    {
-      babl_process (babl_fish (babl_format_with_space ("R'G'B'A double", space),
-                               pixel_format),
-                    color, pixel, 1);
-    }
-  else
-    {
-      gimp_rgba_get_pixel (color, pixel_format, pixel);
-    }
-}
-
 
 /*  internal API  */
 
@@ -938,7 +866,6 @@ _gimp_image_free_color_transforms (GimpImage *image)
   GimpImagePrivate *private = GIMP_IMAGE_GET_PRIVATE (image);
 
   g_clear_object (&private->transform_to_srgb_u8);
-  g_clear_object (&private->transform_from_srgb_u8);
   g_clear_object (&private->transform_to_srgb_double);
   g_clear_object (&private->transform_from_srgb_double);
 
@@ -1065,48 +992,25 @@ gimp_image_convert_profile_colormap (GimpImage                *image,
                                      gboolean                  bpc,
                                      GimpProgress             *progress)
 {
-  GimpColorTransform      *transform;
-  const Babl              *src_format;
-  const Babl              *dest_format;
+  GimpImagePrivate        *private = GIMP_IMAGE_GET_PRIVATE (image);
   GimpColorTransformFlags  flags = 0;
-  guchar                  *cmap;
-  gint                     n_colors;
-
-  n_colors = gimp_image_get_colormap_size (image);
-  cmap     = gimp_image_get_colormap (image);
+  GimpPalette             *palette;
+  const Babl              *space;
+  const Babl              *format;
 
   if (bpc)
+    /* TODO: current implementation ignores the black point compensation
+     * choice because babl doesn't have BCP support yet.
+     * Previous code was using gimp_color_transform_new() which used
+     * LittleCMS directly instead.
+     * This should be fixed.
+     */
     flags |= GIMP_COLOR_TRANSFORM_FLAGS_BLACK_POINT_COMPENSATION;
 
-  src_format  = gimp_color_profile_get_format (src_profile,
-                                               babl_format ("R'G'B' u8"),
-                                               GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
-                                               NULL);
-  dest_format = gimp_color_profile_get_format (dest_profile,
-                                               babl_format ("R'G'B' u8"),
-                                               GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
-                                               NULL);
-
-  transform = gimp_color_transform_new (src_profile,  src_format,
-                                        dest_profile, dest_format,
-                                        intent, flags);
-
-  if (transform)
-    {
-      gimp_color_transform_process_pixels (transform,
-                                           babl_format ("R'G'B' u8"), cmap,
-                                           babl_format ("R'G'B' u8"), cmap,
-                                           n_colors);
-      g_object_unref (transform);
-
-      gimp_image_set_colormap (image, cmap, n_colors, TRUE);
-    }
-  else
-    {
-      g_warning ("gimp_color_transform_new() failed!");
-    }
-
-  g_free (cmap);
+  palette  = gimp_image_get_colormap_palette (image);
+  space    = gimp_image_get_layer_space (image);
+  format   = gimp_babl_format (GIMP_RGB, private->precision, FALSE, space);
+  gimp_palette_restrict_format (palette, format);
 }
 
 static void
@@ -1162,14 +1066,6 @@ gimp_image_create_color_transforms (GimpImage *image)
                                   GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL,
                                   flags);
 
-      private->transform_from_srgb_u8 =
-        gimp_color_transform_new (srgb_profile,
-                                  babl_format ("R'G'B'A u8"),
-                                  private->color_profile,
-                                  gimp_image_get_layer_format (image, TRUE),
-                                  GIMP_COLOR_RENDERING_INTENT_PERCEPTUAL,
-                                  flags);
-
       private->transform_to_srgb_double =
         gimp_color_transform_new (private->color_profile,
                                   gimp_image_get_layer_format (image, TRUE),
