@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_TEMPLATE_H__
@@ -27,17 +27,21 @@
 
 #define GIMP_TEMPLATE_PARAM_COPY_FIRST (1 << (8 + G_PARAM_USER_SHIFT))
 
-/*  The default image aspect ratio is the golden mean. We use
- *  two adjacent fibonacci numbers for the unstable series and
- *  some less odd values for the stable version.
- */
-
 #ifdef GIMP_UNSTABLE
-#define GIMP_DEFAULT_IMAGE_WIDTH   610
-#define GIMP_DEFAULT_IMAGE_HEIGHT  377
+/* Uncommon ratio, with at least one odd value, to encourage testing
+ * GIMP with unusual numbers.
+ * It also has to be a higher resolution, to push GIMP a little further
+ * in tests. */
+#define GIMP_DEFAULT_IMAGE_WIDTH   2001
+#define GIMP_DEFAULT_IMAGE_HEIGHT  1984
 #else
-#define GIMP_DEFAULT_IMAGE_WIDTH   640
-#define GIMP_DEFAULT_IMAGE_HEIGHT  400
+/* 1366x768 is the most common screen resolution in 2016.
+ * 1920x1080 is the second most common.
+ * Since GIMP targets advanced graphics artists, let's go for the
+ * highest common dimension.
+ */
+#define GIMP_DEFAULT_IMAGE_WIDTH   1920
+#define GIMP_DEFAULT_IMAGE_HEIGHT  1080
 #endif
 
 
@@ -77,7 +81,16 @@ gdouble             gimp_template_get_resolution_x    (GimpTemplate *template);
 gdouble             gimp_template_get_resolution_y    (GimpTemplate *template);
 GimpUnit            gimp_template_get_resolution_unit (GimpTemplate *template);
 
-GimpImageBaseType   gimp_template_get_image_type      (GimpTemplate *template);
+GimpImageBaseType   gimp_template_get_base_type       (GimpTemplate *template);
+GimpPrecision       gimp_template_get_precision       (GimpTemplate *template);
+
+GimpColorProfile  * gimp_template_get_color_profile   (GimpTemplate *template);
+GimpColorProfile  * gimp_template_get_simulation_profile
+                                                      (GimpTemplate *template);
+GimpColorRenderingIntent gimp_template_get_simulation_intent
+                                                      (GimpTemplate *template);
+gboolean            gimp_template_get_simulation_bpc  (GimpTemplate *template);
+
 GimpFillType        gimp_template_get_fill_type       (GimpTemplate *template);
 
 const gchar       * gimp_template_get_comment         (GimpTemplate *template);
