@@ -216,7 +216,7 @@ item_options_dialog_new (GimpImage               *image,
            list = g_list_next (list))
         {
           GimpColorTag  color_tag;
-          GimpRGB       color;
+          GimpRGB       rgb;
           GtkWidget    *image;
 
           radio = list->data;
@@ -228,15 +228,19 @@ item_options_dialog_new (GimpImage               *image,
           color_tag = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (radio),
                                                           "gimp-item-data"));
 
-          if (gimp_get_color_tag_color (color_tag, &color, FALSE))
+          if (gimp_get_color_tag_color (color_tag, &rgb, FALSE))
             {
-              gint w, h;
+              GeglColor *color = gegl_color_new (NULL);
+              gint       w, h;
 
-              image = gimp_color_area_new (&color, GIMP_COLOR_AREA_FLAT, 0);
+              gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), &rgb);
+              image = gimp_color_area_new (color, GIMP_COLOR_AREA_FLAT, 0);
               gimp_color_area_set_color_config (GIMP_COLOR_AREA (image),
                                                 context->gimp->config->color_management);
               gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
               gtk_widget_set_size_request (image, w, h);
+
+              g_object_unref (color);
             }
           else
             {
