@@ -86,7 +86,7 @@ static void   gimp_display_shell_drop_pattern   (GtkWidget       *widget,
 static void   gimp_display_shell_drop_color     (GtkWidget       *widget,
                                                  gint             x,
                                                  gint             y,
-                                                 const GimpRGB   *color,
+                                                 GeglColor       *color,
                                                  gpointer         data);
 static void   gimp_display_shell_drop_buffer    (GtkWidget       *widget,
                                                  gint             x,
@@ -388,15 +388,15 @@ gimp_display_shell_dnd_fill (GimpDisplayShell *shell,
           (gimp_fill_options_get_style (options) == GIMP_FILL_STYLE_FG_COLOR ||
            gimp_fill_options_get_style (options) == GIMP_FILL_STYLE_BG_COLOR))
         {
-          GimpRGB color;
+          GeglColor *color;
 
           if (gimp_fill_options_get_style (options) == GIMP_FILL_STYLE_FG_COLOR)
-            gimp_context_get_foreground (GIMP_CONTEXT (options), &color);
+            color = gimp_context_get_foreground (GIMP_CONTEXT (options));
           else
-            gimp_context_get_background (GIMP_CONTEXT (options), &color);
+            color = gimp_context_get_background (GIMP_CONTEXT (options));
 
           gimp_text_layer_set (iter->data, NULL,
-                               "color", &color,
+                               "color", color,
                                NULL);
         }
       else
@@ -436,7 +436,7 @@ static void
 gimp_display_shell_drop_color (GtkWidget     *widget,
                                gint           x,
                                gint           y,
-                               const GimpRGB *color,
+                               GeglColor     *color,
                                gpointer       data)
 {
   GimpDisplayShell *shell   = GIMP_DISPLAY_SHELL (data);
