@@ -40,6 +40,7 @@
 #include "plug-in/gimpenvirontable.h"
 #include "plug-in/gimppluginmanager.h"
 
+#include "display/gimpcanvas-style.h"
 #include "display/gimpdisplay.h"
 #include "display/gimpdisplay-foreach.h"
 #include "display/gimpdisplayshell.h"
@@ -63,6 +64,7 @@
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimpmenufactory.h"
 #include "widgets/gimpmessagebox.h"
+#include "widgets/gimpradioaction.h"
 #include "widgets/gimpsessioninfo.h"
 #include "widgets/gimpuimanager.h"
 #include "widgets/gimpwidgets-utils.h"
@@ -85,6 +87,12 @@
 #include "splash.h"
 #include "themes.h"
 
+#ifdef G_OS_WIN32
+#include <windef.h>
+#include <winbase.h>
+#include <windows.h>
+#endif
+
 #ifdef GDK_WINDOWING_QUARTZ
 #import <AppKit/AppKit.h>
 
@@ -105,48 +113,48 @@
 
 /*  local function prototypes  */
 
-static gchar    * gui_sanity_check              (void);
-static void       gui_help_func                 (const gchar        *help_id,
-                                                 gpointer            help_data);
-static gboolean   gui_get_background_func       (GimpRGB            *color);
-static gboolean   gui_get_foreground_func       (GimpRGB            *color);
+static gchar     * gui_sanity_check              (void);
+static void        gui_help_func                 (const gchar        *help_id,
+                                                  gpointer            help_data);
+static GeglColor * gui_get_background_func       (void);
+static GeglColor * gui_get_foreground_func       (void);
 
-static void       gui_initialize_after_callback (Gimp               *gimp,
-                                                 GimpInitStatusFunc  callback);
+static void        gui_initialize_after_callback (Gimp               *gimp,
+                                                  GimpInitStatusFunc  callback);
 
-static void       gui_restore_callback          (Gimp               *gimp,
-                                                 GimpInitStatusFunc  callback);
-static void       gui_restore_after_callback    (Gimp               *gimp,
-                                                 GimpInitStatusFunc  callback);
+static void        gui_restore_callback          (Gimp               *gimp,
+                                                  GimpInitStatusFunc  callback);
+static void        gui_restore_after_callback    (Gimp               *gimp,
+                                                  GimpInitStatusFunc  callback);
 
-static gboolean   gui_exit_callback             (Gimp               *gimp,
-                                                 gboolean            force);
-static gboolean   gui_exit_after_callback       (Gimp               *gimp,
-                                                 gboolean            force);
+static gboolean    gui_exit_callback             (Gimp               *gimp,
+                                                  gboolean            force);
+static gboolean    gui_exit_after_callback       (Gimp               *gimp,
+                                                  gboolean            force);
 
-static void       gui_show_help_button_notify   (GimpGuiConfig      *gui_config,
-                                                 GParamSpec         *pspec,
-                                                 Gimp               *gimp);
-static void       gui_user_manual_notify        (GimpGuiConfig      *gui_config,
-                                                 GParamSpec         *pspec,
-                                                 Gimp               *gimp);
-static void       gui_single_window_mode_notify (GimpGuiConfig      *gui_config,
-                                                 GParamSpec         *pspec,
-                                                 GimpUIConfigurer   *ui_configurer);
+static void        gui_show_help_button_notify   (GimpGuiConfig      *gui_config,
+                                                  GParamSpec         *pspec,
+                                                  Gimp               *gimp);
+static void        gui_user_manual_notify        (GimpGuiConfig      *gui_config,
+                                                  GParamSpec         *pspec,
+                                                  Gimp               *gimp);
+static void        gui_single_window_mode_notify (GimpGuiConfig      *gui_config,
+                                                  GParamSpec         *pspec,
+                                                  GimpUIConfigurer   *ui_configurer);
 
-static void       gui_clipboard_changed         (Gimp               *gimp);
+static void        gui_clipboard_changed         (Gimp               *gimp);
 
-static void       gui_menu_show_tooltip         (GimpUIManager      *manager,
-                                                 const gchar        *tooltip,
-                                                 Gimp               *gimp);
-static void       gui_menu_hide_tooltip         (GimpUIManager      *manager,
-                                                 Gimp               *gimp);
+static void        gui_menu_show_tooltip         (GimpUIManager      *manager,
+                                                  const gchar        *tooltip,
+                                                  Gimp               *gimp);
+static void        gui_menu_hide_tooltip         (GimpUIManager      *manager,
+                                                  Gimp               *gimp);
 
-static void       gui_display_changed           (GimpContext        *context,
-                                                 GimpDisplay        *display,
-                                                 Gimp               *gimp);
+static void        gui_display_changed           (GimpContext        *context,
+                                                  GimpDisplay        *display,
+                                                  Gimp               *gimp);
 
-static void       gui_check_unique_accelerators (Gimp               *gimp);
+static void        gui_check_unique_accelerators (Gimp               *gimp);
 
 
 /*  private variables  */
@@ -270,6 +278,7 @@ gui_init (Gimp         *gimp,
   gimp_dnd_init (gimp);
 
   themes_init (gimp);
+  gimp_canvas_styles_init ();
 
   initial_monitor = gimp_get_monitor_at_pointer ();
 
@@ -404,26 +413,28 @@ gui_help_func (const gchar *help_id,
   gimp_help (the_gui_gimp, NULL, NULL, help_id);
 }
 
-static gboolean
-gui_get_foreground_func (GimpRGB *color)
+static GeglColor *
+gui_get_foreground_func (void)
 {
-  g_return_val_if_fail (color != NULL, FALSE);
+  GeglColor *color;
+
   g_return_val_if_fail (GIMP_IS_GIMP (the_gui_gimp), FALSE);
 
-  gimp_context_get_foreground (gimp_get_user_context (the_gui_gimp), color);
+  color = gimp_context_get_foreground (gimp_get_user_context (the_gui_gimp));
 
-  return TRUE;
+  return gegl_color_duplicate (color);
 }
 
-static gboolean
-gui_get_background_func (GimpRGB *color)
+static GeglColor *
+gui_get_background_func (void)
 {
-  g_return_val_if_fail (color != NULL, FALSE);
+  GeglColor *color;
+
   g_return_val_if_fail (GIMP_IS_GIMP (the_gui_gimp), FALSE);
 
-  gimp_context_get_background (gimp_get_user_context (the_gui_gimp), color);
+  color = gimp_context_get_background (gimp_get_user_context (the_gui_gimp));
 
-  return TRUE;
+  return gegl_color_duplicate (color);
 }
 
 static void
@@ -538,6 +549,11 @@ gui_restore_after_callback (Gimp               *gimp,
   GimpGuiConfig *gui_config = GIMP_GUI_CONFIG (gimp->config);
   GimpUIManager *image_ui_manager;
   GimpDisplay   *display;
+#ifdef G_OS_WIN32
+  STARTUPINFO    StartupInfo;
+
+  GetStartupInfo (&StartupInfo);
+#endif
 
   if (gimp->be_verbose)
     g_print ("INIT: %s\n", G_STRFUNC);
@@ -605,11 +621,23 @@ gui_restore_after_callback (Gimp               *gimp,
 
       shell = gimp_display_get_shell (display);
 
+#ifdef G_OS_WIN32
+      themes_set_title_bar (gimp);
+#endif
+
       if (gui_config->restore_session)
         session_restore (gimp, initial_monitor);
 
       toplevel = gtk_widget_get_toplevel (GTK_WIDGET (shell));
 
+#ifdef G_OS_WIN32
+      /* Prevents window from reappearing on start-up if the user
+       * requested it to be minimized via window hints
+       */
+      if (StartupInfo.wShowWindow != SW_SHOWMINIMIZED   &&
+          StartupInfo.wShowWindow != SW_SHOWMINNOACTIVE &&
+          StartupInfo.wShowWindow != SW_MINIMIZE)
+#endif
       /*  move keyboard focus to the display  */
       gtk_window_present (GTK_WINDOW (toplevel));
     }
@@ -729,6 +757,7 @@ gui_exit_after_callback (Gimp     *gimp,
   gimp_devices_exit (gimp);
   dialogs_exit (gimp);
   themes_exit (gimp);
+  gimp_canvas_styles_exit ();
 
   g_type_class_unref (g_type_class_peek (GIMP_TYPE_COLOR_SELECT));
 
@@ -860,10 +889,26 @@ gui_check_unique_accelerators (Gimp *gimp)
 
   for (gint i = 0; actions[i] != NULL; i++)
     {
-      gchar **accels;
-      gchar  *detailed_name;
+      gchar      **accels;
+      gchar       *detailed_name;
+      GimpAction  *action;
+      gint         value;
+
+      action = (GimpAction *) g_action_map_lookup_action (G_ACTION_MAP (gimp->app), actions[i]);
+
+      if (GIMP_IS_RADIO_ACTION (action))
+        {
+          g_object_get ((GObject *) action,
+                        "value", &value,
+                        NULL);
+          detailed_name = g_strdup_printf ("app.%s(%i)", actions[i],
+                                           value);
+        }
+      else
+        {
+          detailed_name = g_strdup_printf ("app.%s", actions[i]);
+        }
 
-      detailed_name = g_strdup_printf ("app.%s", actions[i]);
       accels = gtk_application_get_accels_for_action (GTK_APPLICATION (gimp->app),
                                                       detailed_name);
       g_free (detailed_name);
@@ -872,10 +917,25 @@ gui_check_unique_accelerators (Gimp *gimp)
         {
           for (gint k = i + 1; actions[k] != NULL; k++)
             {
-              gchar **accels2;
-              gchar  *detailed_name2;
+              gchar      **accels2;
+              gchar       *detailed_name2;
+              GimpAction  *action2;
+
+              action2 = (GimpAction *) g_action_map_lookup_action (G_ACTION_MAP (gimp->app), actions[k]);
+
+              if (GIMP_IS_RADIO_ACTION (action2))
+                {
+                  g_object_get ((GObject *) action2,
+                                "value", &value,
+                                NULL);
+                  detailed_name2 = g_strdup_printf ("app.%s(%i)", actions[k],
+                                                    value);
+                }
+              else
+                {
+                  detailed_name2 = g_strdup_printf ("app.%s", actions[k]);
+                }
 
-              detailed_name2 = g_strdup_printf ("app.%s", actions[k]);
               accels2 = gtk_application_get_accels_for_action (GTK_APPLICATION (gimp->app),
                                                                detailed_name2);
               g_free (detailed_name2);
@@ -889,12 +949,26 @@ gui_check_unique_accelerators (Gimp *gimp)
                       gchar   **disabled_accels;
                       gint      len;
                       gint      remove;
+                      gboolean  print_warning = TRUE;
 
+                      action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app),
+                                                           actions[i]);
                       /* Just keep the first one (no reason other than we have
                        * to choose), unless it's a secondary shortcut, and the
                        * second is a primary shortcut.
                        */
-                      if (l == 0 && j != 0)
+                      if ((l == 0 && j != 0) ||
+                          /* If the first action is one of "view-zoom-1-*" and
+                           * the shortcut default, we assume it's because of our
+                           * trick to transform `Shift+num` shortcuts based on
+                           * layout and we happen to be on a layout where it
+                           * clashes with other shortcuts. In this case, we
+                           * drop the duplicate shortcut on the zoom action. See
+                           * special code in
+                           * gimp_action_group_add_action_with_accel()
+                           */
+                          (g_str_has_prefix (actions[i], "view-zoom-1-") &&
+                           gimp_action_is_default_accel (GIMP_ACTION (action), accels[j])))
                         {
                           disabled_action = actions[i];
                           disabled_accels = accels;
@@ -906,6 +980,20 @@ gui_check_unique_accelerators (Gimp *gimp)
                           disabled_accels = accels2;
                           remove = l;
                         }
+
+                      action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app),
+                                                           disabled_action);
+
+                      if (g_str_has_prefix (disabled_action, "view-zoom-1-") &&
+                          gimp_action_is_default_accel (GIMP_ACTION (action), disabled_accels[remove]))
+                        /* We drop the shortcut **silently** because it will be
+                         * a case where we have 2 default accelerators clashing
+                         * (because of the conversion code) while not being a
+                         * real bug. Clashes with custom accelerators are
+                         * handled by shortcuts_action_deserialize().
+                         */
+                        print_warning = FALSE;
+
                       /* Remove only the duplicate shortcut but keep others. */
                       len = g_strv_length (disabled_accels);
                       g_free (disabled_accels[remove]);
@@ -913,12 +1001,11 @@ gui_check_unique_accelerators (Gimp *gimp)
                                &disabled_accels[remove + 1],
                                sizeof (char *) * (len - remove));
 
-                      g_printerr ("Actions \"%s\" and \"%s\" use the same accelerator.\n"
-                                  "  Disabling the accelerator on \"%s\".\n",
-                                  actions[i], actions[k], disabled_action);
+                      if (print_warning)
+                        g_printerr ("Actions \"%s\" and \"%s\" use the same accelerator.\n"
+                                    "  Disabling the accelerator on \"%s\".\n",
+                                    actions[i], actions[k], disabled_action);
 
-                      action = g_action_map_lookup_action (G_ACTION_MAP (gimp->app),
-                                                           disabled_action);
                       gimp_action_set_accels (GIMP_ACTION (action),
                                               (const gchar **) disabled_accels);
                     }
