@@ -25,6 +25,12 @@
 #include <gdk/gdkwayland.h>
 #endif
 
+#ifdef G_OS_WIN32
+#include <windef.h>
+#include <winbase.h>
+#include <windows.h>
+#endif
+
 #include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 #include "libgimpcolor/gimpcolor.h"
@@ -121,6 +127,11 @@ splash_create (Gimp         *gimp,
   GdkRectangle        workarea;
   gint                max_width;
   gint                max_height;
+#ifdef G_OS_WIN32
+  STARTUPINFO         StartupInfo;
+
+  GetStartupInfo (&StartupInfo);
+#endif
 
   g_return_if_fail (splash == NULL);
   g_return_if_fail (GDK_IS_MONITOR (monitor));
@@ -251,6 +262,13 @@ splash_create (Gimp         *gimp,
 
   gtk_widget_show (splash->window);
 
+#ifdef G_OS_WIN32
+  if (StartupInfo.wShowWindow == SW_SHOWMINIMIZED   ||
+      StartupInfo.wShowWindow == SW_SHOWMINNOACTIVE ||
+      StartupInfo.wShowWindow == SW_MINIMIZE)
+    gtk_window_iconify (GTK_WINDOW (splash->window));
+#endif
+
   if (FALSE)
     splash->timer = g_timer_new ();
 }
@@ -595,7 +613,7 @@ splash_image_load_from_file (GFile    *file,
     {
       const gchar *content_type;
 
-      content_type = g_file_info_get_content_type (info);
+      content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
       if (content_type)
         {
           gchar *mime_type;
