@@ -31,6 +31,11 @@
 
 #include "core/gimp.h"
 
+#include "display/gimpimagewindow.h"
+
+#include "widgets/gimpwidgets-utils.h"
+
+#include "icon-themes.h"
 #include "themes.h"
 
 #include "gimp-intl.h"
@@ -88,7 +93,10 @@ themes_init (Gimp *gimp)
   g_signal_connect (config, "notify::theme",
                     G_CALLBACK (themes_theme_change_notify),
                     gimp);
-  g_signal_connect (config, "notify::prefer-dark-theme",
+  g_signal_connect_after (config, "notify::icon-theme",
+                          G_CALLBACK (themes_theme_change_notify),
+                          gimp);
+  g_signal_connect (config, "notify::theme-color-scheme",
                     G_CALLBACK (themes_theme_change_notify),
                     gimp);
   g_signal_connect (config, "notify::prefer-symbolic-icons",
@@ -100,8 +108,15 @@ themes_init (Gimp *gimp)
   g_signal_connect (config, "notify::custom-icon-size",
                     G_CALLBACK (themes_theme_change_notify),
                     gimp);
+  g_signal_connect (config, "notify::font-relative-size",
+                    G_CALLBACK (themes_theme_change_notify),
+                    gimp);
 
   themes_theme_change_notify (config, NULL, gimp);
+
+#ifdef G_OS_WIN32
+  themes_set_title_bar (gimp);
+#endif
 }
 
 void
@@ -226,10 +241,12 @@ themes_apply_theme (Gimp          *gimp,
   GFile         *theme_css;
   GOutputStream *output;
   GError        *error = NULL;
+  gboolean       prefer_dark_theme;
 
   g_return_if_fail (GIMP_IS_GIMP (gimp));
   g_return_if_fail (GIMP_IS_GUI_CONFIG (config));
 
+  prefer_dark_theme = (config->theme_scheme != GIMP_THEME_LIGHT);
   theme_css = gimp_directory_file ("theme.css", NULL);
 
   if (gimp->be_verbose)
@@ -252,11 +269,79 @@ themes_apply_theme (Gimp          *gimp,
 
       if (theme_dir)
         {
-          css_files = g_slist_prepend (css_files, g_file_get_child (theme_dir,
-                                                                    "gimp.css"));
-          if (config->prefer_dark_theme)
-            css_files = g_slist_prepend (css_files, g_file_get_child (theme_dir,
-                                                                      "gimp-dark.css"));
+          GFile *file = NULL;
+          GFile *fallback;
+          GFile *light;
+          GFile *gray;
+          GFile *dark;
+
+          fallback = g_file_get_child (theme_dir, "gimp.css");
+          if (! g_file_query_exists (fallback, NULL))
+            g_clear_object (&fallback);
+
+          light = g_file_get_child (theme_dir, "gimp-light.css");
+          if (! g_file_query_exists (light, NULL))
+            g_clear_object (&light);
+
+          gray  = g_file_get_child (theme_dir, "gimp-gray.css");
+          if (! g_file_query_exists (gray, NULL))
+            g_clear_object (&gray);
+
+          dark  = g_file_get_child (theme_dir, "gimp-dark.css");
+          if (! g_file_query_exists (dark, NULL))
+            g_clear_object (&dark);
+
+          switch (config->theme_scheme)
+            {
+            case GIMP_THEME_LIGHT:
+              if (light != NULL)
+                file = g_object_ref (light);
+              else if (fallback != NULL)
+                file = g_object_ref (fallback);
+              else if (gray != NULL)
+                file = g_object_ref (gray);
+              else if (dark != NULL)
+                file = g_object_ref (dark);
+              break;
+            case GIMP_THEME_GRAY:
+              if (gray != NULL)
+                file = g_object_ref (gray);
+              else if (fallback != NULL)
+                file = g_object_ref (fallback);
+              else if (dark != NULL)
+                file = g_object_ref (dark);
+              else if (light != NULL)
+                file = g_object_ref (light);
+              break;
+            case GIMP_THEME_DARK:
+              if (dark != NULL)
+                file = g_object_ref (dark);
+              else if (fallback != NULL)
+                file = g_object_ref (fallback);
+              else if (gray != NULL)
+                file = g_object_ref (gray);
+              else if (light != NULL)
+                file = g_object_ref (light);
+              break;
+            }
+
+          if (file != NULL)
+            {
+              prefer_dark_theme = (file == dark || file == gray);
+              css_files = g_slist_prepend (css_files, file);
+            }
+          else
+            {
+              gimp_message (gimp, NULL, GIMP_MESSAGE_ERROR,
+                            _("Invalid theme: directory '%s' contains neither "
+                              "gimp-dark.css, gimp-gray.css, gimp-light.css nor gimp.css."),
+                            gimp_file_get_utf8_name (theme_dir));
+            }
+
+          g_clear_object (&fallback);
+          g_clear_object (&light);
+          g_clear_object (&gray);
+          g_clear_object (&dark);
         }
       else
         {
@@ -265,18 +350,30 @@ themes_apply_theme (Gimp          *gimp,
           tmp = g_build_filename (gimp_data_directory (),
                                   "themes", "Default", "gimp.css",
                                   NULL);
-          css_files = g_slist_prepend (
-            css_files, g_file_new_for_path (tmp));
+          css_files = g_slist_prepend (css_files, g_file_new_for_path (tmp));
           g_free (tmp);
 
-          if (config->prefer_dark_theme)
+          switch (config->theme_scheme)
             {
+            case GIMP_THEME_LIGHT:
+              tmp = g_build_filename (gimp_data_directory (),
+                                      "themes", "Default", "gimp-light.css",
+                                      NULL);
+              break;
+            case GIMP_THEME_GRAY:
+              tmp = g_build_filename (gimp_data_directory (),
+                                      "themes", "Default", "gimp-gray.css",
+                                      NULL);
+              break;
+            case GIMP_THEME_DARK:
               tmp = g_build_filename (gimp_data_directory (),
                                       "themes", "Default", "gimp-dark.css",
                                       NULL);
-              css_files = g_slist_prepend (css_files, g_file_new_for_path (tmp));
-              g_free (tmp);
+              break;
             }
+
+          css_files = g_slist_prepend (css_files, g_file_new_for_path (tmp));
+          g_free (tmp);
         }
 
       css_files = g_slist_prepend (
@@ -326,8 +423,9 @@ themes_apply_theme (Gimp          *gimp,
             "* { -gtk-icon-style: %s; }\n"
             "\n"
             "%s",
-            config->prefer_symbolic_icons ? "symbolic" : "regular",
-            config->prefer_dark_theme ? "/* prefer-dark-theme */\n" : "");
+            icon_themes_current_prefer_symbolic (gimp) ? "symbolic" : "regular",
+            prefer_dark_theme ? "/* prefer-dark-theme */\n" : "");
+
         }
 
       if (! error && config->override_icon_size)
@@ -335,6 +433,9 @@ themes_apply_theme (Gimp          *gimp,
           const gchar *tool_icon_size   = "large-toolbar";
           const gchar *tab_icon_size    = "small-toolbar";
           const gchar *button_icon_size = "small-toolbar";
+          gint         pal_padding      = 4;
+          gint         tab_padding      = 1;
+          gint         sep_padding      = 1;
 
           switch (config->custom_icon_size)
             {
@@ -342,21 +443,33 @@ themes_apply_theme (Gimp          *gimp,
               tool_icon_size   = "small-toolbar";
               tab_icon_size    = "small-toolbar";
               button_icon_size = "small-toolbar";
+              pal_padding      = 1;
+              tab_padding      = 0;
+              sep_padding      = 1;
               break;
             case GIMP_ICON_SIZE_MEDIUM:
               tool_icon_size   = "large-toolbar";
               tab_icon_size    = "small-toolbar";
               button_icon_size = "small-toolbar";
+              pal_padding      = 4;
+              tab_padding      = 1;
+              sep_padding      = 1;
               break;
             case GIMP_ICON_SIZE_LARGE:
               tool_icon_size   = "dnd";
               tab_icon_size    = "large-toolbar";
               button_icon_size = "large-toolbar";
+              pal_padding      = 5;
+              tab_padding      = 5;
+              sep_padding      = 2;
               break;
             case GIMP_ICON_SIZE_HUGE:
               tool_icon_size   = "dialog";
               tab_icon_size    = "dnd";
               button_icon_size = "dnd";
+              pal_padding      = 5;
+              tab_padding      = 8;
+              sep_padding      = 4;
               break;
             }
 
@@ -367,12 +480,23 @@ themes_apply_theme (Gimp          *gimp,
             "\n"
             "* { -GimpDockbook-tab-icon-size: %s; }"
             "\n"
-            "* { -GimpEditor-button-icon-size: %s; }",
-            tool_icon_size,
-            tab_icon_size,
-            button_icon_size);
+            "* { -GimpEditor-button-icon-size: %s; }"
+            "\n"
+            "toolpalette button { padding: %dpx; }"
+            "\n"
+            "button, tab { padding: %dpx; }"
+            "\n"
+            "paned separator { padding: %dpx; }",
+            tool_icon_size, tab_icon_size, button_icon_size,
+            pal_padding, tab_padding, sep_padding);
         }
 
+      if (! error && config->font_relative_size != 1.0)
+        g_output_stream_printf (output, NULL, NULL, &error,
+                                "\n"
+                                "* { font-size: %frem; }",
+                                config->font_relative_size);
+
       if (! error)
         {
           g_output_stream_printf (
@@ -438,7 +562,7 @@ themes_theme_change_notify (GimpGuiConfig *config,
   GError *error = NULL;
 
   g_object_set (gtk_settings_get_for_screen (gdk_screen_get_default ()),
-                "gtk-application-prefer-dark-theme", config->prefer_dark_theme,
+                "gtk-application-prefer-dark-theme", config->theme_scheme != GIMP_THEME_LIGHT,
                 NULL);
 
   themes_apply_theme (gimp, config);
@@ -460,6 +584,10 @@ themes_theme_change_notify (GimpGuiConfig *config,
   g_object_unref (theme_css);
 
   gtk_style_context_reset_widgets (gdk_screen_get_default ());
+
+#ifdef G_OS_WIN32
+  themes_set_title_bar (gimp);
+#endif
 }
 
 static void
@@ -511,8 +639,8 @@ themes_theme_paths_notify (GimpExtensionManager *manager,
               while ((info = g_file_enumerator_next_file (enumerator,
                                                           NULL, NULL)))
                 {
-                  if (! g_file_info_get_is_hidden (info) &&
-                      g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+                  if (! g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN) &&
+                      g_file_info_get_attribute_uint32 (info, G_FILE_ATTRIBUTE_STANDARD_TYPE) == G_FILE_TYPE_DIRECTORY)
                     {
                       GFile       *file;
                       const gchar *name;
@@ -540,3 +668,22 @@ themes_theme_paths_notify (GimpExtensionManager *manager,
       g_list_free_full (path, (GDestroyNotify) g_object_unref);
     }
 }
+
+void
+themes_set_title_bar (Gimp *gimp)
+{
+#ifdef G_OS_WIN32
+  GList *windows = gimp_get_image_windows (gimp);
+  GList *iter;
+
+  for (iter = windows; iter; iter = g_list_next (iter))
+    {
+      GtkWidget *window = GTK_WIDGET (windows->data);
+
+      gimp_window_set_title_bar_theme (gimp, window, TRUE);
+    }
+
+  if (windows)
+    g_list_free (windows);
+#endif
+}
