@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -27,7 +27,6 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -37,15 +36,12 @@
 #include <sys/wait.h>
 #endif
 
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 
-#include <glib-object.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gegl.h>
 
 #if defined(G_OS_WIN32) || defined(G_WITH_CYGWIN)
 
@@ -79,6 +75,7 @@
 #include "plug-in-types.h"
 
 #include "core/gimp.h"
+#include "core/gimp-spawn.h"
 #include "core/gimpprogress.h"
 
 #include "pdb/gimppdbcontext.h"
@@ -92,15 +89,16 @@
 #include "gimpplugindef.h"
 #include "gimppluginmanager.h"
 #include "gimppluginmanager-help-domain.h"
-#include "gimppluginmanager-locale-domain.h"
 #include "gimptemporaryprocedure.h"
-#include "plug-in-params.h"
 
 #include "gimp-intl.h"
 
 
 static void       gimp_plug_in_finalize      (GObject      *object);
 
+static gboolean   gimp_plug_in_recv_message  (GIOChannel   *channel,
+                                              GIOCondition  cond,
+                                              gpointer      data);
 static gboolean   gimp_plug_in_write         (GIOChannel   *channel,
                                               const guint8 *buf,
                                               gulong        count,
@@ -108,14 +106,14 @@ static gboolean   gimp_plug_in_write         (GIOChannel   *channel,
 static gboolean   gimp_plug_in_flush         (GIOChannel   *channel,
                                               gpointer      data);
 
-static gboolean   gimp_plug_in_recv_message  (GIOChannel   *channel,
-                                              GIOCondition  cond,
-                                              gpointer      data);
+#if defined G_OS_WIN32 && defined WIN32_32BIT_DLL_FOLDER
+static void       gimp_plug_in_set_dll_directory (const gchar *path);
+#endif
 
-#if !defined(G_OS_WIN32) && !defined (G_WITH_CYGWIN)
-static void       gimp_plug_in_prep_for_exec (gpointer      data);
-#else
-#define           gimp_plug_in_prep_for_exec  NULL
+#ifndef G_OS_WIN32
+static void       gimp_plug_in_close_waitpid     (GPid         pid,
+                                                  gint         status,
+                                                  GimpPlugIn  *plug_in);
 #endif
 
 
@@ -143,7 +141,7 @@ static void
 gimp_plug_in_init (GimpPlugIn *plug_in)
 {
   plug_in->manager            = NULL;
-  plug_in->prog               = NULL;
+  plug_in->file               = NULL;
 
   plug_in->call_mode          = GIMP_PLUG_IN_CALL_NONE;
   plug_in->open               = FALSE;
@@ -172,419 +170,13 @@ gimp_plug_in_finalize (GObject *object)
 {
   GimpPlugIn *plug_in = GIMP_PLUG_IN (object);
 
-  g_free (plug_in->prog);
+  g_clear_object (&plug_in->file);
 
   gimp_plug_in_proc_frame_dispose (&plug_in->main_proc_frame, plug_in);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-
-/*  public functions  */
-
-GimpPlugIn *
-gimp_plug_in_new (GimpPlugInManager   *manager,
-                  GimpContext         *context,
-                  GimpProgress        *progress,
-                  GimpPlugInProcedure *procedure,
-                  const gchar         *prog)
-{
-  GimpPlugIn *plug_in;
-
-  g_return_val_if_fail (GIMP_IS_PLUG_IN_MANAGER (manager), NULL);
-  g_return_val_if_fail (GIMP_IS_PDB_CONTEXT (context), NULL);
-  g_return_val_if_fail (progress == NULL || GIMP_IS_PROGRESS (progress), NULL);
-  g_return_val_if_fail (procedure == NULL ||
-                        GIMP_IS_PLUG_IN_PROCEDURE (procedure), NULL);
-  g_return_val_if_fail (prog == NULL || g_path_is_absolute (prog), NULL);
-  g_return_val_if_fail ((procedure != NULL || prog != NULL) &&
-                        ! (procedure != NULL && prog != NULL), NULL);
-
-  plug_in = g_object_new (GIMP_TYPE_PLUG_IN, NULL);
-
-  if (! prog)
-    prog = gimp_plug_in_procedure_get_progname (procedure);
-
-  gimp_object_take_name (GIMP_OBJECT (plug_in),
-                         g_filename_display_basename (prog));
-
-  plug_in->manager = manager;
-  plug_in->prog    = g_strdup (prog);
-
-  gimp_plug_in_proc_frame_init (&plug_in->main_proc_frame,
-                                context, progress, procedure);
-
-  return plug_in;
-}
-
-gboolean
-gimp_plug_in_open (GimpPlugIn         *plug_in,
-                   GimpPlugInCallMode  call_mode,
-                   gboolean            synchronous)
-{
-  gint          my_read[2];
-  gint          my_write[2];
-  gchar       **envp;
-  const gchar  *args[9];
-  gchar       **argv;
-  gint          argc;
-  gchar        *interp, *interp_arg;
-  gchar        *read_fd, *write_fd;
-  const gchar  *mode;
-  gchar        *stm;
-  GError       *error = NULL;
-  gboolean      debug;
-  guint         debug_flag;
-  guint         spawn_flags;
-
-  g_return_val_if_fail (GIMP_IS_PLUG_IN (plug_in), FALSE);
-  g_return_val_if_fail (plug_in->call_mode == GIMP_PLUG_IN_CALL_NONE, FALSE);
-
-  /* Open two pipes. (Bidirectional communication).
-   */
-  if ((pipe (my_read) == -1) || (pipe (my_write) == -1))
-    {
-      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Unable to run plug-in \"%s\"\n(%s)\n\npipe() failed: %s",
-                    gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    g_strerror (errno));
-      return FALSE;
-    }
-
-#if defined(G_WITH_CYGWIN)
-  /* Set to binary mode */
-  setmode (my_read[0], _O_BINARY);
-  setmode (my_write[0], _O_BINARY);
-  setmode (my_read[1], _O_BINARY);
-  setmode (my_write[1], _O_BINARY);
-#endif
-
-#ifdef G_OS_WIN32
-  /* Prevent the plug-in from inheriting our ends of the pipes */
-  SetHandleInformation ((HANDLE) _get_osfhandle (my_read[0]), HANDLE_FLAG_INHERIT, 0);
-  SetHandleInformation ((HANDLE) _get_osfhandle (my_write[1]), HANDLE_FLAG_INHERIT, 0);
-#endif
-
-  plug_in->my_read   = g_io_channel_unix_new (my_read[0]);
-  plug_in->my_write  = g_io_channel_unix_new (my_write[1]);
-  plug_in->his_read  = g_io_channel_unix_new (my_write[0]);
-  plug_in->his_write = g_io_channel_unix_new (my_read[1]);
-
-  g_io_channel_set_encoding (plug_in->my_read, NULL, NULL);
-  g_io_channel_set_encoding (plug_in->my_write, NULL, NULL);
-  g_io_channel_set_encoding (plug_in->his_read, NULL, NULL);
-  g_io_channel_set_encoding (plug_in->his_write, NULL, NULL);
-
-  g_io_channel_set_buffered (plug_in->my_read, FALSE);
-  g_io_channel_set_buffered (plug_in->my_write, FALSE);
-  g_io_channel_set_buffered (plug_in->his_read, FALSE);
-  g_io_channel_set_buffered (plug_in->his_write, FALSE);
-
-  g_io_channel_set_close_on_unref (plug_in->my_read, TRUE);
-  g_io_channel_set_close_on_unref (plug_in->my_write, TRUE);
-  g_io_channel_set_close_on_unref (plug_in->his_read, TRUE);
-  g_io_channel_set_close_on_unref (plug_in->his_write, TRUE);
-
-  /* Remember the file descriptors for the pipes.
-   */
-  read_fd  = g_strdup_printf ("%d",
-                              g_io_channel_unix_get_fd (plug_in->his_read));
-  write_fd = g_strdup_printf ("%d",
-                              g_io_channel_unix_get_fd (plug_in->his_write));
-
-  switch (call_mode)
-    {
-    case GIMP_PLUG_IN_CALL_QUERY:
-      mode = "-query";
-      debug_flag = GIMP_DEBUG_WRAP_QUERY;
-      break;
-
-    case GIMP_PLUG_IN_CALL_INIT:
-      mode = "-init";
-      debug_flag = GIMP_DEBUG_WRAP_INIT;
-      break;
-
-    case GIMP_PLUG_IN_CALL_RUN:
-      mode = "-run";
-      debug_flag = GIMP_DEBUG_WRAP_RUN;
-      break;
-
-    default:
-      g_assert_not_reached ();
-    }
-
-  stm = g_strdup_printf ("%d", plug_in->manager->gimp->stack_trace_mode);
-
-  interp = gimp_interpreter_db_resolve (plug_in->manager->interpreter_db,
-                                        plug_in->prog, &interp_arg);
-
-  argc = 0;
-
-  if (interp)
-    args[argc++] = interp;
-
-  if (interp_arg)
-    args[argc++] = interp_arg;
-
-  args[argc++] = plug_in->prog;
-  args[argc++] = "-gimp";
-  args[argc++] = read_fd;
-  args[argc++] = write_fd;
-  args[argc++] = mode;
-  args[argc++] = stm;
-  args[argc++] = NULL;
-
-  argv = (gchar **) args;
-  envp = gimp_environ_table_get_envp (plug_in->manager->environ_table);
-  spawn_flags = (G_SPAWN_LEAVE_DESCRIPTORS_OPEN |
-                 G_SPAWN_DO_NOT_REAP_CHILD      |
-                 G_SPAWN_CHILD_INHERITS_STDIN);
-
-  debug = FALSE;
-
-  if (plug_in->manager->debug)
-    {
-      gchar **debug_argv = gimp_plug_in_debug_argv (plug_in->manager->debug,
-                                                    plug_in->prog,
-                                                    debug_flag, args);
-
-      if (debug_argv)
-        {
-          debug = TRUE;
-          argv = debug_argv;
-          spawn_flags |= G_SPAWN_SEARCH_PATH;
-        }
-    }
-
-  /* Fork another process. We'll remember the process id so that we
-   * can later use it to kill the filter if necessary.
-   */
-  if (! g_spawn_async (NULL, argv, envp, spawn_flags,
-                       gimp_plug_in_prep_for_exec, plug_in,
-                       &plug_in->pid,
-                       &error))
-    {
-      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Unable to run plug-in \"%s\"\n(%s)\n\n%s",
-                    gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    error->message);
-      g_error_free (error);
-      goto cleanup;
-    }
-
-  g_io_channel_unref (plug_in->his_read);
-  plug_in->his_read = NULL;
-
-  g_io_channel_unref (plug_in->his_write);
-  plug_in->his_write = NULL;
-
-  if (! synchronous)
-    {
-      GSource *source;
-
-      source = g_io_create_watch (plug_in->my_read,
-                                  G_IO_IN  | G_IO_PRI | G_IO_ERR | G_IO_HUP);
-
-      g_source_set_callback (source,
-                             (GSourceFunc) gimp_plug_in_recv_message, plug_in,
-                             NULL);
-
-      g_source_set_can_recurse (source, TRUE);
-
-      plug_in->input_id = g_source_attach (source, NULL);
-      g_source_unref (source);
-    }
-
-  plug_in->open      = TRUE;
-  plug_in->call_mode = call_mode;
-
-  gimp_plug_in_manager_add_open_plug_in (plug_in->manager, plug_in);
-
- cleanup:
-
-  if (debug)
-    g_free (argv);
-
-  g_free (read_fd);
-  g_free (write_fd);
-  g_free (stm);
-  g_free (interp);
-  g_free (interp_arg);
-
-  return plug_in->open;
-}
-
-void
-gimp_plug_in_close (GimpPlugIn *plug_in,
-                    gboolean    kill_it)
-{
-  g_return_if_fail (GIMP_IS_PLUG_IN (plug_in));
-  g_return_if_fail (plug_in->open);
-
-  plug_in->open = FALSE;
-
-  if (plug_in->pid)
-    {
-#ifndef G_OS_WIN32
-      gint status;
-#endif
-
-      /*  Ask the filter to exit gracefully,
-          but not if it is closed because of a broken pipe.  */
-      if (kill_it && ! plug_in->hup)
-        {
-          gp_quit_write (plug_in->my_write, plug_in);
-
-          /*  give the plug-in some time (10 ms)  */
-          g_usleep (10000);
-        }
-
-      /* If necessary, kill the filter. */
-
-#ifndef G_OS_WIN32
-
-      if (kill_it)
-        {
-          if (plug_in->manager->gimp->be_verbose)
-            g_print ("Terminating plug-in: '%s'\n",
-                     gimp_filename_to_utf8 (plug_in->prog));
-
-          /*  If the plug-in opened a process group, kill the group instead
-           *  of only the plug-in, so we kill the plug-in's children too
-           */
-          if (getpgid (0) != getpgid (plug_in->pid))
-            status = kill (- plug_in->pid, SIGKILL);
-          else
-            status = kill (plug_in->pid, SIGKILL);
-        }
-
-      /* Wait for the process to exit. This will happen
-       * immediately if it was just killed.
-       */
-      waitpid (plug_in->pid, &status, 0);
-
-#else /* G_OS_WIN32 */
-
-      if (kill_it)
-        {
-          /* Trying to avoid TerminateProcess (does mostly work).
-           * Otherwise some of our needed DLLs may get into an
-           * unstable state (see Win32 API docs).
-           */
-          DWORD dwExitCode = STILL_ACTIVE;
-          DWORD dwTries    = 10;
-
-          while (dwExitCode == dwExitCode &&
-                 GetExitCodeProcess ((HANDLE) plug_in->pid, &dwExitCode) &&
-                 (dwTries > 0))
-            {
-              Sleep (10);
-              dwTries--;
-            }
-
-          if (dwExitCode == STILL_ACTIVE)
-            {
-              if (plug_in->manager->gimp->be_verbose)
-                g_print ("Terminating plug-in: '%s'\n",
-                         gimp_filename_to_utf8 (plug_in->prog));
-
-              TerminateProcess ((HANDLE) plug_in->pid, 0);
-            }
-        }
-
-#endif /* G_OS_WIN32 */
-
-      g_spawn_close_pid (plug_in->pid);
-      plug_in->pid = 0;
-    }
-
-  /* Remove the input handler. */
-  if (plug_in->input_id)
-    {
-      g_source_remove (plug_in->input_id);
-      plug_in->input_id = 0;
-    }
-
-  /* Close the pipes. */
-  if (plug_in->my_read != NULL)
-    {
-      g_io_channel_unref (plug_in->my_read);
-      plug_in->my_read = NULL;
-    }
-  if (plug_in->my_write != NULL)
-    {
-      g_io_channel_unref (plug_in->my_write);
-      plug_in->my_write = NULL;
-    }
-  if (plug_in->his_read != NULL)
-    {
-      g_io_channel_unref (plug_in->his_read);
-      plug_in->his_read = NULL;
-    }
-  if (plug_in->his_write != NULL)
-    {
-      g_io_channel_unref (plug_in->his_write);
-      plug_in->his_write = NULL;
-    }
-
-  gimp_wire_clear_error ();
-
-  while (plug_in->temp_proc_frames)
-    {
-      GimpPlugInProcFrame *proc_frame = plug_in->temp_proc_frames->data;
-
-#ifdef GIMP_UNSTABLE
-      g_printerr ("plug-in '%s' aborted before sending its "
-                  "temporary procedure return values\n",
-                  gimp_object_get_name (plug_in));
-#endif
-
-      if (proc_frame->main_loop &&
-          g_main_loop_is_running (proc_frame->main_loop))
-        {
-          g_main_loop_quit (proc_frame->main_loop);
-        }
-
-      /* pop the frame here, because normally this only happens in
-       * gimp_plug_in_handle_temp_proc_return(), which can't
-       * be called after plug_in_close()
-       */
-      gimp_plug_in_proc_frame_pop (plug_in);
-    }
-
-  if (plug_in->main_proc_frame.main_loop &&
-      g_main_loop_is_running (plug_in->main_proc_frame.main_loop))
-    {
-#ifdef GIMP_UNSTABLE
-      g_printerr ("plug-in '%s' aborted before sending its "
-                  "procedure return values\n",
-                  gimp_object_get_name (plug_in));
-#endif
-
-      g_main_loop_quit (plug_in->main_proc_frame.main_loop);
-    }
-
-  if (plug_in->ext_main_loop &&
-      g_main_loop_is_running (plug_in->ext_main_loop))
-    {
-#ifdef GIMP_UNSTABLE
-      g_printerr ("extension '%s' aborted before sending its "
-                  "extension_ack message\n",
-                  gimp_object_get_name (plug_in));
-#endif
-
-      g_main_loop_quit (plug_in->ext_main_loop);
-    }
-
-  /* Unregister any temporary procedures. */
-  while (plug_in->temp_procedures)
-    gimp_plug_in_remove_temp_proc (plug_in, plug_in->temp_procedures->data);
-
-  gimp_plug_in_manager_remove_open_plug_in (plug_in->manager, plug_in);
-}
-
 static gboolean
 gimp_plug_in_recv_message (GIOChannel   *channel,
                            GIOCondition  cond,
@@ -645,7 +237,7 @@ gimp_plug_in_recv_message (GIOChannel   *channel,
                       "state. You may want to save your images and restart "
                       "GIMP to be on the safe side."),
                     gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog));
+                    gimp_file_get_utf8_name (plug_in->file));
     }
 
   g_object_unref (plug_in);
@@ -742,22 +334,486 @@ gimp_plug_in_flush (GIOChannel *channel,
   return TRUE;
 }
 
-#if !defined(G_OS_WIN32) && !defined (G_WITH_CYGWIN)
-
+#if defined G_OS_WIN32 && defined WIN32_32BIT_DLL_FOLDER
 static void
-gimp_plug_in_prep_for_exec (gpointer data)
+gimp_plug_in_set_dll_directory (const gchar *path)
 {
-  GimpPlugIn *plug_in = data;
+  LPWSTR       w_path;
+  const gchar *install_dir;
+  gchar       *bin_dir;
+  LPWSTR       w_bin_dir;
+  DWORD        BinaryType;
 
-  g_io_channel_unref (plug_in->my_read);
-  plug_in->my_read  = NULL;
+  w_path = NULL;
+  w_bin_dir = NULL;
+  install_dir = gimp_installation_directory ();
+  if (path &&
+      (w_path = g_utf8_to_utf16 (path, -1, NULL, NULL, NULL)) &&
+      GetBinaryTypeW (w_path, &BinaryType) &&
+      BinaryType == SCS_32BIT_BINARY)
+    bin_dir = g_build_filename (install_dir, WIN32_32BIT_DLL_FOLDER, NULL);
+  else
+    bin_dir = g_build_filename (install_dir, "bin", NULL);
 
-  g_io_channel_unref (plug_in->my_write);
-  plug_in->my_write  = NULL;
+  w_bin_dir = g_utf8_to_utf16 (bin_dir, -1, NULL, NULL, NULL);
+  if (w_bin_dir)
+    {
+      SetDllDirectoryW (w_bin_dir);
+      g_free (w_bin_dir);
+    }
+
+  g_free (bin_dir);
+  g_free (w_path);
+}
+#endif
+
+#ifndef G_OS_WIN32
+static void
+gimp_plug_in_close_waitpid (GPid        pid,
+                            gint        status,
+                            GimpPlugIn *plug_in)
+{
+  GError *error = NULL;
+
+  if (plug_in->manager->gimp->be_verbose &&
+      ! g_spawn_check_wait_status (status, &error))
+    {
+      g_printerr ("Process for plug-in '%s' terminated with error: %s\n",
+                  gimp_object_get_name (plug_in),
+                  error->message);
+    }
+  g_clear_error (&error);
+
+  g_spawn_close_pid (pid);
+}
+#endif
+
+
+/*  public functions  */
+
+GimpPlugIn *
+gimp_plug_in_new (GimpPlugInManager   *manager,
+                  GimpContext         *context,
+                  GimpProgress        *progress,
+                  GimpPlugInProcedure *procedure,
+                  GFile               *file)
+{
+  GimpPlugIn *plug_in;
+
+  g_return_val_if_fail (GIMP_IS_PLUG_IN_MANAGER (manager), NULL);
+  g_return_val_if_fail (GIMP_IS_PDB_CONTEXT (context), NULL);
+  g_return_val_if_fail (progress == NULL || GIMP_IS_PROGRESS (progress), NULL);
+  g_return_val_if_fail (procedure == NULL ||
+                        GIMP_IS_PLUG_IN_PROCEDURE (procedure), NULL);
+  g_return_val_if_fail (file == NULL || G_IS_FILE (file), NULL);
+  g_return_val_if_fail ((procedure != NULL || file != NULL) &&
+                        ! (procedure != NULL && file != NULL), NULL);
+
+  plug_in = g_object_new (GIMP_TYPE_PLUG_IN, NULL);
+
+  if (! file)
+    file = gimp_plug_in_procedure_get_file (procedure);
+
+  gimp_object_take_name (GIMP_OBJECT (plug_in),
+                         g_path_get_basename (gimp_file_get_utf8_name (file)));
+
+  plug_in->manager = manager;
+  plug_in->file    = g_object_ref (file);
+
+  gimp_plug_in_proc_frame_init (&plug_in->main_proc_frame,
+                                context, progress, procedure);
+
+  return plug_in;
 }
 
+gboolean
+gimp_plug_in_open (GimpPlugIn         *plug_in,
+                   GimpPlugInCallMode  call_mode,
+                   gboolean            synchronous)
+{
+  gchar        *progname;
+  gint          my_read[2];
+  gint          my_write[2];
+  gchar       **envp;
+  const gchar  *args[10];
+  gchar       **argv;
+  gint          argc;
+  gchar         protocol_version[8];
+  gchar        *interp, *interp_arg;
+  gchar        *his_read_fd, *his_write_fd;
+  const gchar  *mode;
+  gchar        *stm;
+  GError       *error = NULL;
+  gboolean      debug;
+  guint         debug_flag;
+  guint         spawn_flags;
+
+  g_return_val_if_fail (GIMP_IS_PLUG_IN (plug_in), FALSE);
+  g_return_val_if_fail (plug_in->call_mode == GIMP_PLUG_IN_CALL_NONE, FALSE);
+
+  /* Open two pipes. (Bidirectional communication).
+   */
+  if ((pipe (my_read) == -1) || (pipe (my_write) == -1))
+    {
+      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                    "Unable to run plug-in \"%s\"\n(%s)\n\npipe() failed: %s",
+                    gimp_object_get_name (plug_in),
+                    gimp_file_get_utf8_name (plug_in->file),
+                    g_strerror (errno));
+      return FALSE;
+    }
+
+#if defined(G_WITH_CYGWIN)
+  /* Set to binary mode */
+  setmode (my_read[0], _O_BINARY);
+  setmode (my_write[0], _O_BINARY);
+  setmode (my_read[1], _O_BINARY);
+  setmode (my_write[1], _O_BINARY);
 #endif
 
+  /* Prevent the plug-in from inheriting our end of the pipes */
+  gimp_spawn_set_cloexec (my_read[0]);
+  gimp_spawn_set_cloexec (my_write[1]);
+
+#ifdef G_OS_WIN32
+  plug_in->my_read   = g_io_channel_win32_new_fd (my_read[0]);
+  plug_in->my_write  = g_io_channel_win32_new_fd (my_write[1]);
+  plug_in->his_read  = g_io_channel_win32_new_fd (my_write[0]);
+  plug_in->his_write = g_io_channel_win32_new_fd (my_read[1]);
+#else
+  plug_in->my_read   = g_io_channel_unix_new (my_read[0]);
+  plug_in->my_write  = g_io_channel_unix_new (my_write[1]);
+  plug_in->his_read  = g_io_channel_unix_new (my_write[0]);
+  plug_in->his_write = g_io_channel_unix_new (my_read[1]);
+#endif
+
+  g_io_channel_set_encoding (plug_in->my_read, NULL, NULL);
+  g_io_channel_set_encoding (plug_in->my_write, NULL, NULL);
+  g_io_channel_set_encoding (plug_in->his_read, NULL, NULL);
+  g_io_channel_set_encoding (plug_in->his_write, NULL, NULL);
+
+  g_io_channel_set_buffered (plug_in->my_read, FALSE);
+  g_io_channel_set_buffered (plug_in->my_write, FALSE);
+  g_io_channel_set_buffered (plug_in->his_read, FALSE);
+  g_io_channel_set_buffered (plug_in->his_write, FALSE);
+
+  g_io_channel_set_close_on_unref (plug_in->my_read, TRUE);
+  g_io_channel_set_close_on_unref (plug_in->my_write, TRUE);
+  g_io_channel_set_close_on_unref (plug_in->his_read, TRUE);
+  g_io_channel_set_close_on_unref (plug_in->his_write, TRUE);
+
+  /* Remember the file descriptors for the pipes.
+   */
+  his_read_fd  = g_strdup_printf ("%d",
+                                  g_io_channel_unix_get_fd (plug_in->his_read));
+  his_write_fd = g_strdup_printf ("%d",
+                                  g_io_channel_unix_get_fd (plug_in->his_write));
+
+  switch (call_mode)
+    {
+    case GIMP_PLUG_IN_CALL_QUERY:
+      mode = "-query";
+      debug_flag = GIMP_DEBUG_WRAP_QUERY;
+      break;
+
+    case GIMP_PLUG_IN_CALL_INIT:
+      mode = "-init";
+      debug_flag = GIMP_DEBUG_WRAP_INIT;
+      break;
+
+    case GIMP_PLUG_IN_CALL_RUN:
+      mode = "-run";
+      debug_flag = GIMP_DEBUG_WRAP_RUN;
+      break;
+
+    default:
+      gimp_assert_not_reached ();
+    }
+
+  stm = g_strdup_printf ("%d", plug_in->manager->gimp->stack_trace_mode);
+
+  progname = g_file_get_path (plug_in->file);
+
+  interp = gimp_interpreter_db_resolve (plug_in->manager->interpreter_db,
+                                        progname, &interp_arg);
+
+  argc = 0;
+
+  if (interp)
+    args[argc++] = interp;
+
+  if (interp_arg)
+    args[argc++] = interp_arg;
+
+  g_snprintf (protocol_version, sizeof (protocol_version),
+              "%d", GIMP_PROTOCOL_VERSION);
+
+  args[argc++] = progname;
+  args[argc++] = "-gimp";
+  args[argc++] = protocol_version;
+  args[argc++] = his_read_fd;
+  args[argc++] = his_write_fd;
+  args[argc++] = mode;
+  args[argc++] = stm;
+  args[argc++] = NULL;
+
+  argv = (gchar **) args;
+  envp = gimp_environ_table_get_envp (plug_in->manager->environ_table);
+  spawn_flags = (G_SPAWN_LEAVE_DESCRIPTORS_OPEN |
+                 G_SPAWN_DO_NOT_REAP_CHILD      |
+                 G_SPAWN_CHILD_INHERITS_STDIN);
+
+  debug = FALSE;
+
+  if (plug_in->manager->debug)
+    {
+      gchar **debug_argv = gimp_plug_in_debug_argv (plug_in->manager->debug,
+                                                    progname,
+                                                    debug_flag, args);
+
+      if (debug_argv)
+        {
+          debug = TRUE;
+          argv = debug_argv;
+          spawn_flags |= G_SPAWN_SEARCH_PATH;
+        }
+    }
+
+  /* Fork another process. We'll remember the process id so that we
+   * can later use it to kill the filter if necessary.
+   */
+#if defined G_OS_WIN32 && defined WIN32_32BIT_DLL_FOLDER
+  gimp_plug_in_set_dll_directory (argv[0]);
+#endif
+  if (! gimp_spawn_async (argv, envp, spawn_flags, &plug_in->pid, &error))
+    {
+      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
+                    "Unable to run plug-in \"%s\"\n(%s)\n\n%s",
+                    gimp_object_get_name (plug_in),
+                    gimp_file_get_utf8_name (plug_in->file),
+                    error->message);
+      g_clear_error (&error);
+      goto cleanup;
+    }
+
+  g_clear_pointer (&plug_in->his_read,  g_io_channel_unref);
+  g_clear_pointer (&plug_in->his_write, g_io_channel_unref);
+
+  if (! synchronous)
+    {
+      GSource *source;
+
+      source = g_io_create_watch (plug_in->my_read,
+                                  G_IO_IN  | G_IO_PRI | G_IO_ERR | G_IO_HUP);
+
+      g_source_set_callback (source,
+                             (GSourceFunc) gimp_plug_in_recv_message, plug_in,
+                             NULL);
+
+      g_source_set_can_recurse (source, TRUE);
+
+      plug_in->input_id = g_source_attach (source, NULL);
+      g_source_unref (source);
+    }
+
+  plug_in->open      = TRUE;
+  plug_in->call_mode = call_mode;
+
+  gimp_plug_in_manager_add_open_plug_in (plug_in->manager, plug_in);
+
+ cleanup:
+
+#if defined G_OS_WIN32 && defined WIN32_32BIT_DLL_FOLDER
+  gimp_plug_in_set_dll_directory (NULL);
+#endif
+
+  if (debug)
+    g_free (argv);
+
+  g_free (his_read_fd);
+  g_free (his_write_fd);
+  g_free (stm);
+  g_free (interp);
+  g_free (interp_arg);
+  g_free (progname);
+
+  return plug_in->open;
+}
+
+void
+gimp_plug_in_close (GimpPlugIn *plug_in,
+                    gboolean    kill_it)
+{
+  g_return_if_fail (GIMP_IS_PLUG_IN (plug_in));
+  g_return_if_fail (plug_in->open);
+
+  plug_in->open = FALSE;
+
+  if (plug_in->pid)
+    {
+#ifndef G_OS_WIN32
+      gint status;
+#endif
+
+      /*  Ask the filter to exit gracefully,
+          but not if it is closed because of a broken pipe.  */
+      if (kill_it && ! plug_in->hup)
+        {
+          gp_quit_write (plug_in->my_write, plug_in);
+
+          /*  give the plug-in some time (10 ms)  */
+          g_usleep (10000);
+        }
+
+      /* If necessary, kill the filter. */
+
+#ifndef G_OS_WIN32
+
+      if (kill_it)
+        {
+          if (plug_in->manager->gimp->be_verbose)
+            g_print ("Terminating plug-in: '%s'\n",
+                     gimp_file_get_utf8_name (plug_in->file));
+
+          /*  If the plug-in opened a process group, kill the group instead
+           *  of only the plug-in, so we kill the plug-in's children too
+           */
+          if (getpgid (0) != getpgid (plug_in->pid))
+            status = kill (- plug_in->pid, SIGKILL);
+          else
+            status = kill (plug_in->pid, SIGKILL);
+
+          /* Wait for the process to exit. This will happen
+           * immediately as it was just killed.
+           */
+          waitpid (plug_in->pid, &status, 0);
+        }
+      else
+        {
+          /*
+           * If we don't kill it, it means the plug-in ended normally
+           * hence its process should just return without problem. And
+           * this is the case nearly all the time. Yet I had this one
+           * case where the process would never return (even though the
+           * plug-in returned with a result and no errors) and a
+           * waitpid() would block the main process forever.
+           * Thus use instead a child watch source. My idea was that in
+           * the rare case when the child process doesn't return, it is
+           * better to leave a zombie than freeze the core. As it turns
+           * out, doing this even made the broken process exit (does
+           * g_child_watch_add_full() do something better than
+           * waitpid()?).
+           */
+          g_object_ref (plug_in);
+          g_child_watch_add_full (G_PRIORITY_LOW, plug_in->pid,
+                                  (GChildWatchFunc) gimp_plug_in_close_waitpid,
+                                  plug_in, (GDestroyNotify) g_object_unref);
+        }
+
+#else /* G_OS_WIN32 */
+
+      if (kill_it)
+        {
+          /* Trying to avoid TerminateProcess (does mostly work).
+           * Otherwise some of our needed DLLs may get into an
+           * unstable state (see Win32 API docs).
+           */
+          DWORD dwExitCode = STILL_ACTIVE;
+          DWORD dwTries    = 10;
+
+          while (dwExitCode == STILL_ACTIVE &&
+                 GetExitCodeProcess ((HANDLE) plug_in->pid, &dwExitCode) &&
+                 (dwTries > 0))
+            {
+              Sleep (10);
+              dwTries--;
+            }
+
+          if (dwExitCode == STILL_ACTIVE)
+            {
+              if (plug_in->manager->gimp->be_verbose)
+                g_print ("Terminating plug-in: '%s'\n",
+                         gimp_file_get_utf8_name (plug_in->file));
+
+              TerminateProcess ((HANDLE) plug_in->pid, 0);
+            }
+        }
+
+      g_spawn_close_pid (plug_in->pid);
+#endif /* G_OS_WIN32 */
+
+      plug_in->pid = 0;
+    }
+
+  /* Remove the input handler. */
+  if (plug_in->input_id)
+    {
+      g_source_remove (plug_in->input_id);
+      plug_in->input_id = 0;
+    }
+
+  /* Close the pipes. */
+  g_clear_pointer (&plug_in->my_read,   g_io_channel_unref);
+  g_clear_pointer (&plug_in->my_write,  g_io_channel_unref);
+  g_clear_pointer (&plug_in->his_read,  g_io_channel_unref);
+  g_clear_pointer (&plug_in->his_write, g_io_channel_unref);
+
+  gimp_wire_clear_error ();
+
+  while (plug_in->temp_proc_frames)
+    {
+      GimpPlugInProcFrame *proc_frame = plug_in->temp_proc_frames->data;
+
+#ifdef GIMP_UNSTABLE
+      g_printerr ("plug-in '%s' aborted before sending its "
+                  "temporary procedure return values\n",
+                  gimp_object_get_name (plug_in));
+#endif
+
+      if (proc_frame->main_loop &&
+          g_main_loop_is_running (proc_frame->main_loop))
+        {
+          g_main_loop_quit (proc_frame->main_loop);
+        }
+
+      /* pop the frame here, because normally this only happens in
+       * gimp_plug_in_handle_temp_proc_return(), which can't
+       * be called after plug_in_close()
+       */
+      gimp_plug_in_proc_frame_pop (plug_in);
+    }
+
+  if (plug_in->main_proc_frame.main_loop &&
+      g_main_loop_is_running (plug_in->main_proc_frame.main_loop))
+    {
+#ifdef GIMP_UNSTABLE
+      g_printerr ("plug-in '%s' aborted before sending its "
+                  "procedure return values\n",
+                  gimp_object_get_name (plug_in));
+#endif
+
+      g_main_loop_quit (plug_in->main_proc_frame.main_loop);
+    }
+
+  if (plug_in->ext_main_loop &&
+      g_main_loop_is_running (plug_in->ext_main_loop))
+    {
+#ifdef GIMP_UNSTABLE
+      g_printerr ("extension '%s' aborted before sending its "
+                  "extension_ack message\n",
+                  gimp_object_get_name (plug_in));
+#endif
+
+      g_main_loop_quit (plug_in->ext_main_loop);
+    }
+
+  /* Unregister any temporary procedures. */
+  while (plug_in->temp_procedures)
+    gimp_plug_in_remove_temp_proc (plug_in, plug_in->temp_procedures->data);
+
+  gimp_plug_in_manager_remove_open_plug_in (plug_in->manager, plug_in);
+}
+
 GimpPlugInProcFrame *
 gimp_plug_in_get_proc_frame (GimpPlugIn *plug_in)
 {
@@ -821,12 +877,9 @@ gimp_plug_in_main_loop (GimpPlugIn *plug_in)
 
   proc_frame->main_loop = g_main_loop_new (NULL, FALSE);
 
-  gimp_threads_leave (plug_in->manager->gimp);
   g_main_loop_run (proc_frame->main_loop);
-  gimp_threads_enter (plug_in->manager->gimp);
 
-  g_main_loop_unref (proc_frame->main_loop);
-  proc_frame->main_loop = NULL;
+  g_clear_pointer (&proc_frame->main_loop, g_main_loop_unref);
 }
 
 void
@@ -854,109 +907,52 @@ gimp_plug_in_get_undo_desc (GimpPlugIn *plug_in)
 
   proc_frame = gimp_plug_in_get_proc_frame (plug_in);
 
-  if (proc_frame)
-    {
-      GimpPlugInProcedure *proc;
-
-      proc = GIMP_PLUG_IN_PROCEDURE (proc_frame->procedure);
-
-      if (proc)
-        undo_desc = gimp_plug_in_procedure_get_label (proc);
-    }
+  if (proc_frame && proc_frame->procedure)
+    undo_desc = gimp_procedure_get_label (proc_frame->procedure);
 
   return undo_desc ? undo_desc : gimp_object_get_name (plug_in);
 }
 
-/*  called from the PDB (gimp_plugin_menu_register)  */
-gboolean
-gimp_plug_in_menu_register (GimpPlugIn  *plug_in,
-                            const gchar *proc_name,
-                            const gchar *menu_path)
+void
+gimp_plug_in_add_temp_proc (GimpPlugIn             *plug_in,
+                            GimpTemporaryProcedure *proc)
 {
-  GimpPlugInProcedure *proc  = NULL;
-  GError              *error = NULL;
+  GimpPlugInProcedure *overridden;
+  const gchar         *help_domain;
 
-  g_return_val_if_fail (GIMP_IS_PLUG_IN (plug_in), FALSE);
-  g_return_val_if_fail (proc_name != NULL, FALSE);
-  g_return_val_if_fail (menu_path != NULL, FALSE);
+  g_return_if_fail (GIMP_IS_PLUG_IN (plug_in));
+  g_return_if_fail (GIMP_IS_TEMPORARY_PROCEDURE (proc));
 
-  if (plug_in->plug_in_def)
-    proc = gimp_plug_in_procedure_find (plug_in->plug_in_def->procedures,
-                                        proc_name);
+  overridden = gimp_plug_in_procedure_find (plug_in->temp_procedures,
+                                            gimp_object_get_name (proc));
 
-  if (! proc)
-    proc = gimp_plug_in_procedure_find (plug_in->temp_procedures, proc_name);
+  if (overridden)
+    gimp_plug_in_remove_temp_proc (plug_in,
+                                   GIMP_TEMPORARY_PROCEDURE (overridden));
 
-  if (! proc)
-    {
-      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-in \"%s\"\n(%s)\n"
-                    "attempted to register the menu item \"%s\" "
-                    "for the procedure \"%s\".\n"
-                    "It has however not installed that procedure.  This "
-                    "is not allowed.",
-                    gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    menu_path, proc_name);
+  help_domain = gimp_plug_in_manager_get_help_domain (plug_in->manager,
+                                                      plug_in->file,
+                                                      NULL);
 
-      return FALSE;
-    }
+  gimp_plug_in_procedure_set_help_domain (GIMP_PLUG_IN_PROCEDURE (proc),
+                                          help_domain);
 
-  switch (GIMP_PROCEDURE (proc)->proc_type)
-    {
-    case GIMP_INTERNAL:
-      return FALSE;
+  plug_in->temp_procedures = g_slist_prepend (plug_in->temp_procedures,
+                                              g_object_ref (proc));
+  gimp_plug_in_manager_add_temp_proc (plug_in->manager, proc);
+}
 
-    case GIMP_PLUGIN:
-    case GIMP_EXTENSION:
-      if (plug_in->call_mode != GIMP_PLUG_IN_CALL_QUERY &&
-          plug_in->call_mode != GIMP_PLUG_IN_CALL_INIT)
-        return FALSE;
+void
+gimp_plug_in_remove_temp_proc (GimpPlugIn             *plug_in,
+                               GimpTemporaryProcedure *proc)
+{
+  g_return_if_fail (GIMP_IS_PLUG_IN (plug_in));
+  g_return_if_fail (GIMP_IS_TEMPORARY_PROCEDURE (proc));
 
-    case GIMP_TEMPORARY:
-      break;
-    }
+  plug_in->temp_procedures = g_slist_remove (plug_in->temp_procedures, proc);
 
-  if (! proc->menu_label)
-    {
-      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-in \"%s\"\n(%s)\n"
-                    "attempted to register the menu item \"%s\" "
-                    "for procedure \"%s\".\n"
-                    "The menu label given in gimp_install_procedure() "
-                    "already contained a path.  To make this work, "
-                    "pass just the menu's label to "
-                    "gimp_install_procedure().",
-                    gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    menu_path, proc_name);
-
-      return FALSE;
-    }
-
-  if (! strlen (proc->menu_label))
-    {
-      gimp_message (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-                    "Plug-in \"%s\"\n(%s)\n"
-                    "attempted to register the procedure \"%s\" "
-                    "in the menu \"%s\", but the procedure has no label.  "
-                    "This is not allowed.",
-                    gimp_object_get_name (plug_in),
-                    gimp_filename_to_utf8 (plug_in->prog),
-                    proc_name, menu_path);
-
-      return FALSE;
-    }
-  if (! gimp_plug_in_procedure_add_menu_path (proc, menu_path, &error))
-    {
-      gimp_message_literal (plug_in->manager->gimp, NULL, GIMP_MESSAGE_ERROR,
-			    error->message);
-      g_clear_error (&error);
-
-      return FALSE;
-    }
-
-  return TRUE;
+  gimp_plug_in_manager_remove_temp_proc (plug_in->manager, proc);
+  g_object_unref (proc);
 }
 
 void
@@ -988,51 +984,3 @@ gimp_plug_in_get_error_handler (GimpPlugIn *plug_in)
 
   return GIMP_PDB_ERROR_HANDLER_INTERNAL;
 }
-
-void
-gimp_plug_in_add_temp_proc (GimpPlugIn             *plug_in,
-                            GimpTemporaryProcedure *proc)
-{
-  GimpPlugInProcedure *overridden;
-  const gchar         *locale_domain;
-  const gchar         *help_domain;
-
-  g_return_if_fail (GIMP_IS_PLUG_IN (plug_in));
-  g_return_if_fail (GIMP_IS_TEMPORARY_PROCEDURE (proc));
-
-  overridden = gimp_plug_in_procedure_find (plug_in->temp_procedures,
-                                            gimp_object_get_name (proc));
-
-  if (overridden)
-    gimp_plug_in_remove_temp_proc (plug_in,
-                                   GIMP_TEMPORARY_PROCEDURE (overridden));
-
-  locale_domain = gimp_plug_in_manager_get_locale_domain (plug_in->manager,
-                                                          plug_in->prog,
-                                                          NULL);
-  help_domain = gimp_plug_in_manager_get_help_domain (plug_in->manager,
-                                                      plug_in->prog,
-                                                      NULL);
-
-  gimp_plug_in_procedure_set_locale_domain (GIMP_PLUG_IN_PROCEDURE (proc),
-                                            locale_domain);
-  gimp_plug_in_procedure_set_help_domain (GIMP_PLUG_IN_PROCEDURE (proc),
-                                          help_domain);
-
-  plug_in->temp_procedures = g_slist_prepend (plug_in->temp_procedures,
-                                              g_object_ref (proc));
-  gimp_plug_in_manager_add_temp_proc (plug_in->manager, proc);
-}
-
-void
-gimp_plug_in_remove_temp_proc (GimpPlugIn             *plug_in,
-                               GimpTemporaryProcedure *proc)
-{
-  g_return_if_fail (GIMP_IS_PLUG_IN (plug_in));
-  g_return_if_fail (GIMP_IS_TEMPORARY_PROCEDURE (proc));
-
-  plug_in->temp_procedures = g_slist_remove (plug_in->temp_procedures, proc);
-
-  gimp_plug_in_manager_remove_temp_proc (plug_in->manager, proc);
-  g_object_unref (proc);
-}
