@@ -37,11 +37,15 @@
 #include FT_TRUETYPE_TABLES_H
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpconfig/gimpconfig.h"
 
 #include "text-types.h"
 
 #include "core/gimptempbuf.h"
 
+#include "core/gimp-memsize.h"
+#include "core/gimpcontainer.h"
+
 #include "gimpfont.h"
 
 #include "gimp-intl.h"
@@ -55,6 +59,20 @@
 
 #define DEBUGPRINT(x) /* g_print x */
 
+enum
+{
+  GIMP_FONT_SYMBOL_FONTHASH,
+  GIMP_FONT_SYMBOL_FULLNAME,
+  GIMP_FONT_SYMBOL_FAMILY,
+  GIMP_FONT_SYMBOL_STYLE,
+  GIMP_FONT_SYMBOL_PSNAME,
+  GIMP_FONT_SYMBOL_INDEX,
+  GIMP_FONT_SYMBOL_WEIGHT,
+  GIMP_FONT_SYMBOL_SLANT,
+  GIMP_FONT_SYMBOL_WIDTH,
+  GIMP_FONT_SYMBOL_FONTVERSION
+};
+
 enum
 {
   PROP_0,
@@ -71,56 +89,453 @@ struct _GimpFont
   PangoLayout  *popup_layout;
   gint          popup_width;
   gint          popup_height;
+  gchar        *lookup_name;
+
+  /*properties for serialization*/
+  gchar        *hash;
+  gchar        *fullname;
+  gchar        *family;
+  gchar        *style;
+  gchar        *psname;
+  gint          weight;
+  gint          width;
+  gint          index;
+  gint          slant;
+  gint          fontversion;
+
+  /*for backward compatibility*/
+  gchar        *desc;
 };
 
 struct _GimpFontClass
 {
   GimpDataClass   parent_class;
+
+  GimpContainer  *fontfactory;
 };
 
 
-static void          gimp_font_constructed      (GObject       *object);
-static void          gimp_font_finalize         (GObject       *object);
-static void          gimp_font_set_property     (GObject       *object,
-                                                 guint          property_id,
-                                                 const GValue  *value,
-                                                 GParamSpec    *pspec);
+static void          gimp_font_finalize           (GObject               *object);
+static void          gimp_font_set_property       (GObject               *object,
+                                                   guint                  property_id,
+                                                   const GValue          *value,
+                                                   GParamSpec            *pspec);
 
-static void          gimp_font_get_preview_size (GimpViewable  *viewable,
-                                                 gint           size,
-                                                 gboolean       popup,
-                                                 gboolean       dot_for_dot,
-                                                 gint          *width,
-                                                 gint          *height);
-static gboolean      gimp_font_get_popup_size   (GimpViewable  *viewable,
-                                                 gint           width,
-                                                 gint           height,
-                                                 gboolean       dot_for_dot,
-                                                 gint          *popup_width,
-                                                 gint          *popup_height);
-static GimpTempBuf * gimp_font_get_new_preview  (GimpViewable  *viewable,
-                                                 GimpContext   *context,
-                                                 gint           width,
-                                                 gint           height);
+static void          gimp_font_get_preview_size   (GimpViewable          *viewable,
+                                                   gint                   size,
+                                                   gboolean               popup,
+                                                   gboolean               dot_for_dot,
+                                                   gint                  *width,
+                                                   gint                  *height);
+static gboolean      gimp_font_get_popup_size     (GimpViewable          *viewable,
+                                                   gint                   width,
+                                                   gint                   height,
+                                                   gboolean               dot_for_dot,
+                                                   gint                  *popup_width,
+                                                   gint                  *popup_height);
+static GimpTempBuf * gimp_font_get_new_preview    (GimpViewable          *viewable,
+                                                   GimpContext           *context,
+                                                   gint                   width,
+                                                   gint                   height);
 
-static const gchar * gimp_font_get_sample_string (PangoContext         *context,
-                                                  PangoFontDescription *font_desc);
+static void          gimp_font_config_iface_init  (GimpConfigInterface  *iface);
+static gboolean      gimp_font_serialize          (GimpConfig           *config,
+                                                   GimpConfigWriter     *writer,
+                                                   gpointer              data);
+static GimpConfig  * gimp_font_deserialize_create (GType                 type,
+                                                   GScanner             *scanner,
+                                                   gint                  nest_level,
+                                                   gpointer              data);
+
+static gint64        gimp_font_get_memsize        (GimpObject           *object,
+                                                   gint64               *gui_size);
+
+static inline gboolean gimp_font_covers_string    (PangoFont            *font,
+                                                   const gchar          *sample);
+static hb_tag_t      get_hb_table_type            (PangoOTTableType      table_type);
+static const gchar * gimp_font_get_sample_string  (PangoContext         *context,
+                                                   PangoFontDescription *font_desc);
+
+static const gchar * gimp_font_get_hash           (GimpFont             *font);
 
 
-G_DEFINE_TYPE (GimpFont, gimp_font, GIMP_TYPE_DATA)
+G_DEFINE_TYPE_WITH_CODE (GimpFont, gimp_font, GIMP_TYPE_DATA,
+                         G_IMPLEMENT_INTERFACE (GIMP_TYPE_CONFIG,
+                         gimp_font_config_iface_init))
+
 
 #define parent_class gimp_font_parent_class
 
+static void
+gimp_font_config_iface_init (GimpConfigInterface *iface)
+{
+  iface->serialize          = gimp_font_serialize;
+  iface->deserialize_create = gimp_font_deserialize_create;
+}
+
+static gboolean
+gimp_font_serialize (GimpConfig       *config,
+                     GimpConfigWriter *writer,
+                     gpointer          data)
+{
+  GimpFont *font;
+
+  g_return_val_if_fail (GIMP_IS_FONT (config), FALSE);
+
+  font = GIMP_FONT (config);
+
+  if (font == GIMP_FONT (gimp_font_get_standard ()))
+    return TRUE;
+
+  gimp_config_writer_open   (writer, "fonthash");
+  gimp_config_writer_string (writer, gimp_font_get_hash (font));
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "fullname");
+  gimp_config_writer_string (writer, font->fullname);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "family");
+  gimp_config_writer_string (writer, font->family);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "style");
+  gimp_config_writer_string (writer, font->style);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "psname");
+  gimp_config_writer_string (writer, font->psname);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "index");
+  gimp_config_writer_printf (writer, "%d", font->index);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "weight");
+  gimp_config_writer_printf (writer, "%d", font->weight);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "slant");
+  gimp_config_writer_printf (writer, "%d", font->slant);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "width");
+  gimp_config_writer_printf (writer, "%d", font->width);
+  gimp_config_writer_close  (writer);
+
+  gimp_config_writer_open   (writer, "fontversion");
+  gimp_config_writer_printf (writer, "%d", font->fontversion);
+  gimp_config_writer_close  (writer);
+
+  return TRUE;
+}
+
+static GimpConfig *
+gimp_font_deserialize_create (GType     type,
+                              GScanner *scanner,
+                              gint      nest_level,
+                              gpointer  data)
+{
+  GimpFont      *font;
+  GimpContainer *fonts_container         = GIMP_FONT_CLASS (g_type_class_peek (GIMP_TYPE_FONT))->fontfactory;
+  gint           most_similar_font_index = -1;
+  gint           font_count              = gimp_container_get_n_children (fonts_container);
+  gint           largest_similarity      = 0;
+  GList         *similar_fonts           = NULL;
+  GList         *iter;
+  gint           i;
+  gchar         *fonthash                = NULL;
+  gchar         *fullname                = NULL;
+  gchar         *family                  = NULL;
+  gchar         *psname                  = NULL;
+  gchar         *style                   = NULL;
+  gint           index                   = -1;
+  gint           weight                  = -1;
+  gint           slant                   = -1;
+  gint           width                   = -1;
+  gint           fontversion             = -1;
+  guint          scope_id;
+  guint          old_scope_id;
+
+  /* This is for backward compatibility with older xcf files.
+   * The font used to be serialized as a string containing
+   * its name.
+   */
+  if (g_scanner_peek_next_token (scanner) == G_TOKEN_STRING)
+    {
+      gchar* font_name = NULL;
+
+      gimp_scanner_parse_string (scanner, &font_name);
+
+      for (i = 0; i < font_count; i++)
+        {
+          font = GIMP_FONT (gimp_container_get_child_by_index (fonts_container, i));
+
+          if (!g_strcmp0 (font->desc, font_name))
+            break;
+
+          font = NULL;
+        }
+
+      if (font == NULL)
+        font = GIMP_FONT (gimp_font_get_standard ());
+
+      g_object_ref (font);
+
+      g_free (font_name);
+
+      return GIMP_CONFIG (font);
+    }
+
+  if (g_scanner_peek_next_token (scanner) == G_TOKEN_RIGHT_PAREN)
+    return GIMP_CONFIG (GIMP_FONT (gimp_font_get_standard ()));
+
+  scope_id = g_type_qname (type);
+  old_scope_id = g_scanner_set_scope (scanner, scope_id);
+
+  g_scanner_scope_add_symbol (scanner, scope_id, "fonthash",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_FONTHASH));
+  g_scanner_scope_add_symbol (scanner, scope_id, "fullname",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_FULLNAME));
+  g_scanner_scope_add_symbol (scanner, scope_id, "family",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_FAMILY));
+  g_scanner_scope_add_symbol (scanner, scope_id, "style",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_STYLE));
+  g_scanner_scope_add_symbol (scanner, scope_id, "psname",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_PSNAME));
+  g_scanner_scope_add_symbol (scanner, scope_id, "index",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_INDEX));
+  g_scanner_scope_add_symbol (scanner, scope_id, "weight",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_WEIGHT));
+  g_scanner_scope_add_symbol (scanner, scope_id, "slant",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_SLANT));
+  g_scanner_scope_add_symbol (scanner, scope_id, "width",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_WIDTH));
+  g_scanner_scope_add_symbol (scanner, scope_id, "fontversion",
+                              GINT_TO_POINTER (GIMP_FONT_SYMBOL_FONTVERSION));
+
+  while (g_scanner_peek_next_token (scanner) == G_TOKEN_LEFT_PAREN)
+    {
+      GTokenType token;
+
+      g_scanner_get_next_token  (scanner); /* ( */
+
+      token = g_scanner_get_next_token (scanner);
+
+      if (token != G_TOKEN_SYMBOL)
+        {
+          /* When encountering an unknown symbol, we simply ignore its contents
+           * (up to the next closing parenthese). This would allow us to load
+           * future XCF files in case we add new fields to recognize fonts
+           * without having to bump the XCF version (just ignoring unknown
+           * fields). We still output a small message on stderr.
+           */
+          if (token == G_TOKEN_IDENTIFIER)
+            g_printerr ("%s: ignoring unknown symbol '%s'.\n", G_STRFUNC, scanner->value.v_string);
+          else
+            g_printerr ("%s: ignoring unknown token %d.\n", G_STRFUNC, token);
+
+          while ((token = g_scanner_get_next_token (scanner)) != G_TOKEN_EOF)
+            {
+              if (token == G_TOKEN_RIGHT_PAREN)
+                break;
+            }
+
+          continue;
+        }
+
+      switch (GPOINTER_TO_INT (scanner->value.v_symbol))
+        {
+        case GIMP_FONT_SYMBOL_FONTHASH:
+          gimp_scanner_parse_string (scanner, &fonthash);
+          break;
+        case GIMP_FONT_SYMBOL_FULLNAME:
+          gimp_scanner_parse_string (scanner, &fullname);
+          break;
+        case GIMP_FONT_SYMBOL_FAMILY:
+          gimp_scanner_parse_string (scanner, &family);
+          break;
+        case GIMP_FONT_SYMBOL_STYLE:
+          gimp_scanner_parse_string (scanner, &style);
+          break;
+        case GIMP_FONT_SYMBOL_PSNAME:
+          gimp_scanner_parse_string (scanner, &psname);
+          break;
+        case GIMP_FONT_SYMBOL_INDEX:
+          gimp_scanner_parse_int (scanner, &index);
+          break;
+        case GIMP_FONT_SYMBOL_WEIGHT:
+          gimp_scanner_parse_int (scanner, &weight);
+          break;
+        case GIMP_FONT_SYMBOL_SLANT:
+          gimp_scanner_parse_int (scanner, &slant);
+          break;
+        case GIMP_FONT_SYMBOL_WIDTH:
+          gimp_scanner_parse_int (scanner, &width);
+          break;
+        case GIMP_FONT_SYMBOL_FONTVERSION:
+          gimp_scanner_parse_int (scanner, &fontversion);
+          break;
+        default:
+          break;
+        }
+
+      if (g_scanner_get_next_token (scanner) != G_TOKEN_RIGHT_PAREN)
+        break;
+    }
+
+  g_scanner_scope_remove_symbol (scanner, scope_id, "fonthash");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "fullname");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "family");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "style");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "psname");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "index");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "weight");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "slant");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "width");
+  g_scanner_scope_remove_symbol (scanner, scope_id, "fontversion");
+  g_scanner_set_scope (scanner, old_scope_id);
+
+  for (i = 0; i < font_count; i++)
+    {
+      gint current_font_similarity = 0;
+
+      font = GIMP_FONT (gimp_container_get_child_by_index (fonts_container, i));
+
+      /* Some attrs are more identifying than others,
+       * hence their higher importance in measuring similarity.
+       */
+
+      if (fullname != NULL && !g_strcmp0 (font->fullname, fullname))
+        current_font_similarity += 5;
+
+      if (family != NULL && !g_strcmp0 (font->family, family))
+        current_font_similarity += 5;
+
+      if (psname != NULL && font->psname != NULL && !g_strcmp0 (font->psname, psname))
+        current_font_similarity += 5;
+
+      if (current_font_similarity < 5)
+        continue;
+
+      if (style != NULL && font->style != NULL && !g_strcmp0 (font->style, style))
+        current_font_similarity++;
+
+      if (font->weight != -1 && font->weight == weight)
+        current_font_similarity++;
+
+      if (font->width != -1 && font->width == width)
+        current_font_similarity++;
+
+      if (font->slant != -1 && font->slant == slant)
+        current_font_similarity++;
+
+      if (font->index != -1 && font->index == index)
+        current_font_similarity++;
+
+      if (font->fontversion != -1 && font->fontversion == fontversion)
+        current_font_similarity++;
+
+      if (current_font_similarity > largest_similarity)
+        {
+          largest_similarity      = current_font_similarity;
+          most_similar_font_index = i;
+          g_clear_pointer (&similar_fonts, g_list_free);
+          similar_fonts = g_list_prepend (similar_fonts, GINT_TO_POINTER (i));
+        }
+      else if (current_font_similarity == largest_similarity)
+        {
+          similar_fonts = g_list_prepend (similar_fonts, GINT_TO_POINTER (i));
+        }
+    }
+
+  /* In case there are multiple font with identical info,
+   * the font file hash should be used for comparison.
+   */
+  if (g_list_length (similar_fonts) == 1)
+    g_clear_pointer (&similar_fonts, g_list_free);
+  for (iter = similar_fonts; iter; iter = iter->next)
+    {
+      i    = GPOINTER_TO_INT (iter->data);
+      font = GIMP_FONT (gimp_container_get_child_by_index (fonts_container, i));
+
+      if (g_strcmp0 (gimp_font_get_hash (font), fonthash) == 0)
+        {
+          most_similar_font_index = i;
+          break;
+        }
+    }
+
+  if (most_similar_font_index > -1)
+    font = GIMP_FONT (gimp_container_get_child_by_index (fonts_container, most_similar_font_index));
+  else
+    font = GIMP_FONT (gimp_font_get_standard ());
+
+  g_object_ref (font);
+
+  g_list_free (similar_fonts);
+  g_free (fonthash);
+  g_free (fullname);
+  g_free (family);
+  g_free (psname);
+  g_free (style);
+
+  return GIMP_CONFIG (font);
+}
+
+void
+gimp_font_class_set_font_factory (GimpContainer *factory)
+{
+  GimpFontClass *klass = GIMP_FONT_CLASS (g_type_class_peek (GIMP_TYPE_FONT));
+  klass->fontfactory   = factory;
+}
+
+void
+gimp_font_set_font_info (GimpFont *font,
+                         gpointer  font_info[])
+{
+  font->fullname    =  g_strdup ((gchar*)font_info[PROP_FULLNAME]);
+  font->family      =  g_strdup ((gchar*)font_info[PROP_FAMILY]);
+  font->style       =  g_strdup ((gchar*)font_info[PROP_STYLE]);
+  font->psname      =  g_strdup ((gchar*)font_info[PROP_PSNAME]);
+  font->desc        =  g_strdup ((gchar*)font_info[PROP_DESC]);
+  font->weight      = *(gint*)font_info[PROP_WEIGHT];
+  font->width       = *(gint*)font_info[PROP_WIDTH];
+  font->index       = *(gint*)font_info[PROP_INDEX];
+  font->slant       = *(gint*)font_info[PROP_SLANT];
+  font->fontversion = *(gint*)font_info[PROP_FONTVERSION];
+}
+
+void
+gimp_font_set_lookup_name (GimpFont    *font,
+                           gchar       *name)
+{
+  font->lookup_name = name;
+}
+
+gboolean
+gimp_font_match_by_lookup_name (GimpFont    *font,
+                                const gchar *name)
+{
+  return !g_strcmp0 (font->lookup_name, name);
+}
+
+const gchar*
+gimp_font_get_lookup_name (GimpFont *font)
+{
+  return font->lookup_name;
+}
 
 static void
 gimp_font_class_init (GimpFontClass *klass)
 {
-  GObjectClass      *object_class   = G_OBJECT_CLASS (klass);
-  GimpViewableClass *viewable_class = GIMP_VIEWABLE_CLASS (klass);
+  GObjectClass      *object_class      = G_OBJECT_CLASS (klass);
+  GimpViewableClass *viewable_class    = GIMP_VIEWABLE_CLASS (klass);
+  GimpObjectClass   *gimp_object_class = GIMP_OBJECT_CLASS (klass);
 
-  object_class->constructed         = gimp_font_constructed;
   object_class->finalize            = gimp_font_finalize;
   object_class->set_property        = gimp_font_set_property;
+  gimp_object_class->get_memsize    = gimp_font_get_memsize;
 
   viewable_class->get_preview_size  = gimp_font_get_preview_size;
   viewable_class->get_popup_size    = gimp_font_get_popup_size;
@@ -140,15 +555,6 @@ gimp_font_init (GimpFont *font)
 {
 }
 
-static void
-gimp_font_constructed (GObject *object)
-{
-  G_OBJECT_CLASS (parent_class)->constructed (object);
-
-  gimp_data_make_internal (GIMP_DATA (object),
-                           gimp_object_get_name (object));
-}
-
 static void
 gimp_font_finalize (GObject *object)
 {
@@ -156,6 +562,13 @@ gimp_font_finalize (GObject *object)
 
   g_clear_object (&font->pango_context);
   g_clear_object (&font->popup_layout);
+  g_free (font->lookup_name);
+  g_free (font->hash);
+  g_free (font->fullname);
+  g_free (font->family);
+  g_free (font->style);
+  g_free (font->psname);
+  g_free (font->desc);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -182,6 +595,25 @@ gimp_font_set_property (GObject       *object,
     }
 }
 
+static gint64
+gimp_font_get_memsize (GimpObject *object,
+                       gint64     *gui_size)
+{
+  GimpFont *font    = GIMP_FONT (object);
+  gint64    memsize = 0;
+
+  memsize += gimp_string_get_memsize (font->lookup_name);
+  memsize += gimp_string_get_memsize (font->hash);
+  memsize += gimp_string_get_memsize (font->fullname);
+  memsize += gimp_string_get_memsize (font->family);
+  memsize += gimp_string_get_memsize (font->style);
+  memsize += gimp_string_get_memsize (font->psname);
+  memsize += gimp_string_get_memsize (font->desc);
+
+  return memsize + GIMP_OBJECT_CLASS (parent_class)->get_memsize (object,
+                                                                  gui_size);
+}
+
 static void
 gimp_font_get_preview_size (GimpViewable *viewable,
                             gint          size,
@@ -211,7 +643,7 @@ gimp_font_get_popup_size (GimpViewable *viewable,
   if (! font->pango_context)
     return FALSE;
 
-  name = gimp_object_get_name (font);
+  name = font->lookup_name;
 
   font_desc = pango_font_description_from_string (name);
   g_return_val_if_fail (font_desc != NULL, FALSE);
@@ -266,7 +698,7 @@ gimp_font_get_new_preview (GimpViewable *viewable,
       PangoFontDescription *font_desc;
       const gchar          *name;
 
-      name = gimp_object_get_name (font);
+      name = font->lookup_name;
 
       DEBUGPRINT (("%s: ", name));
 
@@ -337,9 +769,11 @@ gimp_font_get_standard (void)
     {
       g_set_weak_pointer (&standard_font,
                           g_object_new (GIMP_TYPE_FONT,
-                                        "name", "Standard",
+                                        "name", "",
                                         NULL));;
 
+     /* pango_font_description_from_string doesn't accept NULL */
+      GIMP_FONT (standard_font)->lookup_name = g_strdup ("");
       gimp_data_clean (standard_font);
       gimp_data_make_internal (standard_font, "gimp-font-standard");
     }
@@ -348,6 +782,8 @@ gimp_font_get_standard (void)
 }
 
 
+/* Private functions */
+
 static inline gboolean
 gimp_font_covers_string (PangoFont   *font,
                          const gchar *sample)
@@ -857,3 +1293,36 @@ gimp_font_get_sample_string (PangoContext         *context,
   DEBUGPRINT (("=> fallback, use Aa\n"));
   return "Aa";
 }
+
+static const gchar *
+gimp_font_get_hash (GimpFont *font)
+{
+  /* Computing the hash is expensive, so it's only done in a few case, such as
+   * when serializing, when a similar fonts at deserialization looks like it
+   * could be a good identity candidate.
+   */
+  if (font->hash == NULL)
+    {
+      PangoFontDescription *pfd        = pango_font_description_from_string (font->lookup_name);
+      PangoFcFont          *pango_font = PANGO_FC_FONT (pango_context_load_font (font->pango_context, pfd));
+      GChecksum            *checksum   = g_checksum_new (G_CHECKSUM_SHA256);
+      gchar                *file;
+      hb_blob_t            *hb_blob;
+      guint                 length;
+      const char           *hb_data;
+
+      FcPatternGetString (pango_fc_font_get_pattern (pango_font), FC_FILE, 0, (FcChar8 **) &file);
+
+      hb_blob  = hb_blob_create_from_file (file);
+      hb_data  = hb_blob_get_data (hb_blob, &length);
+      g_checksum_update (checksum, (const guchar*) hb_data, length);
+      font->hash = g_strdup (g_checksum_get_string (checksum));
+
+      pango_font_description_free (pfd);
+      g_object_unref (pango_font);
+      hb_blob_destroy (hb_blob);
+      g_checksum_free (checksum);
+    }
+
+  return font->hash;
+}
