@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -21,6 +21,7 @@
 
 #include "widgets-types.h"
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 
 #include "paint/gimpink-blob.h"
@@ -46,8 +47,8 @@ static void      gimp_blob_editor_get_property   (GObject        *object,
                                                   GValue         *value,
                                                   GParamSpec     *pspec);
 
-static gboolean  gimp_blob_editor_expose         (GtkWidget      *widget,
-                                                  GdkEventExpose *event);
+static gboolean  gimp_blob_editor_draw           (GtkWidget      *widget,
+                                                  cairo_t        *cr);
 static gboolean  gimp_blob_editor_button_press   (GtkWidget      *widget,
                                                   GdkEventButton *event);
 static gboolean  gimp_blob_editor_button_release (GtkWidget      *widget,
@@ -78,7 +79,7 @@ gimp_blob_editor_class_init (GimpBlobEditorClass *klass)
   object_class->set_property         = gimp_blob_editor_set_property;
   object_class->get_property         = gimp_blob_editor_get_property;
 
-  widget_class->expose_event         = gimp_blob_editor_expose;
+  widget_class->draw                 = gimp_blob_editor_draw;
   widget_class->button_press_event   = gimp_blob_editor_button_press;
   widget_class->button_release_event = gimp_blob_editor_button_release;
   widget_class->motion_notify_event  = gimp_blob_editor_motion_notify;
@@ -99,7 +100,7 @@ gimp_blob_editor_class_init (GimpBlobEditorClass *klass)
   g_object_class_install_property (object_class, PROP_ANGLE,
                                    g_param_spec_double ("blob-angle",
                                                         NULL, NULL,
-                                                        -90.0, 90.0, 0.0,
+                                                        -G_PI, G_PI, 0.0,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT));
 }
@@ -183,16 +184,14 @@ gimp_blob_editor_get_property (GObject    *object,
 }
 
 static gboolean
-gimp_blob_editor_expose (GtkWidget      *widget,
-                         GdkEventExpose *event)
+gimp_blob_editor_draw (GtkWidget *widget,
+                       cairo_t   *cr)
 {
-  GimpBlobEditor *editor = GIMP_BLOB_EDITOR (widget);
-  GtkStyle       *style  = gtk_widget_get_style (widget);
-  GtkStateType    state  = gtk_widget_get_state (widget);
-  GtkAllocation   allocation;
-  cairo_t        *cr;
-  GdkRectangle    rect;
-  gint            r0;
+  GimpBlobEditor  *editor = GIMP_BLOB_EDITOR (widget);
+  GtkStyleContext *style  = gtk_widget_get_style_context (widget);
+  GtkAllocation    allocation;
+  GdkRectangle     rect;
+  gint             r0;
 
   gtk_widget_get_allocation (widget, &allocation);
 
@@ -201,8 +200,6 @@ gimp_blob_editor_expose (GtkWidget      *widget,
   if (r0 < 2)
     return TRUE;
 
-  cr = gdk_cairo_create (gtk_widget_get_window (widget));
-
   gimp_blob_editor_draw_blob (editor, cr,
                               allocation.width  / 2.0,
                               allocation.height / 2.0,
@@ -210,16 +207,18 @@ gimp_blob_editor_expose (GtkWidget      *widget,
 
   gimp_blob_editor_get_handle (editor, &rect);
 
-  cairo_rectangle (cr,
-                   rect.x + 0.5, rect.y + 0.5, rect.width - 1, rect.width - 1);
-  gdk_cairo_set_source_color (cr, &style->light[state]);
-  cairo_fill_preserve (cr);
+  gtk_style_context_save (style);
 
-  gdk_cairo_set_source_color (cr, &style->dark[state]);
-  cairo_set_line_width (cr, 1);
-  cairo_stroke (cr);
+  gtk_style_context_add_class (style, GTK_STYLE_CLASS_BUTTON);
 
-  cairo_destroy (cr);
+  gtk_style_context_set_state (style,
+                               editor->in_handle ?
+                               GTK_STATE_FLAG_PRELIGHT : 0);
+
+  gtk_render_background (style, cr, rect.x, rect.y, rect.width, rect.height);
+  gtk_render_frame (style, cr, rect.x, rect.y, rect.width, rect.height);
+
+  gtk_style_context_restore (style);
 
   return TRUE;
 }
@@ -229,15 +228,9 @@ gimp_blob_editor_button_press (GtkWidget      *widget,
                                GdkEventButton *event)
 {
   GimpBlobEditor *editor = GIMP_BLOB_EDITOR (widget);
-  GdkRectangle    rect;
 
-  gimp_blob_editor_get_handle (editor, &rect);
-
-  if ((event->x >= rect.x) && (event->x - rect.x < rect.width) &&
-      (event->y >= rect.y) && (event->y - rect.y < rect.height))
-    {
-      editor->active = TRUE;
-    }
+  if (editor->in_handle)
+    editor->active = TRUE;
 
   return TRUE;
 }
@@ -292,6 +285,30 @@ gimp_blob_editor_motion_notify (GtkWidget      *widget,
                         NULL);
         }
     }
+  else
+    {
+      GdkRectangle rect;
+      gboolean     in_handle;
+
+      gimp_blob_editor_get_handle (editor, &rect);
+
+      if ((event->x >= rect.x) && (event->x - rect.x < rect.width) &&
+          (event->y >= rect.y) && (event->y - rect.y < rect.height))
+        {
+          in_handle = TRUE;
+        }
+      else
+        {
+          in_handle = FALSE;
+        }
+
+      if (in_handle != editor->in_handle)
+        {
+          editor->in_handle = in_handle;
+
+          gtk_widget_queue_draw (widget);
+        }
+    }
 
   return TRUE;
 }
@@ -328,11 +345,12 @@ gimp_blob_editor_draw_blob (GimpBlobEditor *editor,
                             gdouble         yc,
                             gdouble         radius)
 {
-  GtkWidget    *widget   = GTK_WIDGET (editor);
-  GtkStyle     *style    = gtk_widget_get_style (widget);
-  GimpBlob     *blob;
-  GimpBlobFunc  function = gimp_blob_ellipse;
-  gint          i;
+  GtkWidget       *widget   = GTK_WIDGET (editor);
+  GtkStyleContext *style    = gtk_widget_get_style_context (widget);
+  GimpBlob        *blob;
+  GimpBlobFunc     function = gimp_blob_ellipse;
+  GdkRGBA          color;
+  gint             i;
 
   switch (editor->type)
     {
@@ -384,6 +402,8 @@ gimp_blob_editor_draw_blob (GimpBlobEditor *editor,
 
   g_free (blob);
 
-  gdk_cairo_set_source_color (cr, &style->fg[gtk_widget_get_state (widget)]);
+  gtk_style_context_get_color (style, gtk_widget_get_state_flags (widget),
+                               &color);
+  gdk_cairo_set_source_rgba (cr, &color);
   cairo_fill (cr);
 }
