@@ -28,6 +28,7 @@
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
 #include "libgimpwidgets/gimpwidgets.h"
+#include "libgimpconfig/gimpconfig.h"
 
 #include "widgets-types.h"
 
@@ -35,6 +36,12 @@
 
 #include "core/gimp.h"
 #include "core/gimpcontext.h"
+#include "core/gimpimage.h"
+#include "core/gimpimage-color-profile.h"
+
+#include "display/display-types.h"
+#include "display/gimpdisplay.h"
+#include "display/gimpdisplayshell.h"
 
 #include "gimpcoloreditor.h"
 #include "gimpcolorhistory.h"
@@ -55,52 +62,66 @@ enum
 
 static void   gimp_color_editor_docked_iface_init (GimpDockedInterface  *iface);
 
-static void   gimp_color_editor_constructed     (GObject           *object);
-static void   gimp_color_editor_dispose         (GObject           *object);
-static void   gimp_color_editor_set_property    (GObject           *object,
-                                                 guint              property_id,
-                                                 const GValue      *value,
-                                                 GParamSpec        *pspec);
-static void   gimp_color_editor_get_property    (GObject           *object,
-                                                 guint              property_id,
-                                                 GValue            *value,
-                                                 GParamSpec        *pspec);
+static void   gimp_color_editor_constructed       (GObject           *object);
+static void   gimp_color_editor_dispose           (GObject           *object);
+static void   gimp_color_editor_set_property      (GObject           *object,
+                                                   guint              property_id,
+                                                   const GValue      *value,
+                                                   GParamSpec        *pspec);
+static void   gimp_color_editor_get_property      (GObject           *object,
+                                                   guint              property_id,
+                                                   GValue            *value,
+                                                   GParamSpec        *pspec);
 
-static void   gimp_color_editor_style_updated   (GtkWidget         *widget);
+static void   gimp_color_editor_style_updated     (GtkWidget         *widget);
 
-static void   gimp_color_editor_set_aux_info    (GimpDocked        *docked,
-                                                 GList             *aux_info);
-static GList *gimp_color_editor_get_aux_info     (GimpDocked       *docked);
-static GtkWidget *gimp_color_editor_get_preview (GimpDocked        *docked,
-                                                 GimpContext       *context,
-                                                 GtkIconSize        size);
-static void   gimp_color_editor_set_context     (GimpDocked        *docked,
-                                                 GimpContext       *context);
+static void   gimp_color_editor_set_aux_info      (GimpDocked        *docked,
+                                                   GList             *aux_info);
+static GList *gimp_color_editor_get_aux_info       (GimpDocked       *docked);
+static GtkWidget *gimp_color_editor_get_preview   (GimpDocked        *docked,
+                                                   GimpContext       *context,
+                                                   GtkIconSize        size);
+static void   gimp_color_editor_set_context       (GimpDocked        *docked,
+                                                   GimpContext       *context);
 
-static void   gimp_color_editor_fg_changed      (GimpContext       *context,
-                                                 const GimpRGB     *rgb,
-                                                 GimpColorEditor   *editor);
-static void   gimp_color_editor_bg_changed      (GimpContext       *context,
-                                                 const GimpRGB     *rgb,
-                                                 GimpColorEditor   *editor);
-static void   gimp_color_editor_color_changed   (GimpColorSelector *selector,
-                                                 const GimpRGB     *rgb,
-                                                 const GimpHSV     *hsv,
-                                                 GimpColorEditor   *editor);
-static void   gimp_color_editor_tab_toggled     (GtkWidget         *widget,
-                                                 GimpColorEditor   *editor);
-static void   gimp_color_editor_fg_bg_notify    (GtkWidget         *widget,
-                                                 GParamSpec        *pspec,
-                                                 GimpColorEditor   *editor);
-static void   gimp_color_editor_color_picked    (GtkWidget         *widget,
-                                                 const GimpRGB     *rgb,
-                                                 GimpColorEditor   *editor);
-static void   gimp_color_editor_entry_changed   (GimpColorHexEntry *entry,
-                                                 GimpColorEditor   *editor);
+static void   gimp_color_editor_fg_changed        (GimpContext       *context,
+                                                   GeglColor         *color,
+                                                   GimpColorEditor   *editor);
+static void   gimp_color_editor_bg_changed        (GimpContext       *context,
+                                                   GeglColor         *color,
+                                                   GimpColorEditor   *editor);
+static void   gimp_color_editor_color_changed     (GimpColorSelector *selector,
+                                                   GeglColor         *color,
+                                                   GimpColorEditor   *editor);
+static void   gimp_color_editor_tab_toggled       (GtkWidget         *widget,
+                                                   GimpColorEditor   *editor);
+static void   gimp_color_editor_fg_bg_notify      (GtkWidget         *widget,
+                                                   GParamSpec        *pspec,
+                                                   GimpColorEditor   *editor);
+static void   gimp_color_editor_color_picked      (GtkWidget         *widget,
+                                                   const GimpRGB     *rgb,
+                                                   GimpColorEditor   *editor);
+static void   gimp_color_editor_entry_changed     (GimpColorHexEntry *entry,
+                                                   GimpColorEditor   *editor);
+
+static void  gimp_color_editor_history_selected   (GimpColorHistory *history,
+                                                   GeglColor        *color,
+                                                   GimpColorEditor  *editor);
+
+static void  gimp_color_editor_display_changed    (GimpContext     *context,
+                                                   GimpDisplay     *display,
+                                                   GimpColorEditor *editor);
+static void  gimp_color_editor_image_changed      (GimpDisplay     *display,
+                                                   GParamSpec      *pspec,
+                                                   GimpColorEditor *editor);
+static void  gimp_color_editor_shell_changed      (GimpDisplay     *display,
+                                                   GParamSpec      *pspec,
+                                                   GimpColorEditor *editor);
+static void  gimp_color_editor_update_simulation  (GimpImage       *image,
+                                                   GimpColorEditor *editor);
+static void  gimp_color_editor_enable_simulation  (GimpColorEditor *editor);
+static void  gimp_color_editor_update_format      (GimpColorEditor *editor);
 
-static void  gimp_color_editor_history_selected (GimpColorHistory *history,
-                                                 const GimpRGB    *rgb,
-                                                 GimpColorEditor  *editor);
 
 G_DEFINE_TYPE_WITH_CODE (GimpColorEditor, gimp_color_editor, GIMP_TYPE_EDITOR,
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_DOCKED,
@@ -155,8 +176,7 @@ gimp_color_editor_init (GimpColorEditor *editor)
   gint         content_spacing;
   gint         button_spacing;
   GtkIconSize  button_icon_size;
-  GimpRGB      rgb;
-  GimpHSV      hsv;
+  GeglColor   *color;
   GList       *list;
   GSList      *group;
   gint         icon_width  = 40;
@@ -165,9 +185,6 @@ gimp_color_editor_init (GimpColorEditor *editor)
   editor->context = NULL;
   editor->edit_bg = FALSE;
 
-  gimp_rgba_set (&rgb, 0.0, 0.0, 0.0, 1.0);
-  gimp_rgb_to_hsv (&rgb, &hsv);
-
   gtk_widget_style_get (GTK_WIDGET (editor),
                         "content-spacing",  &content_spacing,
                         "button-spacing",   &button_spacing,
@@ -179,9 +196,10 @@ gimp_color_editor_init (GimpColorEditor *editor)
   gtk_box_pack_start (GTK_BOX (editor), editor->hbox, FALSE, FALSE, 0);
   gtk_widget_show (editor->hbox);
 
-  editor->notebook = gimp_color_selector_new (GIMP_TYPE_COLOR_NOTEBOOK,
-                                              &rgb, &hsv,
+  color = gegl_color_new ("black");
+  editor->notebook = gimp_color_selector_new (GIMP_TYPE_COLOR_NOTEBOOK, color,
                                               GIMP_COLOR_SELECTOR_RED);
+  g_object_unref (color);
   gimp_color_selector_set_show_alpha (GIMP_COLOR_SELECTOR (editor->notebook),
                                       FALSE);
   gtk_box_pack_start (GTK_BOX (editor), editor->notebook,
@@ -276,7 +294,9 @@ gimp_color_editor_init (GimpColorEditor *editor)
 static void
 gimp_color_editor_constructed (GObject *object)
 {
-  GimpColorEditor *editor = GIMP_COLOR_EDITOR (object);
+  GimpColorEditor *editor       = GIMP_COLOR_EDITOR (object);
+  GimpContext     *user_context = editor->context->gimp->user_context;
+  GimpDisplay     *display      = gimp_context_get_display (user_context);
   GtkWidget       *history;
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
@@ -289,6 +309,12 @@ gimp_color_editor_constructed (GObject *object)
   g_signal_connect (history, "color-selected",
                     G_CALLBACK (gimp_color_editor_history_selected),
                     editor);
+
+  g_signal_connect (user_context, "display-changed",
+                    G_CALLBACK (gimp_color_editor_display_changed),
+                    editor);
+
+  gimp_color_editor_display_changed (user_context, display, editor);
 }
 
 static void
@@ -455,7 +481,7 @@ gimp_color_editor_set_context (GimpDocked  *docked,
 
   if (editor->context)
     {
-      GimpRGB rgb;
+      GeglColor *color;
 
       g_object_ref (editor->context);
 
@@ -481,13 +507,13 @@ gimp_color_editor_set_context (GimpDocked  *docked,
 
       if (editor->edit_bg)
         {
-          gimp_context_get_background (editor->context, &rgb);
-          gimp_color_editor_bg_changed (editor->context, &rgb, editor);
+          color = gimp_context_get_background (editor->context);
+          gimp_color_editor_bg_changed (editor->context, color, editor);
         }
       else
         {
-          gimp_context_get_foreground (editor->context, &rgb);
-          gimp_color_editor_fg_changed (editor->context, &rgb, editor);
+          color = gimp_context_get_foreground (editor->context);
+          gimp_color_editor_fg_changed (editor->context, color, editor);
         }
 
       g_object_set_data (G_OBJECT (context->gimp->config->color_management),
@@ -536,18 +562,13 @@ gimp_color_editor_style_updated (GtkWidget *widget)
 
 static void
 gimp_color_editor_set_color (GimpColorEditor *editor,
-                             const GimpRGB   *rgb)
+                             GeglColor       *color)
 {
-  GimpHSV hsv;
-
-  gimp_rgb_to_hsv (rgb, &hsv);
-
   g_signal_handlers_block_by_func (editor->notebook,
                                    gimp_color_editor_color_changed,
                                    editor);
 
-  gimp_color_selector_set_color (GIMP_COLOR_SELECTOR (editor->notebook),
-                                 rgb, &hsv);
+  gimp_color_selector_set_color (GIMP_COLOR_SELECTOR (editor->notebook), color);
 
   g_signal_handlers_unblock_by_func (editor->notebook,
                                      gimp_color_editor_color_changed,
@@ -557,8 +578,7 @@ gimp_color_editor_set_color (GimpColorEditor *editor,
                                    gimp_color_editor_entry_changed,
                                    editor);
 
-  gimp_color_hex_entry_set_color (GIMP_COLOR_HEX_ENTRY (editor->hex_entry),
-                                  rgb);
+  gimp_color_hex_entry_set_color (GIMP_COLOR_HEX_ENTRY (editor->hex_entry), color);
 
   g_signal_handlers_unblock_by_func (editor->hex_entry,
                                      gimp_color_editor_entry_changed,
@@ -567,26 +587,25 @@ gimp_color_editor_set_color (GimpColorEditor *editor,
 
 static void
 gimp_color_editor_fg_changed (GimpContext     *context,
-                              const GimpRGB   *rgb,
+                              GeglColor       *color,
                               GimpColorEditor *editor)
 {
   if (! editor->edit_bg)
-    gimp_color_editor_set_color (editor, rgb);
+    gimp_color_editor_set_color (editor, color);
 }
 
 static void
 gimp_color_editor_bg_changed (GimpContext     *context,
-                              const GimpRGB   *rgb,
+                              GeglColor       *color,
                               GimpColorEditor *editor)
 {
   if (editor->edit_bg)
-    gimp_color_editor_set_color (editor, rgb);
+    gimp_color_editor_set_color (editor, color);
 }
 
 static void
 gimp_color_editor_color_changed (GimpColorSelector *selector,
-                                 const GimpRGB     *rgb,
-                                 const GimpHSV     *hsv,
+                                 GeglColor         *color,
                                  GimpColorEditor   *editor)
 {
   if (editor->context)
@@ -597,7 +616,7 @@ gimp_color_editor_color_changed (GimpColorSelector *selector,
                                            gimp_color_editor_bg_changed,
                                            editor);
 
-          gimp_context_set_background (editor->context, rgb);
+          gimp_context_set_background (editor->context, color);
 
           g_signal_handlers_unblock_by_func (editor->context,
                                              gimp_color_editor_bg_changed,
@@ -609,7 +628,7 @@ gimp_color_editor_color_changed (GimpColorSelector *selector,
                                            gimp_color_editor_fg_changed,
                                            editor);
 
-          gimp_context_set_foreground (editor->context, rgb);
+          gimp_context_set_foreground (editor->context, color);
 
           g_signal_handlers_unblock_by_func (editor->context,
                                              gimp_color_editor_fg_changed,
@@ -621,8 +640,7 @@ gimp_color_editor_color_changed (GimpColorSelector *selector,
                                    gimp_color_editor_entry_changed,
                                    editor);
 
-  gimp_color_hex_entry_set_color (GIMP_COLOR_HEX_ENTRY (editor->hex_entry),
-                                  rgb);
+  gimp_color_hex_entry_set_color (GIMP_COLOR_HEX_ENTRY (editor->hex_entry), color);
 
   g_signal_handlers_unblock_by_func (editor->hex_entry,
                                      gimp_color_editor_entry_changed,
@@ -670,17 +688,17 @@ gimp_color_editor_fg_bg_notify (GtkWidget       *widget,
 
       if (editor->context)
         {
-          GimpRGB rgb;
+          GeglColor *color;
 
           if (edit_bg)
             {
-              gimp_context_get_background (editor->context, &rgb);
-              gimp_color_editor_bg_changed (editor->context, &rgb, editor);
+              color = gimp_context_get_background (editor->context);
+              gimp_color_editor_bg_changed (editor->context, color, editor);
             }
           else
             {
-              gimp_context_get_foreground (editor->context, &rgb);
-              gimp_color_editor_fg_changed (editor->context, &rgb, editor);
+              color = gimp_context_get_foreground (editor->context);
+              gimp_color_editor_fg_changed (editor->context, color, editor);
             }
         }
     }
@@ -693,10 +711,16 @@ gimp_color_editor_color_picked (GtkWidget       *widget,
 {
   if (editor->context)
     {
+      GeglColor *color = gegl_color_new ("black");
+
+      gegl_color_set_rgba_with_space (color, rgb->r, rgb->g, rgb->b, rgb->a, NULL);
+
       if (editor->edit_bg)
-        gimp_context_set_background (editor->context, rgb);
+        gimp_context_set_background (editor->context, color);
       else
-        gimp_context_set_foreground (editor->context, rgb);
+        gimp_context_set_foreground (editor->context, color);
+
+      g_object_unref (color);
     }
 }
 
@@ -704,29 +728,200 @@ static void
 gimp_color_editor_entry_changed (GimpColorHexEntry *entry,
                                  GimpColorEditor   *editor)
 {
-  GimpRGB  rgb;
+  GeglColor *color;
 
-  gimp_color_hex_entry_get_color (entry, &rgb);
+  color = gimp_color_hex_entry_get_color (entry);
 
   if (editor->context)
     {
       if (editor->edit_bg)
-        gimp_context_set_background (editor->context, &rgb);
+        gimp_context_set_background (editor->context, color);
       else
-        gimp_context_set_foreground (editor->context, &rgb);
+        gimp_context_set_foreground (editor->context, color);
     }
+
+  g_object_unref (color);
 }
 
 static void
 gimp_color_editor_history_selected (GimpColorHistory *history,
-                                    const GimpRGB    *rgb,
+                                    GeglColor        *color,
                                     GimpColorEditor  *editor)
 {
   if (editor->context)
     {
       if (editor->edit_bg)
-        gimp_context_set_background (editor->context, rgb);
+        gimp_context_set_background (editor->context, color);
       else
-        gimp_context_set_foreground (editor->context, rgb);
+        gimp_context_set_foreground (editor->context, color);
     }
 }
+
+static void
+gimp_color_editor_display_changed (GimpContext     *context,
+                                   GimpDisplay     *display,
+                                   GimpColorEditor *editor)
+{
+  if (editor->active_display != display)
+    {
+      if (editor->active_display)
+        {
+          g_signal_handlers_disconnect_by_func (editor->active_display,
+                                                gimp_color_editor_image_changed,
+                                                editor);
+          g_signal_handlers_disconnect_by_func (editor->active_display,
+                                                gimp_color_editor_shell_changed,
+                                                editor);
+        }
+
+      g_set_weak_pointer (&editor->active_display, display);
+
+      if (display)
+        {
+          g_signal_connect (display, "notify::image",
+                            G_CALLBACK (gimp_color_editor_image_changed),
+                            editor);
+          g_signal_connect (display, "notify::shell",
+                            G_CALLBACK (gimp_color_editor_shell_changed),
+                            editor);
+        }
+
+      gimp_color_editor_image_changed (display, NULL, editor);
+      gimp_color_editor_shell_changed (display, NULL, editor);
+    }
+}
+
+static void
+gimp_color_editor_image_changed (GimpDisplay     *display,
+                                 GParamSpec      *pspec,
+                                 GimpColorEditor *editor)
+{
+  GimpImage *image = display ? gimp_display_get_image (display) : NULL;
+
+  if (editor->active_image != image)
+    {
+      if (editor->active_image)
+        {
+          g_signal_handlers_disconnect_by_func (editor->active_image,
+                                                gimp_color_editor_update_simulation,
+                                                editor);
+          g_signal_handlers_disconnect_by_func (editor->active_image,
+                                                gimp_color_editor_update_format,
+                                                editor);
+        }
+
+      g_set_weak_pointer (&editor->active_image, image);
+
+      if (image)
+        {
+          g_signal_connect (image, "simulation-profile-changed",
+                            G_CALLBACK (gimp_color_editor_update_simulation),
+                            editor);
+          g_signal_connect (image, "simulation-intent-changed",
+                            G_CALLBACK (gimp_color_editor_update_simulation),
+                            editor);
+          g_signal_connect (image, "simulation-bpc-changed",
+                            G_CALLBACK (gimp_color_editor_update_simulation),
+                            editor);
+
+          g_signal_connect_swapped (image, "profile-changed",
+                                    G_CALLBACK (gimp_color_editor_update_format),
+                                    editor);
+          g_signal_connect_swapped (image, "precision-changed",
+                                    G_CALLBACK (gimp_color_editor_update_format),
+                                    editor);
+          g_signal_connect_swapped (image, "notify::base-type",
+                                    G_CALLBACK (gimp_color_editor_update_format),
+                                    editor);
+        }
+
+      gimp_color_editor_update_simulation (image, editor);
+      gimp_color_editor_update_format (editor);
+    }
+}
+
+static void
+gimp_color_editor_shell_changed (GimpDisplay     *display,
+                                 GParamSpec      *pspec,
+                                 GimpColorEditor *editor)
+{
+  GimpDisplayShell *shell = display ? gimp_display_get_shell (display) : NULL;
+
+  if (editor->active_shell != shell)
+    {
+      GimpColorConfig *config;
+
+      if (editor->active_shell)
+        {
+          config = gimp_display_shell_get_color_config (editor->active_shell);
+
+          g_signal_handlers_disconnect_by_func (config,
+                                                gimp_color_editor_enable_simulation,
+                                                editor);
+        }
+
+      g_set_weak_pointer (&editor->active_shell, shell);
+
+      if (shell)
+        {
+          config = gimp_display_shell_get_color_config (shell);
+
+          g_signal_connect_swapped (config, "notify::mode",
+                                    G_CALLBACK (gimp_color_editor_enable_simulation),
+                                    editor);
+        }
+
+      gimp_color_editor_enable_simulation (editor);
+    }
+}
+
+static void
+gimp_color_editor_update_simulation (GimpImage       *image,
+                                     GimpColorEditor *editor)
+{
+  g_return_if_fail (GIMP_IS_COLOR_EDITOR (editor));
+
+  if (image)
+    gimp_color_notebook_set_simulation (GIMP_COLOR_NOTEBOOK (editor->notebook),
+                                        gimp_image_get_simulation_profile (image),
+                                        gimp_image_get_simulation_intent (image),
+                                        gimp_image_get_simulation_bpc (image));
+  else
+    gimp_color_notebook_set_simulation (GIMP_COLOR_NOTEBOOK (editor->notebook),
+                                        NULL,
+                                        GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC,
+                                        FALSE);
+}
+
+static void
+gimp_color_editor_enable_simulation (GimpColorEditor *editor)
+{
+  g_return_if_fail (GIMP_IS_COLOR_EDITOR (editor));
+
+  if (editor->active_shell && editor->active_image)
+    {
+      GimpColorConfig         *config;
+      GimpColorManagementMode  mode;
+
+      config = gimp_display_shell_get_color_config (editor->active_shell);
+      mode   = gimp_color_config_get_mode (config);
+      gimp_color_notebook_enable_simulation (GIMP_COLOR_NOTEBOOK (editor->notebook),
+                                             (mode == GIMP_COLOR_MANAGEMENT_SOFTPROOF));
+    }
+  else
+    {
+      gimp_color_notebook_enable_simulation (GIMP_COLOR_NOTEBOOK (editor->notebook), FALSE);
+    }
+}
+
+static void
+gimp_color_editor_update_format (GimpColorEditor *editor)
+{
+  g_return_if_fail (GIMP_IS_COLOR_EDITOR (editor));
+
+  if (editor->active_image)
+    gimp_color_notebook_set_format (GIMP_COLOR_NOTEBOOK (editor->notebook),
+                                    gimp_image_get_layer_format (editor->active_image, FALSE));
+  else
+    gimp_color_notebook_set_format (GIMP_COLOR_NOTEBOOK (editor->notebook), NULL);
+}
