@@ -53,7 +53,8 @@
 enum
 {
   PROP_0,
-  PROP_CONTEXT
+  PROP_CONTEXT,
+  PROP_INDEX
 };
 
 enum
@@ -165,6 +166,10 @@ gimp_colormap_selection_class_init (GimpColormapSelectionClass* klass)
                                                         GIMP_TYPE_CONTEXT,
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT));
+  g_object_class_install_property (object_class, PROP_INDEX,
+                                   g_param_spec_int ("index", NULL, NULL,
+                                                     0, G_MAXINT, 0,
+                                                     GIMP_PARAM_READABLE));
 }
 
 static void
@@ -240,7 +245,7 @@ gimp_colormap_selection_init (GimpColormapSelection *selection)
   selection->color_entry = gimp_color_hex_entry_new ();
   gtk_widget_set_halign (selection->color_entry, GTK_ALIGN_START);
   gimp_grid_attach_aligned (GTK_GRID (grid), 0, 1,
-                            _("HTML notation:"), 0.0, 0.5,
+                            _("HTML notation (sRGB):"), 0.0, 0.5,
                             selection->color_entry, 1);
 
   g_signal_connect (selection->color_entry, "color-changed",
@@ -281,6 +286,9 @@ gimp_colormap_selection_get_property (GObject    *object,
     case PROP_CONTEXT:
       g_value_set_object (value, selection->context);
       break;
+    case PROP_INDEX:
+      g_value_set_int (value, selection->col_index);
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -360,7 +368,7 @@ gimp_colormap_selection_new (GimpContext *context)
 
 gint
 gimp_colormap_selection_get_index (GimpColormapSelection *selection,
-                                   const GimpRGB         *search)
+                                   GeglColor             *search)
 {
   GimpImage *image;
   gint       index;
@@ -376,20 +384,20 @@ gimp_colormap_selection_get_index (GimpColormapSelection *selection,
 
   if (search)
     {
-      GimpRGB temp;
+      GeglColor *temp;
 
-      gimp_image_get_colormap_entry (image, index, &temp);
+      temp = gimp_image_get_colormap_entry (image, index);
 
-      if (gimp_rgb_distance (&temp, search) > RGB_EPSILON)
+      if (! gimp_color_is_perceptually_identical (temp, search))
         {
-          gint n_colors = gimp_image_get_colormap_size (image);
-          gint i;
+          gint n_colors;
 
-          for (i = 0; i < n_colors; i++)
+          n_colors = gimp_palette_get_n_colors (gimp_image_get_colormap_palette (image));
+          for (gint i = 0; i < n_colors; i++)
             {
-              gimp_image_get_colormap_entry (image, i, &temp);
+              temp = gimp_image_get_colormap_entry (image, i);
 
-              if (gimp_rgb_distance (&temp, search) < RGB_EPSILON)
+              if (gimp_color_is_perceptually_identical (temp, search))
                 {
                   index = i;
                   break;
@@ -404,19 +412,20 @@ gimp_colormap_selection_get_index (GimpColormapSelection *selection,
 gboolean
 gimp_colormap_selection_set_index (GimpColormapSelection *selection,
                                    gint                   index,
-                                   GimpRGB               *color)
+                                   GeglColor             *color)
 {
   GimpImage *image;
   gint       size;
 
   g_return_val_if_fail (GIMP_IS_COLORMAP_SELECTION (selection), FALSE);
+  g_return_val_if_fail (color == NULL || GEGL_IS_COLOR (color), FALSE);
 
   image = gimp_context_get_image (selection->context);
 
   if (! HAVE_COLORMAP (image))
     return FALSE;
 
-  size = gimp_image_get_colormap_size (image);
+  size = gimp_palette_get_n_colors (gimp_image_get_colormap_palette (image));
 
   if (size < 1)
     return FALSE;
@@ -429,6 +438,7 @@ gimp_colormap_selection_set_index (GimpColormapSelection *selection,
 
       selection->col_index = index;
 
+      g_object_notify (G_OBJECT (selection), "index");
       gimp_palette_view_select_entry (GIMP_PALETTE_VIEW (selection->view),
                                       gimp_palette_get_entry (palette, index));
 
@@ -436,7 +446,15 @@ gimp_colormap_selection_set_index (GimpColormapSelection *selection,
     }
 
   if (color)
-    gimp_image_get_colormap_entry (image, index, color);
+    {
+      GeglColor  *c = gimp_image_get_colormap_entry (image, index);
+      const Babl *format;
+      guchar      pixel[40];
+
+      format = gegl_color_get_format (c);
+      gegl_color_get_pixel (c, format, pixel);
+      gegl_color_set_pixel (color, format, pixel);
+    }
 
   return TRUE;
 }
@@ -445,6 +463,7 @@ gint
 gimp_colormap_selection_max_index (GimpColormapSelection *selection)
 {
   GimpImage *image;
+  gint       n_colors;
 
   g_return_val_if_fail (GIMP_IS_COLORMAP_SELECTION (selection), -1);
 
@@ -453,7 +472,9 @@ gimp_colormap_selection_max_index (GimpColormapSelection *selection)
   if (! HAVE_COLORMAP (image))
     return -1;
 
-  return MAX (0, gimp_image_get_colormap_size (image) - 1);
+  n_colors = gimp_palette_get_n_colors (gimp_image_get_colormap_palette (image));
+
+  return MAX (0, n_colors - 1);
 }
 
 GimpPaletteEntry *
@@ -553,10 +574,13 @@ gimp_colormap_selection_preview_draw (GtkWidget             *widget,
 static void
 gimp_colormap_selection_update_entries (GimpColormapSelection *selection)
 {
-  GimpImage *image = gimp_context_get_image (selection->context);
+  GimpImage *image    = gimp_context_get_image (selection->context);
+  gint       n_colors = 0;
 
-  if (! HAVE_COLORMAP (image) ||
-      ! gimp_image_get_colormap_size (image))
+  if (gimp_image_get_colormap_palette (image))
+    n_colors = gimp_palette_get_n_colors (gimp_image_get_colormap_palette (image));
+
+  if (! HAVE_COLORMAP (image) || ! n_colors)
     {
       gtk_widget_set_sensitive (selection->index_spinbutton, FALSE);
       gtk_widget_set_sensitive (selection->color_entry, FALSE);
@@ -566,16 +590,20 @@ gimp_colormap_selection_update_entries (GimpColormapSelection *selection)
     }
   else
     {
-      GimpRGB  color;
-      guchar   r, g, b;
-      gchar   *string;
+      GeglColor  *color;
+      guchar      rgb[3];
+      gchar      *string;
 
       gtk_adjustment_set_value (selection->index_adjustment,
                                 selection->col_index);
-      gimp_image_get_colormap_entry (image, selection->col_index, &color);
-      gimp_rgb_get_uchar (&color, &r, &g, &b);
+      color = gimp_image_get_colormap_entry (image, selection->col_index);
+      /* The color entry shows a CSS/SVG notation, which so far means
+       * sRGB. But is it really what we want? Don't we want to edit
+       * colors in the image's specific RGB color space? TODO
+       */
+      gegl_color_get_pixel (color, babl_format ("R'G'B' u8"), rgb);
 
-      string = g_strdup_printf ("%02x%02x%02x", r, g, b);
+      string = g_strdup_printf ("%02x%02x%02x", rgb[0], rgb[1], rgb[2]);
       gtk_entry_set_text (GTK_ENTRY (selection->color_entry), string);
       g_free (string);
 
@@ -662,12 +690,13 @@ gimp_colormap_hex_entry_changed (GimpColorHexEntry     *entry,
 
   if (image)
     {
-      GimpRGB color;
+      GeglColor *color;
 
-      gimp_color_hex_entry_get_color (entry, &color);
+      color = gimp_color_hex_entry_get_color (entry);
 
-      gimp_image_set_colormap_entry (image, selection->col_index, &color, TRUE);
+      gimp_image_set_colormap_entry (image, selection->col_index, color, TRUE);
       gimp_image_flush (image);
+      g_object_unref (color);
     }
 }
 
@@ -785,6 +814,8 @@ gimp_colormap_selection_set_palette (GimpColormapSelection *selection)
     {
       if (selection->active_palette)
         {
+          gimp_colormap_selection_set_index (selection, 0, NULL);
+
           g_signal_handlers_disconnect_by_func (selection->active_palette,
                                                 G_CALLBACK (gtk_widget_queue_draw),
                                                 selection);
@@ -796,13 +827,15 @@ gimp_colormap_selection_set_palette (GimpColormapSelection *selection)
 
       if (palette)
         {
+          gint n_colors;
+
+          n_colors = gimp_palette_get_n_colors (palette);
           g_signal_connect_swapped (palette, "dirty",
                                     G_CALLBACK (gtk_widget_queue_draw),
                                     selection);
           gimp_view_set_viewable (GIMP_VIEW (selection->view),
                                   GIMP_VIEWABLE (palette));
-          gtk_adjustment_set_upper (selection->index_adjustment,
-                                    gimp_image_get_colormap_size (selection->active_image) - 1);
+          gtk_adjustment_set_upper (selection->index_adjustment, n_colors - 1);
         }
     }
 }
