@@ -41,6 +41,8 @@
 #include "core/gimplist.h"
 #include "core/gimpimage.h"
 
+#include "menus/menus.h"
+
 #include "gimpcontainercombobox.h"
 #include "gimpcontainerview.h"
 #include "gimpdialogfactory.h"
@@ -289,7 +291,6 @@ gimp_dock_window_constructed (GObject *object)
   GimpGuiConfig   *config;
   GimpContext     *factory_context;
   GimpMenuFactory *menu_factory;
-  GtkAccelGroup   *accel_group;
   Gimp            *gimp;
   gint             menu_view_width  = -1;
   gint             menu_view_height = -1;
@@ -314,18 +315,15 @@ gimp_dock_window_constructed (GObject *object)
   /* Setup hints */
   gimp_window_set_hint (GTK_WINDOW (dock_window), config->dock_window_hint);
 
-  menu_factory =
-    gimp_dialog_factory_get_menu_factory (dock_window->p->dialog_factory);
+  menu_factory = menus_get_global_menu_factory (gimp);
 
   /* Make image window related keyboard shortcuts work also when a
    * dock window is the focused window
    */
   dock_window->p->ui_manager =
-    gimp_menu_factory_manager_new (menu_factory,
+    gimp_menu_factory_get_manager (menu_factory,
                                    dock_window->p->ui_manager_name,
                                    dock_window);
-  accel_group = gimp_ui_manager_get_accel_group (dock_window->p->ui_manager);
-  gtk_window_add_accel_group (GTK_WINDOW (dock_window), accel_group);
 
   g_signal_connect_object (dock_window->p->context, "display-changed",
                            G_CALLBACK (gimp_dock_window_display_changed),
@@ -486,9 +484,7 @@ gimp_dock_window_dispose (GObject *object)
       dock_window->p->image_flush_handler_id = 0;
     }
 
-  g_clear_object (&dock_window->p->ui_manager);
   g_clear_object (&dock_window->p->dialog_factory);
-  g_clear_object (&dock_window->p->context);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -496,9 +492,22 @@ gimp_dock_window_dispose (GObject *object)
 static void
 gimp_dock_window_finalize (GObject *object)
 {
-  GimpDockWindow *dock_window = GIMP_DOCK_WINDOW (object);
+  GimpDockWindow  *dock_window = GIMP_DOCK_WINDOW (object);
+  GimpMenuFactory *menu_factory;
+  Gimp            *gimp;
 
+  gimp         = GIMP (dock_window->p->context->gimp);
+  menu_factory = menus_get_global_menu_factory (gimp);
+
+  /* If the whole GUI is in destruction, global menu factory might already not
+   * exist anymore.
+   */
+  if (menu_factory != NULL)
+    gimp_menu_factory_delete_manager (menu_factory,
+                                      dock_window->p->ui_manager_name,
+                                      dock_window);
   g_clear_pointer (&dock_window->p->ui_manager_name, g_free);
+  g_clear_object (&dock_window->p->context);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
