@@ -164,6 +164,7 @@ typedef struct _GimpFontSelect               GimpFontSelect;
 typedef struct _GimpGradientSelect           GimpGradientSelect;
 typedef struct _GimpPaletteSelect            GimpPaletteSelect;
 typedef struct _GimpPatternSelect            GimpPatternSelect;
+typedef struct _GimpPickableSelect           GimpPickableSelect;
 typedef struct _GimpPdbDialog                GimpPdbDialog;
 
 
@@ -221,6 +222,7 @@ typedef struct _GimpMeter                    GimpMeter;
 typedef struct _GimpModifiersEditor          GimpModifiersEditor;
 typedef struct _GimpOverlayBox               GimpOverlayBox;
 typedef struct _GimpPickableButton           GimpPickableButton;
+typedef struct _GimpPickableChooser          GimpPickableChooser;
 typedef struct _GimpPickablePopup            GimpPickablePopup;
 typedef struct _GimpPivotSelector            GimpPivotSelector;
 typedef struct _GimpPlugInView               GimpPlugInView;
