@@ -114,6 +114,7 @@ G_DEFINE_QUARK (gimp-plug-in-error-quark, gimp_plug_in_error)
 enum
 {
   PROP_0,
+  PROP_PROGRAM_NAME,
   PROP_READ_CHANNEL,
   PROP_WRITE_CHANNEL,
   N_PROPS
@@ -130,6 +131,8 @@ struct _GimpPlugInMenuBranch
 
 struct _GimpPlugInPrivate
 {
+  gchar      *program_name;
+
   GIOChannel *read_channel;
   GIOChannel *write_channel;
 
@@ -207,7 +210,9 @@ static void       gimp_plug_in_push_procedure    (GimpPlugIn      *plug_in,
 static void       gimp_plug_in_pop_procedure     (GimpPlugIn      *plug_in,
                                                   GimpProcedure   *procedure);
 static void       gimp_plug_in_destroy_hashes    (GimpPlugIn      *plug_in);
-static void       gimp_plug_in_destroy_proxies   (GHashTable      *hash_table,
+static void       gimp_plug_in_destroy_proxies   (GimpPlugIn      *plug_in,
+                                                  GHashTable      *hash_table,
+                                                  const gchar     *type,
                                                   gboolean         destroy_all);
 
 static void       gimp_plug_in_init_i18n         (GimpPlugIn      *plug_in);
@@ -233,6 +238,19 @@ gimp_plug_in_class_init (GimpPlugInClass *klass)
 
   klass->set_i18n            = gimp_plug_in_real_set_i18n;
 
+  /**
+   * GimpPlugIn:program-name:
+   *
+   * The program name as usually found on argv[0]
+   */
+  props[PROP_PROGRAM_NAME] =
+    g_param_spec_string ("program-name",
+                         "The plug-in executable",
+                         "The executable name as usually found on argv[0]",
+                         NULL,
+                         GIMP_PARAM_WRITABLE |
+                         G_PARAM_CONSTRUCT_ONLY);
+
   /**
    * GimpPlugIn:read-channel:
    *
@@ -310,6 +328,7 @@ gimp_plug_in_finalize (GObject *object)
   GimpPlugIn *plug_in = GIMP_PLUG_IN (object);
   GList      *list;
 
+  g_clear_pointer (&plug_in->priv->program_name, g_free);
   g_clear_pointer (&plug_in->priv->translation_domain_name, g_free);
   g_clear_object  (&plug_in->priv->translation_domain_path);
 
@@ -327,10 +346,10 @@ gimp_plug_in_finalize (GObject *object)
 
   g_clear_pointer (&plug_in->priv->menu_branches, g_list_free);
 
-  gimp_plug_in_destroy_proxies (plug_in->priv->displays,  TRUE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->images,    TRUE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->items,     TRUE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->resources, TRUE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->displays,  "display",  TRUE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->images,    "image",    TRUE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->items,     "item",     TRUE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->resources, "resource", TRUE);
 
   gimp_plug_in_destroy_hashes (plug_in);
 
@@ -347,6 +366,10 @@ gimp_plug_in_set_property (GObject      *object,
 
   switch (property_id)
     {
+    case PROP_PROGRAM_NAME:
+      plug_in->priv->program_name = g_value_dup_string (value);
+      break;
+
     case PROP_READ_CHANNEL:
       plug_in->priv->read_channel = g_value_get_boxed (value);
       break;
@@ -1387,7 +1410,7 @@ gimp_plug_in_proc_run_internal (GimpPlugIn    *plug_in,
                                               proc_run->n_params,
                                               FALSE);
 
-  return_values = gimp_procedure_run (procedure, arguments);
+  return_values = _gimp_procedure_run_array (procedure, arguments);
 
   gimp_value_array_unref (arguments);
 
@@ -1441,17 +1464,17 @@ gimp_plug_in_pop_procedure (GimpPlugIn    *plug_in,
 
   _gimp_procedure_destroy_proxies (procedure);
 
-  gimp_plug_in_destroy_proxies (plug_in->priv->displays,  FALSE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->images,    FALSE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->items,     FALSE);
-  gimp_plug_in_destroy_proxies (plug_in->priv->resources, FALSE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->displays,  "display",  FALSE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->images,    "image",    FALSE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->items,     "item",     FALSE);
+  gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->resources, "resource", FALSE);
 
   if (! plug_in->priv->procedure_stack)
     {
-      gimp_plug_in_destroy_proxies (plug_in->priv->displays,  TRUE);
-      gimp_plug_in_destroy_proxies (plug_in->priv->images,    TRUE);
-      gimp_plug_in_destroy_proxies (plug_in->priv->items,     TRUE);
-      gimp_plug_in_destroy_proxies (plug_in->priv->resources, TRUE);
+      gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->displays,  "display",  TRUE);
+      gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->images,    "image",    TRUE);
+      gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->items,     "item",     TRUE);
+      gimp_plug_in_destroy_proxies (plug_in, plug_in->priv->resources, "resource", TRUE);
 
       gimp_plug_in_destroy_hashes (plug_in);
     }
@@ -1483,7 +1506,7 @@ _gimp_plug_in_get_display (GimpPlugIn *plug_in,
 
       g_hash_table_insert (plug_in->priv->displays,
                            GINT_TO_POINTER (display_id),
-                           g_object_ref (display) /* add debug ref */);
+                           display);
     }
 
   return display;
@@ -1515,7 +1538,7 @@ _gimp_plug_in_get_image (GimpPlugIn *plug_in,
 
       g_hash_table_insert (plug_in->priv->images,
                            GINT_TO_POINTER (image_id),
-                           g_object_ref (image) /* add debug ref */);
+                           image);
     }
 
   return image;
@@ -1581,7 +1604,7 @@ _gimp_plug_in_get_item (GimpPlugIn *plug_in,
       if (item)
         g_hash_table_insert (plug_in->priv->items,
                              GINT_TO_POINTER (item_id),
-                             g_object_ref (item) /* add debug ref */);
+                             item);
     }
 
   return item;
@@ -1641,12 +1664,48 @@ _gimp_plug_in_get_resource (GimpPlugIn *plug_in,
       if (resource)
         g_hash_table_insert (plug_in->priv->resources,
                              GINT_TO_POINTER (resource_id),
-                             g_object_ref (resource) /* add debug ref */);
+                             resource);
     }
 
   return resource;
 }
 
+gboolean
+_gimp_plug_in_manage_memory_manually (GimpPlugIn *plug_in)
+{
+  gboolean manual_management = TRUE;
+
+  if (plug_in->priv->program_name)
+    {
+      GFile *file = g_file_new_for_path (plug_in->priv->program_name);
+
+      /* Limitations:
+       * 1. Checking a file extension (and trusting argv[0] in general) is not
+       *    the best idea. I thought about the possibility of passing through
+       *    the name of the used interpreter from core to plug-in, but even this
+       *    should not be trusted because it is configurable.
+       * 2. An alternative could be to pass the information if a plug-in is
+       *    interpreted (i.e. it has an interp file). Though I'm unsure if there
+       *    might be some interpreted language out there with manual memory
+       *    management.
+       * 3. This list only contains the bindings we actually tested with (and
+       *    have demo plug-ins for). Much more bindings are possible.
+       *
+       * So in the end, this test is just worth so far, but it's
+       * acceptable for the officially supported bindings at least.
+       */
+      if (gimp_file_has_extension (file, ".lua") ||
+          gimp_file_has_extension (file, ".py")  ||
+          gimp_file_has_extension (file, ".js")  ||
+          gimp_file_has_extension (file, ".gjs"))
+        manual_management = FALSE;
+
+      g_object_unref (file);
+    }
+
+  return manual_management;
+}
+
 static void
 gimp_plug_in_destroy_hashes (GimpPlugIn *plug_in)
 {
@@ -1657,8 +1716,10 @@ gimp_plug_in_destroy_hashes (GimpPlugIn *plug_in)
 }
 
 static void
-gimp_plug_in_destroy_proxies (GHashTable *hash_table,
-                              gboolean    destroy_all)
+gimp_plug_in_destroy_proxies (GimpPlugIn  *plug_in,
+                              GHashTable  *hash_table,
+                              const gchar *type,
+                              gboolean     destroy_all)
 {
   GHashTableIter iter;
   gpointer       key, value;
@@ -1672,52 +1733,56 @@ gimp_plug_in_destroy_proxies (GHashTable *hash_table,
     {
       GObject *object = value;
 
-      if (object->ref_count == 2)
+      if (object->ref_count == 1)
         {
-          /* this is the normal case for an unused proxy */
-
-          g_object_unref (object);
+          /* this is the normal case for an unused proxy, since we already
+           * destroyed the only other reference in procedure with
+           * _gimp_procedure_destroy_proxies().
+           */
           g_hash_table_iter_remove (&iter);
         }
-      else if (object->ref_count == 1)
+      else if (! G_IS_OBJECT (object))
         {
-          /* this is debug code, a plug-in MUST NOT unref a proxy, we
-           * only catch one unref here, more then one will crash right
-           * in this function
+          /* this is debug code, a plug-in MUST NOT unref a proxy. To be nice,
+           * we steal the object from the table, as removing it normally would
+           * crash, since the object is not valid anymore.
            */
-
-          gint id;
-
-          g_object_get (object, "id", &id, NULL);
-
-          g_printerr ("%s: ERROR: %s proxy with ID %d was unrefed "
+          g_printerr ("%s: ERROR: %s proxy was unrefed "
                       "by plug-in, it MUST NOT do that!\n",
-                      G_STRFUNC, G_OBJECT_TYPE_NAME (object), id);
+                      G_STRFUNC, type);
 
-          g_hash_table_iter_remove (&iter);
+          g_hash_table_iter_steal (&iter);
         }
       else if (destroy_all)
         {
           /* this is debug code, a plug-in MUST NOT ref a proxy */
-
           gint id;
 
           g_object_get (object, "id", &id, NULL);
 
-          g_printerr ("%s: ERROR: %s proxy with ID %d was refed "
-                      "by plug-in, it MUST NOT do that!\n",
-                      G_STRFUNC, G_OBJECT_TYPE_NAME (object), id);
+          /* Some bindings are always keeping a reference to objects, in
+           * particular this is the behavior of lua and gjs bindings where
+           * object->ref_count == 2 at this point.
+           * As for the Python binding, it is only true for global
+           * variables (scoped variables are properly freed before the
+           * plug-in ends), which anyway is not the best practice.
+           * In any case, it's not something which the plug-in developer can do
+           * anything about so this error is only confusing.
+           * See #3912.
+           */
+          if (_gimp_plug_in_manage_memory_manually (plug_in))
+            g_printerr ("%s: ERROR: %s proxy with ID %d was refed "
+                        "by plug-in, it MUST NOT do that!\n",
+                        G_STRFUNC, G_OBJECT_TYPE_NAME (object), id);
 
 #if 0
-          /* the code used to do this, but it appears that some bindings
-           * keep references around until later, let's keep this for
-           * reference and as a reminder to figure if we can do anything
-           * generic about it that works for all bindings.
+          /* The code used to do this, which is only meaningful when the bug is
+           * really in the plug-in code. If it's one of the cases where the
+           * reference is actually held by a bindings, it's unwise and will
+           * likely end up in double-free crashes.
            */
           while (object->ref_count > 1)
             g_object_unref (object);
-#else
-          g_object_unref (object);
 #endif
           g_hash_table_iter_remove (&iter);
         }
