@@ -22,9 +22,14 @@
 
 #include "gimp.h"
 
-#include "gimpsaveprocedure.h"
-#include "gimppdb_pdb.h"
+#include "libgimpbase/gimpwire.h" /* FIXME kill this include */
 
+#include "gimppdb_pdb.h"
+#include "gimpplugin-private.h"
+#include "gimpsaveprocedure.h"
+#include "gimpprocedureconfig-private.h"
+
+#include "libgimp-intl.h"
 
 enum
 {
@@ -40,16 +45,18 @@ enum
 
 struct _GimpSaveProcedurePrivate
 {
-  GimpRunSaveFunc run_func;
-  gpointer        run_data;
-  GDestroyNotify  run_data_destroy;
+  GimpRunSaveFunc  run_func;
+  gpointer         run_data;
+  GDestroyNotify   run_data_destroy;
 
-  gboolean        supports_exif;
-  gboolean        supports_iptc;
-  gboolean        supports_xmp;
-  gboolean        supports_profile;
-  gboolean        supports_thumbnail;
-  gboolean        supports_comment;
+  gboolean         supports_exif;
+  gboolean         supports_iptc;
+  gboolean         supports_xmp;
+  gboolean         supports_profile;
+  gboolean         supports_thumbnail;
+  gboolean         supports_comment;
+
+  gboolean         export_metadata;
 };
 
 
@@ -184,6 +191,8 @@ static void
 gimp_save_procedure_init (GimpSaveProcedure *procedure)
 {
   procedure->priv = gimp_save_procedure_get_instance_private (procedure);
+
+  procedure->priv->export_metadata = FALSE;
 }
 
 static void
@@ -332,15 +341,19 @@ static GimpValueArray *
 gimp_save_procedure_run (GimpProcedure        *procedure,
                          const GimpValueArray *args)
 {
-  GimpSaveProcedure *save_proc = GIMP_SAVE_PROCEDURE (procedure);
-  GimpValueArray    *remaining;
-  GimpValueArray    *return_values;
-  GimpRunMode        run_mode;
-  GimpImage         *image;
-  GimpDrawable     **drawables;
-  GFile             *file;
-  gint               n_drawables;
-  gint               i;
+  GimpPlugIn           *plug_in;
+  GimpSaveProcedure    *save_proc = GIMP_SAVE_PROCEDURE (procedure);
+  GimpValueArray       *remaining;
+  GimpValueArray       *return_values;
+  GimpProcedureConfig  *config;
+  GimpRunMode           run_mode;
+  GimpImage            *image;
+  GimpDrawable        **drawables;
+  GFile                *file;
+  gint                  n_drawables;
+  GimpMetadata         *metadata;
+  gchar                *mimetype = NULL;
+  GimpPDBStatusType     status   = GIMP_PDB_EXECUTION_ERROR;
 
   run_mode    = GIMP_VALUES_GET_ENUM         (args, 0);
   image       = GIMP_VALUES_GET_IMAGE        (args, 1);
@@ -350,21 +363,68 @@ gimp_save_procedure_run (GimpProcedure        *procedure,
 
   remaining = gimp_value_array_new (gimp_value_array_length (args) - ARG_OFFSET);
 
-  for (i = ARG_OFFSET; i < gimp_value_array_length (args); i++)
+  for (gint i = ARG_OFFSET; i < gimp_value_array_length (args); i++)
     {
       GValue *value = gimp_value_array_index (args, i);
 
       gimp_value_array_append (remaining, value);
     }
 
-  return_values = save_proc->priv->run_func (procedure,
-                                             run_mode,
-                                             image,
-                                             n_drawables,
-                                             drawables,
-                                             file,
-                                             remaining,
+  config = gimp_procedure_create_config (procedure);
+  if (save_proc->priv->export_metadata)
+    {
+      mimetype = (gchar *) gimp_file_procedure_get_mime_types (GIMP_FILE_PROCEDURE (procedure));
+      if (mimetype == NULL)
+        {
+          g_printerr ("%s: ERROR: the GimpSaveProcedureConfig object was created "
+                      "with export_metadata but you didn't set any MimeType with gimp_file_procedure_set_mime_types()!\n",
+                      G_STRFUNC);
+        }
+      else
+        {
+          char *delim;
+
+          mimetype = g_strdup (mimetype);
+          mimetype = g_strstrip (mimetype);
+          delim = strstr (mimetype, ",");
+          if (delim)
+            *delim = '\0';
+          /* Though docs only writes about the list being comma-separated, our
+           * code apparently also split by spaces.
+           */
+          delim = strstr (mimetype, " ");
+          if (delim)
+            *delim = '\0';
+          delim = strstr (mimetype, "\t");
+          if (delim)
+            *delim = '\0';
+        }
+    }
+  metadata = _gimp_procedure_config_begin_export (config, image, run_mode, remaining, mimetype);
+  g_free (mimetype);
+
+  return_values = save_proc->priv->run_func (procedure, run_mode,
+                                             image, n_drawables, drawables,
+                                             file, metadata, config,
                                              save_proc->priv->run_data);
+
+  if (return_values != NULL                       &&
+      gimp_value_array_length (return_values) > 0 &&
+      G_VALUE_HOLDS_ENUM (gimp_value_array_index (return_values, 0)))
+    status = GIMP_VALUES_GET_ENUM (return_values, 0);
+
+  _gimp_procedure_config_end_export (config, image, file, status);
+
+  /* This is debug printing to help plug-in developers figure out best
+   * practices.
+   */
+  plug_in = gimp_procedure_get_plug_in (procedure);
+  if (G_OBJECT (config)->ref_count > 1 &&
+      _gimp_plug_in_manage_memory_manually (plug_in))
+    g_printerr ("%s: ERROR: the GimpSaveProcedureConfig object was refed "
+                "by plug-in, it MUST NOT do that!\n", G_STRFUNC);
+
+  g_object_unref (config);
   gimp_value_array_unref (remaining);
 
   return return_values;
@@ -404,44 +464,44 @@ gimp_save_procedure_add_metadata (GimpSaveProcedure *save_procedure)
 
   if (save_procedure->priv->supports_exif)
     GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-exif",
-                               "Save _Exif",
-                               "Save Exif (Exchangeable image file format) metadata",
+                               _("Save _Exif"),
+                               _("Save Exif (Exchangeable image file format) metadata"),
                                gimp_export_exif (),
                                G_PARAM_READWRITE);
   if (save_procedure->priv->supports_iptc)
     GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-iptc",
-                               "Save _IPTC",
-                               "Save IPTC (International Press Telecommunications Council) metadata",
+                               _("Save _IPTC"),
+                               _("Save IPTC (International Press Telecommunications Council) metadata"),
                                gimp_export_iptc (),
                                G_PARAM_READWRITE);
   if (save_procedure->priv->supports_xmp)
     GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-xmp",
-                               "Save _XMP",
-                               "Save XMP (Extensible Metadata Platform) metadata",
+                               _("Save _XMP"),
+                               _("Save XMP (Extensible Metadata Platform) metadata"),
                                gimp_export_xmp (),
                                G_PARAM_READWRITE);
   if (save_procedure->priv->supports_profile)
     GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-color-profile",
-                               "Save color _profile",
-                               "Save the ICC color profile as metadata",
+                               _("Save color _profile"),
+                               _("Save the ICC color profile as metadata"),
                                gimp_export_color_profile (),
                                G_PARAM_READWRITE);
   if (save_procedure->priv->supports_thumbnail)
     GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-thumbnail",
-                               "Save _thumbnail",
-                               "Save a smaller representation of the image as metadata",
+                               _("Save _thumbnail"),
+                               _("Save a smaller representation of the image as metadata"),
                                gimp_export_thumbnail (),
                                G_PARAM_READWRITE);
   if (save_procedure->priv->supports_comment)
     {
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "save-comment",
-                                 "Save c_omment",
-                                 "Save a comment as metadata",
+                                 _("Save c_omment"),
+                                 _("Save a comment as metadata"),
                                  gimp_export_comment (),
                                  G_PARAM_READWRITE);
       GIMP_PROC_AUX_ARG_STRING (procedure, "gimp-comment",
-                                "Comment",
-                                "Image comment",
+                                _("Comment"),
+                                _("Image comment"),
                                 gimp_get_default_comment (),
                                 G_PARAM_READWRITE);
 
@@ -460,6 +520,7 @@ gimp_save_procedure_add_metadata (GimpSaveProcedure *save_procedure)
  * @plug_in:          a #GimpPlugIn.
  * @name:             the new procedure's name.
  * @proc_type:        the new procedure's #GimpPDBProcType.
+ * @export_metadata:  whether GIMP should handle metadata exporting.
  * @run_func:         the run function for the new procedure.
  * @run_data:         user data passed to @run_func.
  * @run_data_destroy: (nullable): free function for @run_data, or %NULL.
@@ -481,17 +542,22 @@ gimp_save_procedure_add_metadata (GimpSaveProcedure *save_procedure)
  *
  * When invoked via gimp_procedure_run(), it unpacks these standard
  * arguments and calls @run_func which is a #GimpRunSaveFunc. The
- * "args" #GimpValueArray of #GimpRunSaveFunc only contains
- * additionally added arguments.
+ * #GimpProcedureConfig of #GimpRunSaveFunc only contains additionally added
+ * arguments.
+ *
+ * If @export_metadata is TRUE, then the class will also handle the metadata
+ * export if the format is supported by our backend. This requires you to also
+ * set appropriate MimeType with gimp_file_procedure_set_mime_types().
  *
  * Returns: a new #GimpProcedure.
  *
  * Since: 3.0
  **/
-GimpProcedure  *
+GimpProcedure *
 gimp_save_procedure_new (GimpPlugIn      *plug_in,
                          const gchar     *name,
                          GimpPDBProcType  proc_type,
+                         gboolean         export_metadata,
                          GimpRunSaveFunc  run_func,
                          gpointer         run_data,
                          GDestroyNotify   run_data_destroy)
@@ -511,6 +577,7 @@ gimp_save_procedure_new (GimpPlugIn      *plug_in,
                             NULL);
 
   procedure->priv->run_func         = run_func;
+  procedure->priv->export_metadata  = export_metadata;
   procedure->priv->run_data         = run_data;
   procedure->priv->run_data_destroy = run_data_destroy;
 
