@@ -258,7 +258,6 @@ gimp_save_procedure_dialog_fill_list (GimpProcedureDialog *dialog,
       for (iter = save_dialog->priv->additional_metadata; iter; iter = iter->next)
         {
           widget = gimp_procedure_dialog_get_widget (dialog, iter->data, G_TYPE_NONE);
-          g_object_ref (widget);
           gtk_grid_attach (GTK_GRID (grid), widget, left, top, 6 / n_metadata, 1);
           left += 6 / n_metadata;
           if (left >= 6)
@@ -329,11 +328,13 @@ static gpointer
 gimp_save_procedure_dialog_edit_metadata_thread (gpointer data)
 {
   GimpSaveProcedureDialog *dialog = data;
+  GimpProcedure           *procedure;
 
-  gimp_pdb_run_procedure (gimp_get_pdb (),    "plug-in-metadata-editor",
-                          GIMP_TYPE_RUN_MODE, GIMP_RUN_INTERACTIVE,
-                          GIMP_TYPE_IMAGE,    dialog->priv->image,
-                          G_TYPE_NONE);
+  procedure = gimp_pdb_lookup_procedure (gimp_get_pdb (), "plug-in-metadata-editor");
+  gimp_procedure_run (procedure,
+                      "run-mode", GIMP_RUN_INTERACTIVE,
+                      "image",    dialog->priv->image,
+                      NULL);
 
   g_mutex_lock (&dialog->priv->metadata_thread_mutex);
   g_thread_unref (dialog->priv->metadata_thread);
