@@ -16,11 +16,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpcolor/gimpcolor.h"
@@ -34,12 +35,10 @@
 #include "gimpcolorselect.h"
 #include "gimpcolorselection.h"
 #include "gimphelpui.h"
-#include "gimpstock.h"
+#include "gimpicons.h"
 #include "gimpwidgets.h"
 #include "gimpwidgets-private.h"
 
-#include "gimpwidgetsmarshal.h"
-
 #include "libgimp/libgimp-intl.h"
 
 
@@ -81,6 +80,27 @@ enum
 };
 
 
+struct _GimpColorSelectionPrivate
+{
+  gboolean                  show_alpha;
+
+  GeglColor                *color;
+  GimpColorSelectorChannel  channel;
+
+  GtkWidget                *left_vbox;
+  GtkWidget                *right_vbox;
+
+  GtkWidget                *notebook;
+  GtkWidget                *scales;
+
+  GtkWidget                *new_color;
+  GtkWidget                *old_color;
+};
+
+#define GET_PRIVATE(obj) (((GimpColorSelection *) (obj))->priv)
+
+
+static void   gimp_color_selection_finalize          (GObject            *object);
 static void   gimp_color_selection_set_property      (GObject            *object,
                                                       guint               property_id,
                                                       const GValue       *value,
@@ -91,18 +111,14 @@ static void   gimp_color_selection_switch_page       (GtkWidget          *widget
                                                       guint               page_num,
                                                       GimpColorSelection *selection);
 static void   gimp_color_selection_notebook_changed  (GimpColorSelector  *selector,
-                                                      const GimpRGB      *rgb,
-                                                      const GimpHSV      *hsv,
+                                                      GeglColor          *color,
                                                       GimpColorSelection *selection);
 static void   gimp_color_selection_scales_changed    (GimpColorSelector  *selector,
-                                                      const GimpRGB      *rgb,
-                                                      const GimpHSV      *hsv,
+                                                      GeglColor          *color,
                                                       GimpColorSelection *selection);
-#ifndef GDK_WINDOWING_QUARTZ
 static void   gimp_color_selection_color_picked      (GtkWidget          *widget,
                                                       const GimpRGB      *rgb,
                                                       GimpColorSelection *selection);
-#endif
 static void   gimp_color_selection_entry_changed     (GimpColorHexEntry  *entry,
                                                       GimpColorSelection *selection);
 static void   gimp_color_selection_channel_changed   (GimpColorSelector  *selector,
@@ -115,11 +131,12 @@ static void   gimp_color_selection_update            (GimpColorSelection *select
                                                       UpdateType          update);
 
 
-G_DEFINE_TYPE (GimpColorSelection, gimp_color_selection, GTK_TYPE_BOX)
+G_DEFINE_TYPE_WITH_PRIVATE (GimpColorSelection, gimp_color_selection,
+                            GTK_TYPE_BOX)
 
 #define parent_class gimp_color_selection_parent_class
 
-static guint selection_signals[LAST_SIGNAL] = { 0 };
+static guint selection_signals[LAST_SIGNAL] = { 0, };
 
 
 static void
@@ -127,13 +144,15 @@ gimp_color_selection_class_init (GimpColorSelectionClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->finalize     = gimp_color_selection_finalize;
   object_class->set_property = gimp_color_selection_set_property;
 
   klass->color_changed       = NULL;
 
   g_object_class_install_property (object_class, PROP_CONFIG,
                                    g_param_spec_object ("config",
-                                                        NULL, NULL,
+                                                        "Config",
+                                                        "The color config used by this color selection",
                                                         GIMP_TYPE_COLOR_CONFIG,
                                                         G_PARAM_WRITABLE));
 
@@ -142,44 +161,47 @@ gimp_color_selection_class_init (GimpColorSelectionClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpColorSelectionClass, color_changed),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
+  gtk_widget_class_set_css_name (GTK_WIDGET_CLASS (klass), "GimpColorSelection");
 }
 
 static void
 gimp_color_selection_init (GimpColorSelection *selection)
 {
-  GtkWidget    *main_hbox;
-  GtkWidget    *hbox;
-  GtkWidget    *vbox;
-  GtkWidget    *frame;
-  GtkWidget    *label;
-  GtkWidget    *entry;
-  GtkWidget    *button;
-  GtkSizeGroup *new_group;
-  GtkSizeGroup *old_group;
+  GimpColorSelectionPrivate *priv;
+  GtkWidget                 *main_hbox;
+  GtkWidget                 *hbox;
+  GtkWidget                 *vbox;
+  GtkWidget                 *frame;
+  GtkWidget                 *label;
+  GtkWidget                 *entry;
+  GtkWidget                 *button;
+  GtkSizeGroup              *new_group;
+  GtkSizeGroup              *old_group;
 
-  selection->show_alpha = TRUE;
+  selection->priv = gimp_color_selection_get_instance_private (selection);
+
+  priv = selection->priv;
+
+  priv->show_alpha = TRUE;
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (selection),
                                   GTK_ORIENTATION_VERTICAL);
 
-  gimp_rgba_set (&selection->rgb, 0.0, 0.0, 0.0, 1.0);
-  gimp_rgb_to_hsv (&selection->rgb, &selection->hsv);
-
-  selection->channel = GIMP_COLOR_SELECTOR_HUE;
+  priv->color   = gegl_color_new ("black");
+  priv->channel = GIMP_COLOR_SELECTOR_RED;
 
   main_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_box_pack_start (GTK_BOX (selection), main_hbox, TRUE, TRUE, 0);
   gtk_widget_show (main_hbox);
 
   /*  The left vbox with the notebook  */
-  selection->left_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_box_pack_start (GTK_BOX (main_hbox), selection->left_vbox,
+  priv->left_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_pack_start (GTK_BOX (main_hbox), priv->left_vbox,
                       TRUE, TRUE, 0);
-  gtk_widget_show (selection->left_vbox);
+  gtk_widget_show (priv->left_vbox);
 
   if (_gimp_ensure_modules_func)
     {
@@ -187,31 +209,28 @@ gimp_color_selection_init (GimpColorSelection *selection)
       _gimp_ensure_modules_func ();
     }
 
-  selection->notebook = gimp_color_selector_new (GIMP_TYPE_COLOR_NOTEBOOK,
-                                                 &selection->rgb,
-                                                 &selection->hsv,
-                                                 selection->channel);
+  priv->notebook = gimp_color_selector_new (GIMP_TYPE_COLOR_NOTEBOOK, priv->color, priv->channel);
 
   if (_gimp_ensure_modules_func)
     g_type_class_unref (g_type_class_peek (GIMP_TYPE_COLOR_SELECT));
 
   gimp_color_selector_set_toggles_visible
-    (GIMP_COLOR_SELECTOR (selection->notebook), FALSE);
-  gtk_box_pack_start (GTK_BOX (selection->left_vbox), selection->notebook,
+    (GIMP_COLOR_SELECTOR (priv->notebook), FALSE);
+  gtk_box_pack_start (GTK_BOX (priv->left_vbox), priv->notebook,
                       TRUE, TRUE, 0);
-  gtk_widget_show (selection->notebook);
+  gtk_widget_show (priv->notebook);
 
-  g_signal_connect (selection->notebook, "color-changed",
+  g_signal_connect (priv->notebook, "color-changed",
                     G_CALLBACK (gimp_color_selection_notebook_changed),
                     selection);
-  g_signal_connect (GIMP_COLOR_NOTEBOOK (selection->notebook)->notebook,
+  g_signal_connect (gimp_color_notebook_get_notebook (GIMP_COLOR_NOTEBOOK (priv->notebook)),
                     "switch-page",
                     G_CALLBACK (gimp_color_selection_switch_page),
                     selection);
 
   /*  The hbox for the color_areas  */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_box_pack_end (GTK_BOX (selection->left_vbox), hbox, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (priv->left_vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
   /*  The labels  */
@@ -220,7 +239,7 @@ gimp_color_selection_init (GimpColorSelection *selection)
   gtk_widget_show (vbox);
 
   label = gtk_label_new (_("Current:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0);
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
   gtk_widget_show (label);
 
@@ -229,7 +248,7 @@ gimp_color_selection_init (GimpColorSelection *selection)
   g_object_unref (new_group);
 
   label = gtk_label_new (_("Old:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0);
   gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
   gtk_widget_show (label);
 
@@ -247,61 +266,58 @@ gimp_color_selection_init (GimpColorSelection *selection)
   gtk_container_add (GTK_CONTAINER (frame), vbox);
   gtk_widget_show (vbox);
 
-  selection->new_color = gimp_color_area_new (&selection->rgb,
-                                              selection->show_alpha ?
-                                              GIMP_COLOR_AREA_SMALL_CHECKS :
-                                              GIMP_COLOR_AREA_FLAT,
-                                              GDK_BUTTON1_MASK |
-                                              GDK_BUTTON2_MASK);
-  gtk_size_group_add_widget (new_group, selection->new_color);
-  gtk_box_pack_start (GTK_BOX (vbox), selection->new_color, FALSE, FALSE, 0);
-  gtk_widget_show (selection->new_color);
+  priv->new_color = gimp_color_area_new (priv->color,
+                                         priv->show_alpha ?
+                                         GIMP_COLOR_AREA_SMALL_CHECKS :
+                                         GIMP_COLOR_AREA_FLAT,
+                                         GDK_BUTTON1_MASK |
+                                         GDK_BUTTON2_MASK);
+  gtk_size_group_add_widget (new_group, priv->new_color);
+  gtk_box_pack_start (GTK_BOX (vbox), priv->new_color, FALSE, FALSE, 0);
+  gtk_widget_show (priv->new_color);
 
-  g_signal_connect (selection->new_color, "color-changed",
+  g_signal_connect (priv->new_color, "color-changed",
                     G_CALLBACK (gimp_color_selection_new_color_changed),
                     selection);
 
-  selection->old_color = gimp_color_area_new (&selection->rgb,
-                                              selection->show_alpha ?
-                                              GIMP_COLOR_AREA_SMALL_CHECKS :
-                                              GIMP_COLOR_AREA_FLAT,
-                                              GDK_BUTTON1_MASK |
-                                              GDK_BUTTON2_MASK);
-  gtk_drag_dest_unset (selection->old_color);
-  gtk_size_group_add_widget (old_group, selection->old_color);
-  gtk_box_pack_start (GTK_BOX (vbox), selection->old_color, FALSE, FALSE, 0);
-  gtk_widget_show (selection->old_color);
+  priv->old_color = gimp_color_area_new (priv->color,
+                                         priv->show_alpha ?
+                                         GIMP_COLOR_AREA_SMALL_CHECKS :
+                                         GIMP_COLOR_AREA_FLAT,
+                                         GDK_BUTTON1_MASK |
+                                         GDK_BUTTON2_MASK);
+  gtk_drag_dest_unset (priv->old_color);
+  gtk_size_group_add_widget (old_group, priv->old_color);
+  gtk_box_pack_start (GTK_BOX (vbox), priv->old_color, FALSE, FALSE, 0);
+  gtk_widget_show (priv->old_color);
 
   /*  The right vbox with color scales  */
-  selection->right_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_box_pack_start (GTK_BOX (main_hbox), selection->right_vbox,
+  priv->right_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  gtk_box_pack_start (GTK_BOX (main_hbox), priv->right_vbox,
                       TRUE, TRUE, 0);
-  gtk_widget_show (selection->right_vbox);
+  gtk_widget_show (priv->right_vbox);
+
+  priv->scales = gimp_color_selector_new (GIMP_TYPE_COLOR_SCALES, priv->color, priv->channel);
 
-  selection->scales = gimp_color_selector_new (GIMP_TYPE_COLOR_SCALES,
-                                               &selection->rgb,
-                                               &selection->hsv,
-                                               selection->channel);
   gimp_color_selector_set_toggles_visible
-    (GIMP_COLOR_SELECTOR (selection->scales), TRUE);
-  gimp_color_selector_set_show_alpha (GIMP_COLOR_SELECTOR (selection->scales),
-                                      selection->show_alpha);
-  gtk_box_pack_start (GTK_BOX (selection->right_vbox), selection->scales,
+    (GIMP_COLOR_SELECTOR (priv->scales), TRUE);
+  gimp_color_selector_set_show_alpha (GIMP_COLOR_SELECTOR (priv->scales),
+                                      priv->show_alpha);
+  gtk_box_pack_start (GTK_BOX (priv->right_vbox), priv->scales,
                       TRUE, TRUE, 0);
-  gtk_widget_show (selection->scales);
+  gtk_widget_show (priv->scales);
 
-  g_signal_connect (selection->scales, "channel-changed",
+  g_signal_connect (priv->scales, "channel-changed",
                     G_CALLBACK (gimp_color_selection_channel_changed),
                     selection);
-  g_signal_connect (selection->scales, "color-changed",
+  g_signal_connect (priv->scales, "color-changed",
                     G_CALLBACK (gimp_color_selection_scales_changed),
                     selection);
 
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-  gtk_box_pack_start (GTK_BOX (selection->right_vbox), hbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (priv->right_vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-#ifndef GDK_WINDOWING_QUARTZ
   /*  The color picker  */
   button = gimp_pick_button_new ();
   gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
@@ -310,14 +326,9 @@ gimp_color_selection_init (GimpColorSelection *selection)
   g_signal_connect (button, "color-picked",
                     G_CALLBACK (gimp_color_selection_color_picked),
                     selection);
-#endif
 
   /* The hex triplet entry */
   entry = gimp_color_hex_entry_new ();
-  gimp_help_set_help_data (entry,
-                           _("Hexadecimal color notation as used in HTML and "
-                             "CSS.  This entry also accepts CSS color names."),
-                           NULL);
   gtk_box_pack_end (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
   gtk_widget_show (entry);
 
@@ -333,6 +344,16 @@ gimp_color_selection_init (GimpColorSelection *selection)
                     selection);
 }
 
+static void
+gimp_color_selection_finalize (GObject *object)
+{
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (object);
+
+  g_object_unref (priv->color);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gimp_color_selection_set_property (GObject      *object,
                                    guint         property_id,
@@ -359,7 +380,7 @@ gimp_color_selection_set_property (GObject      *object,
  *
  * Creates a new #GimpColorSelection widget.
  *
- * Return value: The new #GimpColorSelection widget.
+ * Returns: The new #GimpColorSelection widget.
  **/
 GtkWidget *
 gimp_color_selection_new (void)
@@ -378,23 +399,27 @@ void
 gimp_color_selection_set_show_alpha (GimpColorSelection *selection,
                                      gboolean            show_alpha)
 {
+  GimpColorSelectionPrivate *priv;
+
   g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
 
-  if (show_alpha != selection->show_alpha)
+  priv = GET_PRIVATE (selection);
+
+  if (show_alpha != priv->show_alpha)
     {
-      selection->show_alpha = show_alpha ? TRUE : FALSE;
+      priv->show_alpha = show_alpha ? TRUE : FALSE;
 
       gimp_color_selector_set_show_alpha
-        (GIMP_COLOR_SELECTOR (selection->notebook), selection->show_alpha);
+        (GIMP_COLOR_SELECTOR (priv->notebook), priv->show_alpha);
       gimp_color_selector_set_show_alpha
-        (GIMP_COLOR_SELECTOR (selection->scales), selection->show_alpha);
+        (GIMP_COLOR_SELECTOR (priv->scales), priv->show_alpha);
 
-      gimp_color_area_set_type (GIMP_COLOR_AREA (selection->new_color),
-                                selection->show_alpha ?
+      gimp_color_area_set_type (GIMP_COLOR_AREA (priv->new_color),
+                                priv->show_alpha ?
                                 GIMP_COLOR_AREA_SMALL_CHECKS :
                                 GIMP_COLOR_AREA_FLAT);
-      gimp_color_area_set_type (GIMP_COLOR_AREA (selection->old_color),
-                                selection->show_alpha ?
+      gimp_color_area_set_type (GIMP_COLOR_AREA (priv->old_color),
+                                priv->show_alpha ?
                                 GIMP_COLOR_AREA_SMALL_CHECKS :
                                 GIMP_COLOR_AREA_FLAT);
     }
@@ -406,14 +431,14 @@ gimp_color_selection_set_show_alpha (GimpColorSelection *selection,
  *
  * Returns the @selection's @show_alpha property.
  *
- * Return value: #TRUE if the #GimpColorSelection has alpha controls.
+ * Returns: %TRUE if the #GimpColorSelection has alpha controls.
  **/
 gboolean
 gimp_color_selection_get_show_alpha (GimpColorSelection *selection)
 {
   g_return_val_if_fail (GIMP_IS_COLOR_SELECTION (selection), FALSE);
 
-  return selection->show_alpha;
+  return GET_PRIVATE (selection)->show_alpha;
 }
 
 /**
@@ -425,34 +450,42 @@ gimp_color_selection_get_show_alpha (GimpColorSelection *selection)
  **/
 void
 gimp_color_selection_set_color (GimpColorSelection *selection,
-                                const GimpRGB      *color)
+                                GeglColor          *color)
 {
+  GimpColorSelectionPrivate *priv;
+  GeglColor                 *old_color;
+
   g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
-  g_return_if_fail (color != NULL);
+  g_return_if_fail (GEGL_IS_COLOR (color));
 
-  selection->rgb = *color;
-  gimp_rgb_to_hsv (&selection->rgb, &selection->hsv);
+  priv = GET_PRIVATE (selection);
 
-  gimp_color_selection_update (selection, UPDATE_ALL);
+  old_color = priv->color;
+  priv->color = gegl_color_duplicate (color);
 
-  gimp_color_selection_color_changed (selection);
+  if (! gimp_color_is_perceptually_identical (priv->color, old_color))
+    {
+      gimp_color_selection_update (selection, UPDATE_ALL);
+      gimp_color_selection_color_changed (selection);
+    }
+
+  g_object_unref (old_color);
 }
 
 /**
  * gimp_color_selection_get_color:
  * @selection: A #GimpColorSelection widget.
- * @color:     Return location for the @selection's current @color.
  *
  * This function returns the #GimpColorSelection's current color.
+ *
+ * Returns: (transfer full): the currently selected color.
  **/
-void
-gimp_color_selection_get_color (GimpColorSelection *selection,
-                                GimpRGB            *color)
+GeglColor *
+gimp_color_selection_get_color (GimpColorSelection *selection)
 {
-  g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
-  g_return_if_fail (color != NULL);
+  g_return_val_if_fail (GIMP_IS_COLOR_SELECTION (selection), NULL);
 
-  *color = selection->rgb;
+  return gegl_color_duplicate (GET_PRIVATE (selection)->color);
 }
 
 /**
@@ -464,29 +497,34 @@ gimp_color_selection_get_color (GimpColorSelection *selection,
  **/
 void
 gimp_color_selection_set_old_color (GimpColorSelection *selection,
-                                    const GimpRGB      *color)
+                                    GeglColor          *color)
 {
-  g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
-  g_return_if_fail (color != NULL);
+  GimpColorSelectionPrivate *priv;
 
-  gimp_color_area_set_color (GIMP_COLOR_AREA (selection->old_color), color);
+  g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
+  g_return_if_fail (GEGL_IS_COLOR (color));
+
+  priv = GET_PRIVATE (selection);
+
+  gimp_color_area_set_color (GIMP_COLOR_AREA (priv->old_color), color);
 }
 
 /**
  * gimp_color_selection_get_old_color:
  * @selection: A #GimpColorSelection widget.
- * @color:     Return location for the @selection's old @color.
  *
- * This function returns the #GimpColorSelection's old color.
+ * Returns: (transfer full): the old color.
  **/
-void
-gimp_color_selection_get_old_color (GimpColorSelection *selection,
-                                    GimpRGB            *color)
+GeglColor *
+gimp_color_selection_get_old_color (GimpColorSelection *selection)
 {
-  g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
-  g_return_if_fail (color != NULL);
+  GimpColorSelectionPrivate *priv;
 
-  gimp_color_area_get_color (GIMP_COLOR_AREA (selection->old_color), color);
+  g_return_val_if_fail (GIMP_IS_COLOR_SELECTION (selection), NULL);
+
+  priv = GET_PRIVATE (selection);
+
+  return gimp_color_area_get_color (GIMP_COLOR_AREA (priv->old_color));
 }
 
 /**
@@ -498,12 +536,17 @@ gimp_color_selection_get_old_color (GimpColorSelection *selection,
 void
 gimp_color_selection_reset (GimpColorSelection *selection)
 {
-  GimpRGB color;
+  GimpColorSelectionPrivate *priv;
+  GeglColor                 *color;
 
   g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
 
-  gimp_color_area_get_color (GIMP_COLOR_AREA (selection->old_color), &color);
-  gimp_color_selection_set_color (selection, &color);
+  priv = GET_PRIVATE (selection);
+
+  color = gimp_color_area_get_color (GIMP_COLOR_AREA (priv->old_color));
+  gimp_color_selection_set_color (selection, color);
+
+  g_object_unref (color);
 }
 
 /**
@@ -520,6 +563,56 @@ gimp_color_selection_color_changed (GimpColorSelection *selection)
   g_signal_emit (selection, selection_signals[COLOR_CHANGED], 0);
 }
 
+/**
+ * gimp_color_selection_set_format:
+ * @selection: A #GimpColorSelection widget.
+ * @format:    A Babl format, with space.
+ *
+ * Updates all selectors with the current format.
+ *
+ * Since: 3.0
+ */
+void
+gimp_color_selection_set_format (GimpColorSelection *selection,
+                                const Babl          *format)
+{
+  g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
+
+  gimp_color_notebook_set_format (GIMP_COLOR_NOTEBOOK (selection->priv->notebook),
+                                  format);
+  gimp_color_selector_set_format (GIMP_COLOR_SELECTOR (selection->priv->scales),
+                                  format);
+
+  g_signal_emit (selection, selection_signals[COLOR_CHANGED], 0);
+}
+
+/**
+ * gimp_color_selection_set_simulation:
+ * @selection: A #GimpColorSelection widget.
+ * @profile:   A #GimpColorProfile object.
+ * @intent:    A #GimpColorRenderingIntent enum.
+ * @bpc:       A gboolean.
+ *
+ * Sets the simulation options to use with this color selection.
+ *
+ * Since: 3.0
+ */
+void
+gimp_color_selection_set_simulation (GimpColorSelection *selection,
+                                     GimpColorProfile   *profile,
+                                     GimpColorRenderingIntent intent,
+                                     gboolean            bpc)
+{
+  g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
+
+  gimp_color_notebook_set_simulation (GIMP_COLOR_NOTEBOOK (selection->priv->notebook),
+                                      profile,
+                                      intent,
+                                      bpc);
+
+  g_signal_emit (selection, selection_signals[COLOR_CHANGED], 0);
+}
+
 /**
  * gimp_color_selection_set_config:
  * @selection: A #GimpColorSelection widget.
@@ -527,21 +620,63 @@ gimp_color_selection_color_changed (GimpColorSelection *selection)
  *
  * Sets the color management configuration to use with this color selection.
  *
- * Since: GIMP 2.4
+ * Since: 2.4
  */
 void
 gimp_color_selection_set_config (GimpColorSelection *selection,
                                  GimpColorConfig    *config)
 {
+  GimpColorSelectionPrivate *priv;
+
   g_return_if_fail (GIMP_IS_COLOR_SELECTION (selection));
   g_return_if_fail (config == NULL || GIMP_IS_COLOR_CONFIG (config));
 
-  gimp_color_selector_set_config (GIMP_COLOR_SELECTOR (selection->notebook),
+  priv = GET_PRIVATE (selection);
+
+  gimp_color_selector_set_config (GIMP_COLOR_SELECTOR (priv->notebook),
                                   config);
-  gimp_color_selector_set_config (GIMP_COLOR_SELECTOR (selection->scales),
+  gimp_color_selector_set_config (GIMP_COLOR_SELECTOR (priv->scales),
                                   config);
+  gimp_color_area_set_color_config (GIMP_COLOR_AREA (priv->old_color),
+                                    config);
+  gimp_color_area_set_color_config (GIMP_COLOR_AREA (priv->new_color),
+                                    config);
 }
 
+/**
+ * gimp_color_selection_get_notebook:
+ * @selection: A #GimpColorSelection widget.
+ *
+ * Returns: (transfer none): The selection's #GimpColorNotebook.
+ *
+ * Since: 3.0
+ */
+GtkWidget *
+gimp_color_selection_get_notebook (GimpColorSelection *selection)
+{
+  g_return_val_if_fail (GIMP_IS_COLOR_SELECTION (selection), NULL);
+
+  return GET_PRIVATE (selection)->notebook;
+}
+
+/**
+ * gimp_color_selection_get_right_vbox:
+ * @selection: A #GimpColorSelection widget.
+ *
+ * Returns: (transfer none) (type GtkBox): The selection's right #GtkBox which
+ *          contains the color scales.
+ *
+ * Since: 3.0
+ */
+GtkWidget *
+gimp_color_selection_get_right_vbox (GimpColorSelection *selection)
+{
+  g_return_val_if_fail (GIMP_IS_COLOR_SELECTION (selection), NULL);
+
+  return GET_PRIVATE (selection)->right_vbox;
+}
+
+
 /*  private functions  */
 
 static void
@@ -550,61 +685,82 @@ gimp_color_selection_switch_page (GtkWidget          *widget,
                                   guint               page_num,
                                   GimpColorSelection *selection)
 {
-  GimpColorNotebook *notebook = GIMP_COLOR_NOTEBOOK (selection->notebook);
-  gboolean           sensitive;
+  GimpColorSelectionPrivate *priv     = GET_PRIVATE (selection);
+  GimpColorNotebook         *notebook = GIMP_COLOR_NOTEBOOK (priv->notebook);
+  GimpColorSelector         *current;
+  gboolean                   sensitive;
 
-  sensitive =
-    (GIMP_COLOR_SELECTOR_GET_CLASS (notebook->cur_page)->set_channel != NULL);
+  current = gimp_color_notebook_get_current_selector (notebook);
+
+  sensitive = (GIMP_COLOR_SELECTOR_GET_CLASS (current)->set_channel != NULL);
 
   gimp_color_selector_set_toggles_sensitive
-    (GIMP_COLOR_SELECTOR (selection->scales), sensitive);
+    (GIMP_COLOR_SELECTOR (priv->scales), sensitive);
 }
 
 static void
 gimp_color_selection_notebook_changed (GimpColorSelector  *selector,
-                                       const GimpRGB      *rgb,
-                                       const GimpHSV      *hsv,
+                                       GeglColor          *color,
                                        GimpColorSelection *selection)
 {
-  selection->hsv = *hsv;
-  selection->rgb = *rgb;
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (selection);
+  GeglColor                 *old_color;
 
-  gimp_color_selection_update (selection,
-                               UPDATE_SCALES | UPDATE_ENTRY | UPDATE_COLOR);
-  gimp_color_selection_color_changed (selection);
+  old_color = priv->color;
+  priv->color = gegl_color_duplicate (color);
+
+  if (! gimp_color_is_perceptually_identical (priv->color, old_color))
+    {
+      gimp_color_selection_update (selection,
+                                   UPDATE_SCALES | UPDATE_ENTRY | UPDATE_COLOR);
+      gimp_color_selection_color_changed (selection);
+    }
+
+  g_object_unref (old_color);
 }
 
 static void
 gimp_color_selection_scales_changed (GimpColorSelector  *selector,
-                                     const GimpRGB      *rgb,
-                                     const GimpHSV      *hsv,
+                                     GeglColor          *color,
                                      GimpColorSelection *selection)
 {
-  selection->rgb = *rgb;
-  selection->hsv = *hsv;
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (selection);
+  GeglColor                 *old_color;
 
-  gimp_color_selection_update (selection,
-                               UPDATE_ENTRY | UPDATE_NOTEBOOK | UPDATE_COLOR);
-  gimp_color_selection_color_changed (selection);
+  old_color = priv->color;
+  priv->color = gegl_color_duplicate (color);
+
+  if (! gimp_color_is_perceptually_identical (priv->color, old_color))
+    {
+      gimp_color_selection_update (selection,
+                                   UPDATE_ENTRY | UPDATE_NOTEBOOK | UPDATE_COLOR);
+      gimp_color_selection_color_changed (selection);
+    }
+
+  g_object_unref (old_color);
 }
 
-#ifndef GDK_WINDOWING_QUARTZ
 static void
 gimp_color_selection_color_picked (GtkWidget          *widget,
                                    const GimpRGB      *rgb,
                                    GimpColorSelection *selection)
 {
-  gimp_color_selection_set_color (selection, rgb);
+  GeglColor *color = gegl_color_new (NULL);
+
+  gegl_color_set_pixel (color, babl_format ("R'G'B'A double"), rgb);
+  gimp_color_selection_set_color (selection, color);
+
+  g_object_unref (color);
 }
-#endif
 
 static void
 gimp_color_selection_entry_changed (GimpColorHexEntry  *entry,
                                     GimpColorSelection *selection)
 {
-  gimp_color_hex_entry_get_color (entry, &selection->rgb);
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (selection);
 
-  gimp_rgb_to_hsv (&selection->rgb, &selection->hsv);
+  g_object_unref (priv->color);
+  priv->color = gimp_color_hex_entry_get_color (entry);
 
   gimp_color_selection_update (selection,
                                UPDATE_NOTEBOOK | UPDATE_SCALES | UPDATE_COLOR);
@@ -616,18 +772,22 @@ gimp_color_selection_channel_changed (GimpColorSelector        *selector,
                                       GimpColorSelectorChannel  channel,
                                       GimpColorSelection       *selection)
 {
-  selection->channel = channel;
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (selection);
 
-  gimp_color_selector_set_channel (GIMP_COLOR_SELECTOR (selection->notebook),
-                                   selection->channel);
+  priv->channel = channel;
+
+  gimp_color_selector_set_channel (GIMP_COLOR_SELECTOR (priv->notebook),
+                                   priv->channel);
 }
 
 static void
 gimp_color_selection_new_color_changed (GtkWidget          *widget,
                                         GimpColorSelection *selection)
 {
-  gimp_color_area_get_color (GIMP_COLOR_AREA (widget), &selection->rgb);
-  gimp_rgb_to_hsv (&selection->rgb, &selection->hsv);
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (selection);
+
+  g_object_unref (priv->color);
+  priv->color = gimp_color_area_get_color (GIMP_COLOR_AREA (widget));
 
   gimp_color_selection_update (selection,
                                UPDATE_NOTEBOOK | UPDATE_SCALES | UPDATE_ENTRY);
@@ -638,32 +798,30 @@ static void
 gimp_color_selection_update (GimpColorSelection *selection,
                              UpdateType          update)
 {
+  GimpColorSelectionPrivate *priv = GET_PRIVATE (selection);
+
   if (update & UPDATE_NOTEBOOK)
     {
-      g_signal_handlers_block_by_func (selection->notebook,
+      g_signal_handlers_block_by_func (priv->notebook,
                                        gimp_color_selection_notebook_changed,
                                        selection);
 
-      gimp_color_selector_set_color (GIMP_COLOR_SELECTOR (selection->notebook),
-                                     &selection->rgb,
-                                     &selection->hsv);
+      gimp_color_selector_set_color (GIMP_COLOR_SELECTOR (priv->notebook), priv->color);
 
-      g_signal_handlers_unblock_by_func (selection->notebook,
+      g_signal_handlers_unblock_by_func (priv->notebook,
                                          gimp_color_selection_notebook_changed,
                                          selection);
     }
 
   if (update & UPDATE_SCALES)
     {
-      g_signal_handlers_block_by_func (selection->scales,
+      g_signal_handlers_block_by_func (priv->scales,
                                        gimp_color_selection_scales_changed,
                                        selection);
 
-      gimp_color_selector_set_color (GIMP_COLOR_SELECTOR (selection->scales),
-                                     &selection->rgb,
-                                     &selection->hsv);
+      gimp_color_selector_set_color (GIMP_COLOR_SELECTOR (priv->scales), priv->color);
 
-      g_signal_handlers_unblock_by_func (selection->scales,
+      g_signal_handlers_unblock_by_func (priv->scales,
                                          gimp_color_selection_scales_changed,
                                          selection);
     }
@@ -678,7 +836,7 @@ gimp_color_selection_update (GimpColorSelection *selection,
                                        gimp_color_selection_entry_changed,
                                        selection);
 
-      gimp_color_hex_entry_set_color (entry, &selection->rgb);
+      gimp_color_hex_entry_set_color (entry, priv->color);
 
       g_signal_handlers_unblock_by_func (entry,
                                          gimp_color_selection_entry_changed,
@@ -687,14 +845,13 @@ gimp_color_selection_update (GimpColorSelection *selection,
 
   if (update & UPDATE_COLOR)
     {
-      g_signal_handlers_block_by_func (selection->new_color,
+      g_signal_handlers_block_by_func (priv->new_color,
                                        gimp_color_selection_new_color_changed,
                                        selection);
 
-      gimp_color_area_set_color (GIMP_COLOR_AREA (selection->new_color),
-                                 &selection->rgb);
+      gimp_color_area_set_color (GIMP_COLOR_AREA (priv->new_color), priv->color);
 
-      g_signal_handlers_unblock_by_func (selection->new_color,
+      g_signal_handlers_unblock_by_func (priv->new_color,
                                          gimp_color_selection_new_color_changed,
                                          selection);
     }
