@@ -21,6 +21,7 @@
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpbase/gimpbase.h"
@@ -29,9 +30,18 @@
 
 #include "gimpdialog.h"
 #include "gimphelpui.h"
+#include "gimpwidgetsutils.h"
 
 #include "libgimp/libgimp-intl.h"
 
+#ifdef G_OS_WIN32
+#include <dwmapi.h>
+#include <gdk/gdkwin32.h>
+
+#ifndef DWMWA_USE_IMMERSIVE_DARK_MODE
+#define DWMWA_USE_IMMERSIVE_DARK_MODE 20
+#endif
+#endif
 
 /**
  * SECTION: gimpdialog
@@ -58,32 +68,37 @@ struct _GimpDialogPrivate
   GimpHelpFunc  help_func;
   gchar        *help_id;
   GtkWidget    *help_button;
+
+  GBytes       *window_handle;
 };
 
 #define GET_PRIVATE(obj) (((GimpDialog *) (obj))->priv)
 
 
-static void       gimp_dialog_constructed  (GObject      *object);
-static void       gimp_dialog_dispose      (GObject      *object);
-static void       gimp_dialog_finalize     (GObject      *object);
-static void       gimp_dialog_set_property (GObject      *object,
-                                            guint         property_id,
-                                            const GValue *value,
-                                            GParamSpec   *pspec);
-static void       gimp_dialog_get_property (GObject      *object,
-                                            guint         property_id,
-                                            GValue       *value,
-                                            GParamSpec   *pspec);
+static void       gimp_dialog_constructed         (GObject      *object);
+static void       gimp_dialog_dispose             (GObject      *object);
+static void       gimp_dialog_finalize            (GObject      *object);
+static void       gimp_dialog_set_property        (GObject      *object,
+                                                   guint         property_id,
+                                                   const GValue *value,
+                                                   GParamSpec   *pspec);
+static void       gimp_dialog_get_property        (GObject      *object,
+                                                   guint         property_id,
+                                                   GValue       *value,
+                                                   GParamSpec   *pspec);
 
-static void       gimp_dialog_hide         (GtkWidget    *widget);
-static gboolean   gimp_dialog_delete_event (GtkWidget    *widget,
-                                            GdkEventAny  *event);
+static void       gimp_dialog_hide                (GtkWidget    *widget);
+static gboolean   gimp_dialog_delete_event        (GtkWidget    *widget,
+                                                   GdkEventAny  *event);
 
-static void       gimp_dialog_close        (GtkDialog    *dialog);
+static void       gimp_dialog_close               (GtkDialog    *dialog);
 
-static void       gimp_dialog_response     (GtkDialog    *dialog,
-                                            gint          response_id);
+static void       gimp_dialog_response            (GtkDialog    *dialog,
+                                                   gint          response_id);
 
+#ifdef G_OS_WIN32
+static void       gimp_dialog_set_title_bar_theme (GtkWidget    *dialog);
+#endif
 
 G_DEFINE_TYPE_WITH_PRIVATE (GimpDialog, gimp_dialog, GTK_TYPE_DIALOG)
 
@@ -157,6 +172,12 @@ gimp_dialog_init (GimpDialog *dialog)
   g_signal_connect (dialog, "response",
                     G_CALLBACK (gimp_dialog_response),
                     NULL);
+
+#ifdef G_OS_WIN32
+  g_signal_connect (GTK_WIDGET (dialog), "map",
+                    G_CALLBACK (gimp_dialog_set_title_bar_theme),
+                    NULL);
+#endif
 }
 
 static void
@@ -174,9 +195,11 @@ gimp_dialog_constructed (GObject *object)
   if (show_help_button && private->help_func && private->help_id)
     {
       private->help_button = gtk_dialog_add_button (GTK_DIALOG (object),
-                                                    ("_Help"),
+                                                    _("_Help"),
                                                     GTK_RESPONSE_HELP);
     }
+
+  gimp_widget_set_native_handle (GTK_WIDGET (object), &private->window_handle);
 }
 
 static void
@@ -650,6 +673,10 @@ gimp_dialog_run (GimpDialog *dialog)
 
   gtk_window_present (GTK_WINDOW (dialog));
 
+#ifdef G_OS_WIN32
+  gimp_dialog_set_title_bar_theme (GTK_WIDGET (dialog));
+#endif
+
   response_handler = g_signal_connect (dialog, "response",
                                        G_CALLBACK (run_response_handler),
                                        &ri);
@@ -691,10 +718,10 @@ gimp_dialog_run (GimpDialog *dialog)
  * @n_buttons:                       The size of @order
  * @order: (array length=n_buttons): array of buttons' response ids.
  *
- * Reorder @dialog's buttons if "gtk-alternative-button-order" setting
+ * Reorder @dialog's buttons if [property@Gtk.Settings:gtk-alternative-button-order]
  * is set to TRUE. This is mostly a wrapper around the GTK function
- * gtk_dialog_set_alternative_button_order(), except it won't output a
- * deprecation warning.
+ * [method@Gtk.Dialog.set_alternative_button_order], except it won't
+ * output a deprecation warning.
  *
  * Since: 3.0
  **/
@@ -711,6 +738,26 @@ gimp_dialog_set_alternative_button_order_from_array (GimpDialog *dialog,
   G_GNUC_END_IGNORE_DEPRECATIONS;
 }
 
+/**
+ * gimp_dialog_get_native_handle:
+ * @dialog: The #GimpDialog
+ *
+ * Returns an opaque data handle representing the window in the currently
+ * running platform. You should not try to use this directly. Usually this is to
+ * be used in functions such as [func@Gimp.brushes_popup] which will allow the
+ * core process to set this [class@Dialog] as parent to the newly created popup.
+ *
+ * Returns: (transfer none): an opaque [struct@GLib.Bytes] identifying this
+ *                           window.
+ *
+ * Since: 3.0
+ **/
+GBytes *
+gimp_dialog_get_native_handle (GimpDialog *dialog)
+{
+  return dialog->priv->window_handle;
+}
+
 /**
  * gimp_dialogs_show_help_button: (skip)
  * @show: whether a help button should be added when creating a GimpDialog
@@ -724,3 +771,46 @@ gimp_dialogs_show_help_button (gboolean  show)
 {
   show_help_button = show ? TRUE : FALSE;
 }
+
+#ifdef G_OS_WIN32
+void
+gimp_dialog_set_title_bar_theme (GtkWidget *dialog)
+{
+  HWND             hwnd;
+  gboolean         use_dark_mode = FALSE;
+  GdkWindow       *window        = NULL;
+
+  window = gtk_widget_get_window (GTK_WIDGET (dialog));
+  if (window)
+    {
+      GtkStyleContext *style;
+      GdkRGBA         *color = NULL;
+
+      hwnd = (HWND) gdk_win32_window_get_handle (window);
+      /* Workaround since we don't have access to GimpGuiConfig.
+       * If the background color is below the threshold, then we're
+       * likely in dark mode.
+       */
+      style = gtk_widget_get_style_context (GTK_WIDGET (dialog));
+      gtk_style_context_get (style, gtk_style_context_get_state (style),
+                             GTK_STYLE_PROPERTY_BACKGROUND_COLOR, &color,
+                             NULL);
+      if (color)
+        {
+          if (color->red < 0.5 && color->green < 0.5 && color->blue < 0.5)
+            use_dark_mode = TRUE;
+
+          gdk_rgba_free (color);
+        }
+
+      DwmSetWindowAttribute (hwnd, DWMWA_USE_IMMERSIVE_DARK_MODE,
+                             &use_dark_mode, sizeof (use_dark_mode));
+      UpdateWindow (hwnd);
+      ShowWindow (hwnd, 5);
+
+      /* Toggle the window's visibility so the title bar change appears */
+      gdk_window_hide (window);
+      gdk_window_show (window);
+    }
+}
+#endif
