@@ -33,7 +33,7 @@ package Gimp::CodeGen::pdb;
 		     const_type      => 'guchar ',
 		     init_value      => '0',
 		     get_value_func  => '$var = g_value_get_uchar ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_GET_UVHAR ($value)',
+		     dup_value_func  => '$var = GIMP_VALUES_GET_UCHAR ($value)',
 		     set_value_func  => 'g_value_set_uchar ($value, $var)',
 		     take_value_func => 'g_value_set_uchar ($value, $var)' },
 
@@ -107,18 +107,17 @@ package Gimp::CodeGen::pdb;
 		     set_value_func  => 'gimp_value_set_float_array ($value, $var, $var_len)',
 		     take_value_func => 'gimp_value_take_float_array ($value, $var, $var_len)' },
 
-    colorarray  => { name            => 'COLORARRAY',
-		     gtype           => 'GIMP_TYPE_RGB_ARRAY',
-		     type            => 'GimpRGB *',
-		     const_type      => 'const GimpRGB *',
-		     array           => 1,
+    colorarray  => { name            => 'COLOR_ARRAY',
+		     gtype           => 'GIMP_TYPE_COLOR_ARRAY',
+		     type            => 'GeglColor **',
+		     const_type      => 'const GeglColor **',
 		     init_value      => 'NULL',
-		     in_annotate     => '(element-type GimpRGB)',
-		     out_annotate    => '(element-type GimpRGB) (transfer full)',
-		     get_value_func  => '$var = gimp_value_get_rgb_array ($value)',
-		     dup_value_func  => '$var = GIMP_VALUES_DUP_RGB_ARRAY ($value)',
-		     set_value_func  => 'gimp_value_set_rgb_array ($value, $var, $var_len)',
-		     take_value_func => 'gimp_value_take_rgb_array ($value, $var, $var_len)' },
+		     in_annotate     => '(array zero-terminated=1)',
+		     out_annotate    => '(array zero-terminated=1) (transfer full)',
+		     get_value_func  => '$var = g_value_get_boxed ($value)',
+		     dup_value_func  => '$var = gimp_color_array_copy (g_value_get_boxed (gimp_value_array_index ($value)))',
+		     set_value_func  => 'g_value_set_boxed ($value, $var)',
+		     take_value_func => 'g_value_take_boxed ($value, $var)' },
 
     imagearray  => { name            => 'IMAGEARRAY',
 		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
@@ -185,6 +184,84 @@ package Gimp::CodeGen::pdb;
 		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_VECTORS, (GObject **) $var, $var_len)',
 		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_VECTORS, (GObject **) $var, $var_len)' },
 
+    resourcearray => { name          => 'RESOURCEARRAY',
+		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     type            => 'GimpResource **',
+		     const_type      => 'const GimpResource **',
+		     array           => 1,
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpResource)',
+		     out_annotate    => '(element-type GimpResource) (transfer container)',
+		     get_value_func  => '$var = (const GimpResource **) gimp_value_get_object_array ($value)',
+		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
+		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_RESOURCE, (GObject **) $var, $var_len)',
+		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_RESOURCE, (GObject **) $var, $var_len)' },
+
+    brusharray  => { name            => 'BRUSHARRAY',
+		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     type            => 'GimpBrush **',
+		     const_type      => 'const GimpBrush **',
+		     array           => 1,
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpBrush)',
+		     out_annotate    => '(element-type GimpBrush) (transfer container)',
+		     get_value_func  => '$var = (const GimpBrush **) gimp_value_get_object_array ($value)',
+		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
+		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_BRUSH, (GObject **) $var, $var_len)',
+		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_BRUSH, (GObject **) $var, $var_len)' },
+
+    fontarray   => { name            => 'FONTARRAY',
+		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     type            => 'GimpFont **',
+		     const_type      => 'const GimpFont **',
+		     array           => 1,
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpFont)',
+		     out_annotate    => '(element-type GimpFont) (transfer container)',
+		     get_value_func  => '$var = (const GimpFont **) gimp_value_get_object_array ($value)',
+		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
+		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_FONT, (GObject **) $var, $var_len)',
+		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_FONT, (GObject **) $var, $var_len)' },
+
+    gradientarray => { name          => 'GRADIENTARRAY',
+		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     type            => 'GimpGradient **',
+		     const_type      => 'const GimpGradient **',
+		     array           => 1,
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpGradient)',
+		     out_annotate    => '(element-type GimpGradient) (transfer container)',
+		     get_value_func  => '$var = (const GimpGradient **) gimp_value_get_object_array ($value)',
+		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
+		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_GRADIENT, (GObject **) $var, $var_len)',
+		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_GRADIENT, (GObject **) $var, $var_len)' },
+
+    palettearray => { name           => 'PALETTEARRAY',
+		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     type            => 'GimpPalette **',
+		     const_type      => 'const GimpPalette **',
+		     array           => 1,
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpPalette)',
+		     out_annotate    => '(element-type GimpPalette) (transfer container)',
+		     get_value_func  => '$var = (const GimpPalette **) gimp_value_get_object_array ($value)',
+		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
+		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_PALETTE, (GObject **) $var, $var_len)',
+		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_PALETTE, (GObject **) $var, $var_len)' },
+
+    patternarray => { name           => 'PATTERNARRAY',
+		     gtype           => 'GIMP_TYPE_OBJECT_ARRAY',
+		     type            => 'GimpPattern **',
+		     const_type      => 'const GimpPattern **',
+		     array           => 1,
+		     init_value      => 'NULL',
+		     in_annotate     => '(element-type GimpPattern)',
+		     out_annotate    => '(element-type GimpPattern) (transfer container)',
+		     get_value_func  => '$var = (const GimpPattern **) gimp_value_get_object_array ($value)',
+		     dup_value_func  => '{ GimpObjectArray *a = g_value_get_boxed (gimp_value_array_index ($value)); if (a) $var = g_memdup2 (a->data, a->length * sizeof (gpointer)); }',
+		     set_value_func  => 'gimp_value_set_object_array ($value, GIMP_TYPE_PATTERN, (GObject **) $var, $var_len)',
+		     take_value_func => 'gimp_value_take_object_array ($value, GIMP_TYPE_PATTERN, (GObject **) $var, $var_len)' },
+
     color       => { name            => 'COLOR',
 		     gtype           => 'GIMP_TYPE_RGB',
 		     type            => 'GimpRGB ',
@@ -197,6 +274,17 @@ package Gimp::CodeGen::pdb;
 		     take_value_func => 'gimp_value_set_rgb ($value, &$var)',
 		     headers         => [ qw(<cairo.h> "libgimpcolor/gimpcolor.h") ] },
 
+    geglcolor   => { name            => 'GEGLCOLOR',
+		     gtype           => 'GEGL_TYPE_COLOR',
+		     type            => 'GeglColor *',
+		     const_type      => 'GeglColor *',
+		     init_value      => 'NULL',
+		     out_annotate    => '(transfer full)',
+		     get_value_func  => '$var = g_value_get_object ($value)',
+		     dup_value_func  => '$var = g_value_dup_object (gimp_value_array_index ($value))',
+		     set_value_func  => 'g_value_set_object ($value, $var)',
+		     take_value_func => 'g_value_take_object ($value, $var)' },
+
     display     => { name            => 'DISPLAY',
 		     gtype           => 'GIMP_TYPE_DISPLAY',
 		     type            => 'GimpDisplay *',
@@ -419,7 +507,7 @@ package Gimp::CodeGen::pdb;
 		     type            => 'GimpResource *',
 		     const_type      => 'GimpResource *',
 		     init_value      => 'NULL',
-		     out_annotate    => '(transfer full)',
+		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_GET_RESOURCE ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
@@ -431,7 +519,7 @@ package Gimp::CodeGen::pdb;
 		     type            => 'GimpBrush *',
 		     const_type      => 'GimpBrush *',
 		     init_value      => 'NULL',
-		     out_annotate    => '(transfer full)',
+		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_GET_BRUSH ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
@@ -444,7 +532,7 @@ package Gimp::CodeGen::pdb;
 		     type            => 'GimpPattern *',
 		     const_type      => 'GimpPattern *',
 		     init_value      => 'NULL',
-		     out_annotate    => '(transfer full)',
+		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_GET_PATTERN ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
@@ -456,7 +544,7 @@ package Gimp::CodeGen::pdb;
 		     type            => 'GimpGradient *',
 		     const_type      => 'GimpGradient *',
 		     init_value      => 'NULL',
-		     out_annotate    => '(transfer full)',
+		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_GET_GRADIENT ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
@@ -468,7 +556,7 @@ package Gimp::CodeGen::pdb;
 		     type            => 'GimpPalette *',
 		     const_type      => 'GimpPalette *',
 		     init_value      => 'NULL',
-		     out_annotate    => '(transfer full)',
+		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_GET_PALETTE ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
@@ -480,7 +568,7 @@ package Gimp::CodeGen::pdb;
 		     type            => 'GimpFont *',
 		     const_type      => 'GimpFont *',
 		     init_value      => 'NULL',
-		     out_annotate    => '(transfer full)',
+		     out_annotate    => '(transfer none)',
 		     get_value_func  => '$var = g_value_get_object ($value)',
 		     dup_value_func  => '$var = GIMP_VALUES_GET_FONT ($value)',
 		     set_value_func  => 'g_value_set_object ($value, $var)',
