@@ -52,7 +52,7 @@ struct _GimpRemap
   guchar          map[256];
   gint            n_cols;
 
-  GtkBuilder     *builder;
+  GMenu          *menu;
 };
 
 
@@ -73,7 +73,7 @@ static GimpValueArray * remap_run              (GimpProcedure        *procedure,
                                                 GimpImage            *image,
                                                 gint                  n_drawables,
                                                 GimpDrawable        **drawables,
-                                                const GimpValueArray *args,
+                                                GimpProcedureConfig  *config,
                                                 gpointer              run_data);
 
 static gboolean         real_remap             (GimpImage            *image,
@@ -146,7 +146,7 @@ gimp_remap_finalize (GObject *object)
 
   G_OBJECT_CLASS (gimp_remap_parent_class)->finalize (object);
 
-  g_clear_object (&remap->builder);
+  g_clear_object (&remap->menu);
 }
 
 static void
@@ -185,15 +185,15 @@ remap_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("R_earrange Colormap..."));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_COLORMAP);
-      gimp_procedure_add_menu_path (procedure, "<Image>/Colors/Map/Colormap");
+      gimp_procedure_add_menu_path (procedure, "<Image>/Colors/Map/[Colormap]");
       gimp_procedure_add_menu_path (procedure, "<Colormap>");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Rearrange the colormap"),
-                                        "This procedure takes an indexed "
-                                        "image and lets you alter the "
-                                        "positions of colors in the colormap "
-                                        "without visually changing the image.",
+                                        _("This procedure takes an indexed "
+                                          "image and lets you alter the "
+                                          "positions of colors in the colormap "
+                                          "without visually changing the image."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Mukund Sivaraman <muks@mukund.org>",
@@ -201,8 +201,8 @@ remap_create_procedure (GimpPlugIn  *plug_in,
                                       "June 2006");
 
       GIMP_PROC_ARG_BYTES (procedure, "map",
-                           "Map",
-                           "Remap array for the colormap",
+                           _("Map"),
+                           _("Remap array for the colormap"),
                            G_PARAM_READWRITE);
     }
   else if (! strcmp (name, PLUG_IN_PROC_SWAP))
@@ -222,11 +222,11 @@ remap_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_documentation (procedure,
                                         _("Swap two colors in the colormap"),
-                                        "This procedure takes an indexed "
-                                        "image and lets you swap the "
-                                        "positions of two colors in the "
-                                        "colormap without visually changing "
-                                        "the image.",
+                                        _("This procedure takes an indexed "
+                                          "image and lets you swap the "
+                                          "positions of two colors in the "
+                                          "colormap without visually changing "
+                                          "the image."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Mukund Sivaraman <muks@mukund.org>",
@@ -234,14 +234,14 @@ remap_create_procedure (GimpPlugIn  *plug_in,
                                       "June 2006");
 
       GIMP_PROC_ARG_INT (procedure, "index1",
-                         "Index 1",
-                         "First index in the colormap",
+                         _("Index 1"),
+                         _("First index in the colormap"),
                          0, 255, 0,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "index2",
-                         "Index 2",
-                         "Second (other) index in the colormap",
+                         _("Index 2"),
+                         _("Second (other) index in the colormap"),
                          0, 255, 0,
                          G_PARAM_READWRITE);
     }
@@ -255,10 +255,11 @@ remap_run (GimpProcedure        *procedure,
            GimpImage            *image,
            gint                  n_drawables,
            GimpDrawable        **drawables,
-           const GimpValueArray *args,
+           GimpProcedureConfig  *config,
            gpointer              run_data)
 {
-  gint i;
+  GMenu *section;
+  gint   i;
 
   GimpRemap *remap = GIMP_REMAP (run_data);
 
@@ -270,36 +271,20 @@ remap_run (GimpProcedure        *procedure,
 #endif
   remap->image = image;
 
-  remap->builder = gtk_builder_new_from_string (
-      "<interface>"
-        "<menu id=\"remap-popup\">"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Sort on Hue</attribute>"
-              "<attribute name=\"action\">win.sort-hue</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Sort on Saturation</attribute>"
-              "<attribute name=\"action\">win.sort-sat</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Sort on Value</attribute>"
-              "<attribute name=\"action\">win.sort-val</attribute>"
-            "</item>"
-          "</section>"
-          "<section>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Reverse Order</attribute>"
-              "<attribute name=\"action\">win.reverse</attribute>"
-            "</item>"
-            "<item>"
-              "<attribute name=\"label\" translatable=\"yes\">Reset Order</attribute>"
-              "<attribute name=\"action\">win.reset</attribute>"
-            "</item>"
-          "</section>"
-        "</menu>"
-      "</interface>",
-      -1);
+  remap->menu = g_menu_new ();
+
+  section = g_menu_new ();
+  g_menu_append (section, _("Sort on Hue"), "win.sort-hue");
+  g_menu_append (section, _("Sort on Saturation"), "win.sort-sat");
+  g_menu_append (section, _("Sort on Value"), "win.sort-val");
+  g_menu_append_section (remap->menu, NULL, G_MENU_MODEL (section));
+  g_clear_object (&section);
+
+  section = g_menu_new ();
+  g_menu_append (section, _("Reverse Order"), "win.reverse");
+  g_menu_append (section, _("Reset Order"), "win.reset");
+  g_menu_append_section (remap->menu, NULL, G_MENU_MODEL (section));
+  g_clear_object (&section);
 
   gegl_init (NULL, NULL);
 
@@ -320,8 +305,11 @@ remap_run (GimpProcedure        *procedure,
 
       g_free (gimp_image_get_colormap (image, NULL, &remap->n_cols));
 
-      col_args_bytes = GIMP_VALUES_GET_BYTES (args, 0);
-      col_args       = g_bytes_get_data (col_args_bytes, NULL);
+      g_object_get (config,
+                    "map", &col_args_bytes,
+                    NULL);
+
+      col_args = g_bytes_get_data (col_args_bytes, NULL);
 
       switch (run_mode)
         {
@@ -347,7 +335,6 @@ remap_run (GimpProcedure        *procedure,
           break;
 
         case GIMP_RUN_WITH_LAST_VALS:
-          gimp_get_data (PLUG_IN_PROC_REMAP, remap->map);
           break;
         }
 
@@ -356,19 +343,21 @@ remap_run (GimpProcedure        *procedure,
                                                  GIMP_PDB_EXECUTION_ERROR,
                                                  NULL);
 
-      if (run_mode == GIMP_RUN_INTERACTIVE)
-        gimp_set_data (PLUG_IN_PROC_REMAP, remap->map, sizeof (remap->map));
-
       if (run_mode != GIMP_RUN_NONINTERACTIVE)
         gimp_displays_flush ();
     }
   else if (strcmp (gimp_procedure_get_name (procedure),
                    PLUG_IN_PROC_SWAP) == 0)
     {
-      gint   index1 = GIMP_VALUES_GET_INT (args, 0);
-      gint   index2 = GIMP_VALUES_GET_INT (args, 1);
       guchar tmp;
       gint   n_cols;
+      gint   index1;
+      gint   index2;
+
+      g_object_get (config,
+                    "index1", &index1,
+                    "index2", &index2,
+                    NULL);
 
       if (run_mode != GIMP_RUN_NONINTERACTIVE)
         return gimp_procedure_new_return_values (procedure,
@@ -592,23 +581,26 @@ remap_sort (GtkTreeSortable *store,
                                         GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, 0);
 }
 
-static void             remap_sort_hue_action  (GSimpleAction        *action,
-                                                GVariant             *parameter,
-                                                gpointer              user_data)
+static void
+remap_sort_hue_action  (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
 {
   remap_sort (GTK_TREE_SORTABLE (store), COLOR_H, GTK_SORT_ASCENDING);
 }
 
-static void             remap_sort_sat_action  (GSimpleAction        *action,
-                                                GVariant             *parameter,
-                                                gpointer              user_data)
+static void
+remap_sort_sat_action  (GSimpleAction *action,
+                        GVariant      *parameter,
+                        gpointer       user_data)
 {
   remap_sort (GTK_TREE_SORTABLE (store), COLOR_S, GTK_SORT_ASCENDING);
 }
 
-static void             remap_sort_val_action  (GSimpleAction        *action,
-                                                GVariant             *parameter,
-                                                gpointer              user_data)
+static void
+remap_sort_val_action  (GSimpleAction *action,
+                         GVariant     *parameter,
+                         gpointer      user_data)
 {
   remap_sort (GTK_TREE_SORTABLE (store), COLOR_V, GTK_SORT_ASCENDING);
 }
@@ -636,10 +628,8 @@ remap_popup_menu (GtkWidget      *widget,
                   GimpRemap      *remap)
 {
   GtkWidget  *menu;
-  GMenuModel *model;
 
-  model = G_MENU_MODEL (gtk_builder_get_object (remap->builder, "remap-popup"));
-  menu = gtk_menu_new_from_model (model);
+  menu = gtk_menu_new_from_model (G_MENU_MODEL (remap->menu));
 
   gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (window), NULL);
   gtk_menu_popup_at_pointer (GTK_MENU (menu), (GdkEvent *) event);
@@ -659,9 +649,9 @@ remap_button_press (GtkWidget      *widget,
 }
 
 static void
-remap_response (GtkWidget       *dialog,
-                gint             response_id,
-                GimpRemap       *remap)
+remap_response (GtkWidget *dialog,
+                gint       response_id,
+                GimpRemap *remap)
 {
   switch (response_id)
     {
@@ -707,10 +697,10 @@ remap_dialog (GimpImage *image,
                             NULL);
 
   gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                           RESPONSE_RESET,
-                                           GTK_RESPONSE_OK,
-                                           GTK_RESPONSE_CANCEL,
-                                           -1);
+                                            RESPONSE_RESET,
+                                            GTK_RESPONSE_OK,
+                                            GTK_RESPONSE_CANCEL,
+                                            -1);
 
   gimp_window_set_transient (GTK_WINDOW (dialog));
 
