@@ -57,46 +57,48 @@ struct _IffClass
 
 
 #define IFF_TYPE  (iff_get_type ())
-#define IFF (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), IFF_TYPE, Iff))
+#define IFF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), IFF_TYPE, Iff))
 
 GType                   iff_get_type         (void) G_GNUC_CONST;
 
 
-static GList          * iff_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * iff_create_procedure (GimpPlugIn           *plug_in,
-                                              const gchar          *name);
+static GList          * iff_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * iff_create_procedure (GimpPlugIn            *plug_in,
+                                              const gchar           *name);
 
-static GimpValueArray * iff_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
+static GimpValueArray * iff_load             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpMetadataLoadFlags *flags,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
 
-static GimpImage      * load_image           (GFile                *file,
-                                              GObject              *config,
-                                              GimpRunMode           run_mode,
-                                              GError              **error);
+static GimpImage      * load_image           (GFile                 *file,
+                                              GObject               *config,
+                                              GimpRunMode            run_mode,
+                                              GError               **error);
 
-static void             deleave_indexed_row  (IFF_UByte            *bitplanes,
-                                              guchar               *pixel_row,
-                                              gint                  width,
-                                              gint                  nPlanes);
+static void             deleave_indexed_row  (IFF_UByte             *bitplanes,
+                                              guchar                *pixel_row,
+                                              gint                   width,
+                                              gint                   nPlanes);
 
-static void             deleave_rgb_row      (IFF_UByte            *bitplanes,
-                                              guchar               *pixel_row,
-                                              gint                  width,
-                                              gint                  nPlanes,
-                                              gint                  pixel_size);
+static void             deleave_rgb_row      (IFF_UByte             *bitplanes,
+                                              guchar                *pixel_row,
+                                              gint                   width,
+                                              gint                   nPlanes,
+                                              gint                   pixel_size);
 
-static void             deleave_ham_row      (const guchar         *gimp_cmap,
-                                              IFF_UByte            *bitplanes,
-                                              guchar               *pixel_row,
-                                              gint                  width,
-                                              gint                  nPlanes);
+static void             deleave_ham_row      (const guchar          *gimp_cmap,
+                                              IFF_UByte             *bitplanes,
+                                              guchar                *pixel_row,
+                                              gint                   width,
+                                              gint                   nPlanes);
 
-static void             pbm_row              (IFF_UByte            *bitplanes,
-                                              guchar               *pixel_row,
-                                              gint                  width);
+static void             pbm_row              (IFF_UByte             *bitplanes,
+                                              guchar                *pixel_row,
+                                              gint                   width);
 
 
 G_DEFINE_TYPE (Iff, iff, GIMP_TYPE_PLUG_IN)
@@ -165,36 +167,26 @@ iff_create_procedure (GimpPlugIn  *plug_in,
 }
 
 static GimpValueArray *
-iff_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+iff_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
-  GimpProcedureConfig *config;
-  GimpValueArray      *return_vals;
-  GimpImage           *image;
-  GError              *error = NULL;
+  GimpValueArray *return_vals;
+  GimpImage      *image;
+  GError         *error = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
   image = load_image (file, G_OBJECT (config), run_mode, &error);
 
   if (! image)
-    {
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_EXECUTION_ERROR,
-                                               error);
-    }
-
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_EXECUTION_ERROR,
+                                             error);
 
   return_vals = gimp_procedure_new_return_values (procedure,
                                                   GIMP_PDB_SUCCESS,
@@ -238,10 +230,10 @@ load_image (GFile        *file,
     {
       ILBM_Image        *true_image   = iff_image[i];
       /* Struct representing bitmap header properties */
-      ILBM_BitMapHeader *bitMapHeader = true_image->bitMapHeader;
+      ILBM_BitMapHeader *bitMapHeader;
       /* Struct containing the color palette */
-      ILBM_ColorMap     *colorMap     = true_image->colorMap;
-      ILBM_Viewport     *camg         = true_image->viewport;
+      ILBM_ColorMap     *colorMap;
+      ILBM_Viewport     *camg;
       IFF_UByte         *bitplanes;
       GimpImageType      image_type;
       guchar             gimp_cmap[768];  /* Max index is (2^nplanes) - 1 */
@@ -252,29 +244,38 @@ load_image (GFile        *file,
       gint               row_length;
       gint               pixel_size   = 1;
       gint               y_height     = 0;
+      gint               aspect_x     = 0;
+      gint               aspect_y     = 0;
       gboolean           ehb_mode     = FALSE;
       gboolean           ham_mode     = FALSE;
 
-      if (! true_image || ! bitMapHeader)
+      if (! true_image)
         {
           g_message (_("Invalid or missing ILBM image"));
           return image;
         }
-      if (! true_image->body)
-        {
-          g_message (_("ILBM contains no image data - likely a palette file"));
-          return NULL;
-        }
+
+      colorMap = true_image->colorMap;
+      camg     = true_image->viewport;
 
       /* Convert ACBM files to ILBM format */
       if (ILBM_imageIsACBM (true_image))
         ILBM_convertACBMToILBM (true_image);
 
+      bitMapHeader = true_image->bitMapHeader;
+      if (! bitMapHeader || ! true_image->body)
+        {
+          g_message (_("ILBM contains no image data - likely a palette file"));
+          return NULL;
+        }
+
       width      = bitMapHeader->w;
       height     = bitMapHeader->h;
       nPlanes    = bitMapHeader->nPlanes;
       row_length = (width + 15) / 16;
       pixel_size = nPlanes / 8;
+      aspect_x   = bitMapHeader->xAspect;
+      aspect_y   = bitMapHeader->yAspect;
 
       /* Check for ILBM variants in CMAG chunk */
       if (camg)
@@ -337,8 +338,9 @@ load_image (GFile        *file,
 
       ILBM_unpackByteRun (true_image);
 
-      image = gimp_image_new (width, height,
-                              pixel_size == 1 ? GIMP_INDEXED : GIMP_RGB);
+      if (! image)
+        image = gimp_image_new (width, height,
+                                pixel_size == 1 ? GIMP_INDEXED : GIMP_RGB);
 
       layer = gimp_layer_new (image, _("Background"), width, height,
                               image_type, 100,
@@ -349,6 +351,24 @@ load_image (GFile        *file,
 
       bitplanes = true_image->body->chunkData;
 
+      /* Setting resolution for non-square pixel aspect ratios */
+      if (aspect_x != aspect_y && aspect_x > 0 && aspect_y > 0)
+        {
+          gdouble image_xres;
+          gdouble image_yres;
+          gfloat  ratio = (gfloat) aspect_x / aspect_y;
+
+          g_message (_("Non-square pixels. Image might look squashed if "
+                       "Dot for Dot mode is enabled."));
+
+          gimp_image_get_resolution (image, &image_xres, &image_yres);
+          if (ratio < 1)
+            image_xres = image_yres * (1 / ratio);
+          else
+            image_yres = image_xres * ratio;
+          gimp_image_set_resolution (image, image_xres, image_yres);
+        }
+
       /* Loading rows */
       for (gint j = 0; j < height; j++)
         {
