@@ -149,7 +149,7 @@ struct _MngClass
 
 
 #define MNG_TYPE  (mng_get_type ())
-#define MNG (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MNG_TYPE, Mng))
+#define MNG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MNG_TYPE, Mng))
 
 GType                   mng_get_type             (void) G_GNUC_CONST;
 
@@ -163,7 +163,8 @@ static GimpValueArray * mng_save                 (GimpProcedure        *procedur
                                                   gint                  n_drawables,
                                                   GimpDrawable        **drawables,
                                                   GFile                *file,
-                                                  const GimpValueArray *args,
+                                                  GimpMetadata         *metadata,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static mng_ptr  MNG_DECL  myalloc       (mng_size_t  size);
@@ -247,7 +248,7 @@ mng_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           mng_save, NULL, NULL);
+                                           FALSE, mng_save, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "*");
 
@@ -361,19 +362,16 @@ mng_save (GimpProcedure        *procedure,
           gint                  n_drawables,
           GimpDrawable        **drawables,
           GFile                *file,
-          const GimpValueArray *args,
+          GimpMetadata         *metadata,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpPDBStatusType    status = GIMP_PDB_SUCCESS;
-  GimpExportReturn     export = GIMP_EXPORT_IGNORE;
-  GError              *error  = NULL;
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  GimpExportReturn   export = GIMP_EXPORT_IGNORE;
+  GError            *error  = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_export (config, image, run_mode, args, NULL);
-
   if (run_mode == GIMP_RUN_INTERACTIVE ||
       run_mode == GIMP_RUN_WITH_LAST_VALS)
     {
@@ -407,9 +405,6 @@ mng_save (GimpProcedure        *procedure,
         }
     }
 
-  gimp_procedure_config_end_export (config, image, file, status);
-  g_object_unref (config);
-
   if (export == GIMP_EXPORT_EXPORT)
     {
       gimp_image_delete (image);
@@ -1593,7 +1588,8 @@ mng_save_dialog (GimpImage     *image,
                                   "bkgd", "gama", "phys", "time",
                                   NULL);
   gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
-                                   "options-label", _("MNG Options"));
+                                   "options-label", _("MNG Options"),
+                                   FALSE, FALSE);
   gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (dialog),
                                     "options-frame", "options-label",
                                     FALSE, "options-vbox");
@@ -1639,7 +1635,7 @@ mng_save_dialog (GimpImage     *image,
   /* MNG Animation Options */
   label = gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
                                            "milliseconds-label",
-                                           _("milliseconds"));
+                                           _("milliseconds"), FALSE, FALSE);
   gtk_widget_set_margin_start (label, 6);
   hbox = gimp_procedure_dialog_fill_box (GIMP_PROCEDURE_DIALOG (dialog),
                                          "animation-box", "default-delay",
@@ -1652,7 +1648,7 @@ mng_save_dialog (GimpImage     *image,
                                   "animation-box", NULL);
   gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
                                    "mng-animation-label",
-                                   _("Animated MNG Options"));
+                                   _("Animated MNG Options"), FALSE, FALSE);
   frame = gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (dialog),
                                             "mng-animation-frame",
                                             "mng-animation-label", FALSE,
