@@ -70,7 +70,7 @@
  * May 6, 2009 Barak | Itkin <lightningismyname@gmail.com>
  *   Added new features and several bugfixes:
  *   - Added handling for image resolutions
- *   - fixed the behaviour of getting font sizes
+ *   - fixed the behavior of getting font sizes
  *   - Added various optimizations (solid rectangles instead of bitmaps, ignoring
  *     invisible layers, etc.) as a macro flag.
  *   - Added handling for layer masks, use CAIRO_FORMAT_A8 for grayscale drawables.
@@ -175,7 +175,7 @@ struct _PdfClass
 
 
 #define PDF_TYPE  (pdf_get_type ())
-#define PDF (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PDF_TYPE, Pdf))
+#define PDF(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PDF_TYPE, Pdf))
 
 GType                   pdf_get_type             (void) G_GNUC_CONST;
 
@@ -189,10 +189,11 @@ static GimpValueArray * pdf_save                 (GimpProcedure        *procedur
                                                   gint                  n_drawables,
                                                   GimpDrawable        **drawables,
                                                   GFile                *file,
-                                                  const GimpValueArray *args,
+                                                  GimpMetadata         *metadata,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 static GimpValueArray * pdf_save_multi           (GimpProcedure        *procedure,
-                                                  const GimpValueArray *args,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static GimpPDBStatusType pdf_save_image          (GimpProcedure        *procedure,
@@ -307,7 +308,7 @@ pdf_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pdf_save, NULL, NULL);
+                                           TRUE, pdf_save, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "*");
 
@@ -334,45 +335,45 @@ pdf_create_procedure (GimpPlugIn  *plug_in,
                                           "pdf");
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "vectorize",
-                             "Convert bitmaps to vector graphics where possible",
-                             "Convert bitmaps to vector graphics where possible",
+                             _("Convert bitmaps to vector graphics where possible"),
+                             _("Convert bitmaps to vector graphics where possible"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "ignore-hidden",
-                             "Omit hidden layers and layers with zero opacity",
-                             "Non-visible layers will not be exported",
+                             _("Omit hidden layers and layers with zero opacity"),
+                             _("Non-visible layers will not be exported"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "apply-masks",
-                             "Apply layer masks",
-                             "Apply layer masks before saving (Keeping the mask "
-                             "will not change the output, only the PDF structure),",
+                             _("Apply layer masks"),
+                             _("Apply layer masks before saving (Keeping the mask "
+                             "will not change the output, only the PDF structure)"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "layers-as-pages",
-                             "Layers as pages",
-                             "Layers as pages (bottom layers first).",
+                             _("Layers as pages"),
+                             _("Layers as pages (bottom layers first)."),
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "reverse-order",
-                             "Reverse order",
-                             "Reverse the pages order (top layers first).",
+                             _("Reverse order"),
+                             _("Reverse the pages order (top layers first)."),
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "root-layers-only",
-                             "Root layers only",
-                             "Only the root layers are considered pages",
+                             _("Root layers only"),
+                             _("Only the root layers are considered pages"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "convert-text-layers",
-                             "Convert text layers to image",
-                             "Convert text layers to raster graphics",
+                             _("Convert text layers to image"),
+                             _("Convert text layers to raster graphics"),
                              FALSE,
                              G_PARAM_READWRITE);
 
@@ -472,46 +473,21 @@ pdf_save (GimpProcedure        *procedure,
           gint                  n_drawables,
           GimpDrawable        **drawables,
           GFile                *file,
-          const GimpValueArray *args,
+          GimpMetadata         *metadata,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GError              *error          = NULL;
-  GimpPDBStatusType    status         = GIMP_PDB_SUCCESS;
-  gboolean             had_saved_list = FALSE;
+  GError            *error          = NULL;
+  GimpPDBStatusType  status         = GIMP_PDB_SUCCESS;
 
   gegl_init (NULL, NULL);
 
   /* Initializing all the settings */
   multi_page.image_count = 0;
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_export (config, image, run_mode,
-                                      args, "application/pdf");
   file_name = g_file_get_path (file);
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_NONINTERACTIVE:
-      init_image_list_defaults (image);
-      break;
-
-    case GIMP_RUN_INTERACTIVE:
-      /* Possibly retrieve data */
-      had_saved_list = gimp_get_data (DATA_IMAGE_LIST, &multi_page);
-
-      if (had_saved_list && (file_name == NULL || strlen (file_name) == 0))
-        {
-          file_name = multi_page.file_name;
-        }
-
-      init_image_list_defaults (image);
-      break;
-
-    case GIMP_RUN_WITH_LAST_VALS:
-      init_image_list_defaults (image);
-      break;
-    }
+  init_image_list_defaults (image);
 
   validate_image_list ();
 
@@ -526,73 +502,40 @@ pdf_save (GimpProcedure        *procedure,
                              (run_mode != GIMP_RUN_NONINTERACTIVE),
                              &error);
 
-  gimp_procedure_config_end_export (config, image, file, status);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
 static GimpValueArray *
 pdf_save_multi (GimpProcedure        *procedure,
-                const GimpValueArray *args,
+                GimpProcedureConfig  *config,
                 gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GError              *error  = NULL;
-  GimpPDBStatusType    status = GIMP_PDB_SUCCESS;
-  GimpRunMode          run_mode;
-  gchar               *uri;
-  const gint32        *image_ids;
-
-  GimpImage   *image = NULL;
-  GFile       *file;
-  gboolean     had_saved_list = FALSE;
+  GError            *error  = NULL;
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  GimpRunMode        run_mode;
+  gchar             *uri;
+  const gint32      *image_ids = NULL;
+  GimpImage         *image = NULL;
+  GFile             *file;
 
   gegl_init (NULL, NULL);
 
-  run_mode = GIMP_VALUES_GET_ENUM (args, 0);
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
   g_object_get (config,
-                "uri",    &uri,
-                "count",  &multi_page.image_count,
-                "images", &image_ids,
+                "run-mode", &run_mode,
+                "uri",      &uri,
+                "count",    &multi_page.image_count,
+                "images",   &image_ids,
                 NULL);
 
   file = g_file_new_for_uri (uri);
   g_free (uri);
   file_name = g_file_get_path (file);
 
-  switch (run_mode)
-    {
-      gint i;
-
-    case GIMP_RUN_NONINTERACTIVE:
-      if (image_ids)
-        for (i = 0; i < multi_page.image_count; i++)
-          multi_page.images[i] = gimp_image_get_by_id (image_ids[i]);
-      break;
-
-    case GIMP_RUN_INTERACTIVE:
-      /* Possibly retrieve data */
-      had_saved_list = gimp_get_data (DATA_IMAGE_LIST, &multi_page);
-
-      if (had_saved_list && (file_name == NULL || strlen (file_name) == 0))
-        {
-          file_name = multi_page.file_name;
-        }
-
-      if (! had_saved_list)
-        init_image_list_defaults (image);
-      break;
-
-    case GIMP_RUN_WITH_LAST_VALS:
-      /* Possibly retrieve data */
-      had_saved_list = gimp_get_data (DATA_IMAGE_LIST, &multi_page);
-      if (had_saved_list)
-        file_name = multi_page.file_name;
-      break;
-    }
+  if (image_ids)
+    for (gint i = 0; i < multi_page.image_count; i++)
+      multi_page.images[i] = gimp_image_get_by_id (image_ids[i]);
+  else
+    init_image_list_defaults (image);
 
   validate_image_list ();
 
@@ -608,9 +551,6 @@ pdf_save_multi (GimpProcedure        *procedure,
                              (run_mode != GIMP_RUN_NONINTERACTIVE),
                              &error);
 
-  gimp_procedure_config_end_run (config, status);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -652,7 +592,7 @@ get_missing_fonts (GList *layers)
         }
       else if (gimp_item_is_text_layer (GIMP_ITEM (layer)))
         {
-          gchar                *font_family;
+          GimpFont             *gimp_font;
           PangoFontDescription *font_description;
           PangoFontDescription *font_description2;
           PangoFontMap         *fontmap;
@@ -662,8 +602,8 @@ get_missing_fonts (GList *layers)
           fontmap = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
           context = pango_font_map_create_context (fontmap);
 
-          font_family = gimp_text_layer_get_font (GIMP_TEXT_LAYER (layer));
-          font_description = pango_font_description_from_string (font_family);
+          gimp_font        = gimp_text_layer_get_font (GIMP_TEXT_LAYER (layer));
+          font_description = gimp_font_get_pango_font_description (gimp_font);
 
           font = pango_font_map_load_font (fontmap, context, font_description);
           font_description2 = pango_font_describe (font);
@@ -680,7 +620,6 @@ get_missing_fonts (GList *layers)
           g_object_unref (font);
           pango_font_description_free (font_description);
           pango_font_description_free (font_description2);
-          g_free (font_family);
           g_object_unref (context);
           g_object_unref (fontmap);
         }
@@ -829,17 +768,18 @@ pdf_save_image (GimpProcedure        *procedure,
       if (gimp_drawable_has_alpha (GIMP_DRAWABLE (layers[n_layers - 1])) &&
           fill_background_color)
         {
-          GimpRGB color;
+          GeglColor *color;
+          double     rgb[3];
 
           cairo_rectangle (cr, 0.0, 0.0,
                            gimp_image_get_width  (image),
                            gimp_image_get_height (image));
-          gimp_context_get_background (&color);
-          cairo_set_source_rgb (cr,
-                                color.r,
-                                color.g,
-                                color.b);
+          color = gimp_context_get_background ();
+          gegl_color_get_pixel (color, babl_format_with_space ("R'G'B'A double", NULL), rgb);
+          cairo_set_source_rgb (cr, rgb[0], rgb[1], rgb[2]);
           cairo_fill (cr);
+
+          g_object_unref (color);
         }
 
       /* Now, we should loop over the layers of each image */
@@ -888,12 +828,6 @@ pdf_save_image (GimpProcedure        *procedure,
 
   fclose (fp);
 
-  if (! single_image)
-    {
-      g_strlcpy (multi_page.file_name, file_name, MAX_FILE_NAME_LENGTH);
-      gimp_set_data (DATA_IMAGE_LIST, &multi_page, sizeof (multi_page));
-    }
-
   return GIMP_PDB_SUCCESS;
 }
 
@@ -949,12 +883,13 @@ gui_single (GimpProcedure       *procedure,
             GimpProcedureConfig *config,
             GimpImage           *image)
 {
-  GtkWidget *window;
-  GtkWidget *widget;
-  GList     *missing_fonts;
-  GList     *dialog_props = NULL;
-  gboolean   run;
-  gint32     n_layers;
+  GtkWidget  *window;
+  GtkWidget  *widget;
+  GimpLayer **layers;
+  GList      *missing_fonts;
+  GList      *dialog_props = NULL;
+  gboolean    run;
+  gint32      n_layers;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
@@ -973,7 +908,13 @@ gui_single (GimpProcedure       *procedure,
   widget = gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (window),
                                             "pages-frame", "layers-as-pages", FALSE,
                                             "pages-box");
-  g_free (gimp_image_get_layers (multi_page.images[0], &n_layers));
+  /* Enable "layers-as-pages" if more than one layer, or there's a single
+   * layer group has more than one layer */
+  layers = gimp_image_get_layers (multi_page.images[0], &n_layers);
+  if (n_layers == 1 && gimp_item_is_group (GIMP_ITEM (layers[0])))
+    g_free (gimp_item_get_children (GIMP_ITEM (layers[0]), &n_layers));
+  g_free (layers);
+
   gtk_widget_set_sensitive (widget, n_layers > 1);
 
   /* Warning for missing fonts (non-embeddable with rasterization
@@ -1013,8 +954,8 @@ gui_single (GimpProcedure       *procedure,
        * the warning more obvious.
        */
       widget = gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (window),
-                                       "missing-fonts-label",
-                                       text);
+                                                "missing-fonts-label",
+                                                text, FALSE, FALSE);
       gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
       gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (window),
                                         "convert-text-layers-frame",
@@ -1497,7 +1438,7 @@ get_cairo_surface (GimpDrawable  *drawable,
       return NULL;
     }
 
-  dest_buffer = gimp_cairo_surface_create_buffer (surface);
+  dest_buffer = gimp_cairo_surface_create_buffer (surface, NULL);
   if (as_mask)
     {
       /* src_buffer represents a mask in "Y u8", "Y u16", etc. formats.
@@ -1610,12 +1551,12 @@ drawText (GimpLayer *layer,
   GimpImageType         type   = gimp_drawable_type (GIMP_DRAWABLE (layer));
   gchar                *text   = gimp_text_layer_get_text (GIMP_TEXT_LAYER (layer));
   gchar                *markup = gimp_text_layer_get_markup (GIMP_TEXT_LAYER (layer));
-  gchar                *font_family;
   gchar                *language;
   cairo_font_options_t *options;
   gint                  x;
   gint                  y;
-  GimpRGB               color;
+  GeglColor            *color;
+  gdouble               rgb[3];
   GimpUnit              unit;
   gdouble               size;
   GimpTextHintStyle     hinting;
@@ -1625,6 +1566,7 @@ drawText (GimpLayer *layer,
   GimpTextDirection     dir;
   PangoLayout          *layout;
   PangoContext         *context;
+  GimpFont             *font;
   PangoFontDescription *font_description;
   gdouble               indent;
   gdouble               line_spacing;
@@ -1647,13 +1589,18 @@ drawText (GimpLayer *layer,
   /* When dealing with a gray/indexed image, the viewed color of the text layer
    * can be different than the one kept in the memory */
   if (type == GIMP_RGBA_IMAGE)
-    gimp_text_layer_get_color (GIMP_TEXT_LAYER (layer), &color);
+    color = gimp_text_layer_get_color (GIMP_TEXT_LAYER (layer));
   else
     gimp_image_pick_color (gimp_item_get_image (GIMP_ITEM (layer)), 1,
                            (const GimpItem**) &layer, x, y, FALSE, FALSE, 0,
                            &color);
 
-  cairo_set_source_rgba (cr, color.r, color.g, color.b, opacity);
+  /* TODO: this export plug-in is not space-aware yet, so we draw everything as
+   * sRGB for the time being.
+   */
+  gegl_color_get_pixel (color, babl_format_with_space ("R'G'B' double", NULL), rgb);
+  cairo_set_source_rgba (cr, rgb[0], rgb[1], rgb[2], opacity);
+  g_object_unref (color);
 
   /* Hinting */
   hinting = gimp_text_layer_get_hint_style (GIMP_TEXT_LAYER (layer));
@@ -1687,6 +1634,15 @@ drawText (GimpLayer *layer,
    */
   fontmap = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
 
+  /* Font */
+  font = gimp_text_layer_get_font (GIMP_TEXT_LAYER (layer));
+  font_description = gimp_font_get_pango_font_description (font);
+
+  /* This function breaks rendering with some fonts if it's called before
+   * gimp_font_get_pango_font_description(). I'm still unsure why yet it
+   * probably means there is a bug somewhere we must fix. Until then, let's make
+   * sure we keep this order. XXX
+   */
   pango_cairo_font_map_set_resolution (PANGO_CAIRO_FONT_MAP (fontmap), y_res);
 
   context = pango_font_map_create_context (fontmap);
@@ -1698,7 +1654,7 @@ drawText (GimpLayer *layer,
   language = gimp_text_layer_get_language (GIMP_TEXT_LAYER (layer));
   if (language)
     pango_context_set_language (context,
-                                pango_language_from_string(language));
+                                pango_language_from_string (language));
 
   /* Text Direction */
   dir = gimp_text_layer_get_base_direction (GIMP_TEXT_LAYER (layer));
@@ -1748,14 +1704,6 @@ drawText (GimpLayer *layer,
   layout = pango_layout_new (context);
   pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
 
-  /* Font */
-  font_family = gimp_text_layer_get_font (GIMP_TEXT_LAYER (layer));
-
-  /* We need to find a way to convert GIMP's returned font name to a
-   * normal Pango name... Hopefully GIMP 2.8 with Pango will fix it.
-   */
-  font_description = pango_font_description_from_string (font_family);
-
   /* Font Size */
   size = gimp_text_layer_get_font_size (GIMP_TEXT_LAYER (layer), &unit);
   size = gimp_units_to_pixels (size, unit, y_res);
@@ -1836,7 +1784,6 @@ drawText (GimpLayer *layer,
   pango_cairo_show_layout (cr, layout);
 
   g_free (text);
-  g_free (font_family);
   g_free (language);
 
   g_object_unref (layout);
