@@ -136,101 +136,104 @@ struct _PnmClass
 
 
 #define PNM_TYPE  (pnm_get_type ())
-#define PNM (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PNM_TYPE, Pnm))
+#define PNM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), PNM_TYPE, Pnm))
 
 GType                   pnm_get_type         (void) G_GNUC_CONST;
 
-static GList          * pnm_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * pnm_create_procedure (GimpPlugIn           *plug_in,
-                                              const gchar          *name);
+static GList          * pnm_query_procedures (GimpPlugIn             *plug_in);
+static GimpProcedure  * pnm_create_procedure (GimpPlugIn             *plug_in,
+                                              const gchar            *name);
 
-static GimpValueArray * pnm_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
-static GimpValueArray * pnm_save             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GimpImage            *image,
-                                              gint                  n_drawables,
-                                              GimpDrawable        **drawables,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
+static GimpValueArray * pnm_load             (GimpProcedure          *procedure,
+                                              GimpRunMode             run_mode,
+                                              GFile                  *file,
+                                              GimpMetadata           *metadata,
+                                              GimpMetadataLoadFlags  *flags,
+                                              GimpProcedureConfig    *config,
+                                              gpointer                run_data);
+static GimpValueArray * pnm_save             (GimpProcedure          *procedure,
+                                              GimpRunMode             run_mode,
+                                              GimpImage              *image,
+                                              gint                    n_drawables,
+                                              GimpDrawable          **drawables,
+                                              GFile                  *file,
+                                              GimpMetadata           *metadata,
+                                              GimpProcedureConfig    *config,
+                                              gpointer                run_data);
 
-static GimpImage      * load_image           (GFile                *file,
-                                              GError              **error);
-static gint             save_image           (GFile                *file,
-                                              GimpImage            *image,
-                                              GimpDrawable         *drawable,
-                                              FileType              file_type,
-                                              GObject              *config,
-                                              GError              **error);
+static GimpImage      * load_image           (GFile                  *file,
+                                              GError                **error);
+static gint             save_image           (GFile                  *file,
+                                              GimpImage              *image,
+                                              GimpDrawable           *drawable,
+                                              FileType                file_type,
+                                              GObject                *config,
+                                              GError                **error);
 
-static gboolean         save_dialog          (GimpProcedure        *procedure,
-                                              GObject              *config,
-                                              GimpImage            *image);
+static gboolean         save_dialog          (GimpProcedure          *procedure,
+                                              GObject                *config,
+                                              GimpImage              *image);
 
-static void             process_pam_header   (PNMScanner           *scan,
-                                              PNMInfo              *info);
+static void             process_pam_header   (PNMScanner             *scan,
+                                              PNMInfo                *info);
 
-static void             pnm_load_ascii       (PNMScanner           *scan,
-                                              PNMInfo              *info,
-                                              GeglBuffer           *buffer);
-static void             pnm_load_raw         (PNMScanner           *scan,
-                                              PNMInfo              *info,
-                                              GeglBuffer           *buffer);
-static void             pnm_load_rawpbm      (PNMScanner           *scan,
-                                              PNMInfo              *info,
-                                              GeglBuffer           *buffer);
-static void             pnm_load_rawpfm      (PNMScanner           *scan,
-                                              PNMInfo              *info,
-                                              GeglBuffer           *buffer);
+static void             pnm_load_ascii       (PNMScanner             *scan,
+                                              PNMInfo                *info,
+                                              GeglBuffer             *buffer);
+static void             pnm_load_raw         (PNMScanner             *scan,
+                                              PNMInfo                *info,
+                                              GeglBuffer             *buffer);
+static void             pnm_load_rawpbm      (PNMScanner             *scan,
+                                              PNMInfo                *info,
+                                              GeglBuffer             *buffer);
+static void             pnm_load_rawpfm      (PNMScanner             *scan,
+                                              PNMInfo                *info,
+                                              GeglBuffer             *buffer);
 
-static void         create_pam_header        (const gchar         **header_string,
-                                              PNMRowInfo           *rowinfo,
-                                              PNMSaverowFunc       *saverow,
-                                              GimpImageType         drawable_type,
-                                              GeglBuffer           *buffer,
-                                              const Babl          **format,
-                                              gint                 *rowbufsize,
-                                              gint                 *np,
-                                              gchar                *comment);
+static void         create_pam_header        (const gchar           **header_string,
+                                              PNMRowInfo             *rowinfo,
+                                              PNMSaverowFunc         *saverow,
+                                              GimpImageType           drawable_type,
+                                              GeglBuffer             *buffer,
+                                              const Babl            **format,
+                                              gint                   *rowbufsize,
+                                              gint                   *np,
+                                              gchar                  *comment);
 
-static gboolean     pnmsaverow_ascii         (PNMRowInfo           *ri,
-                                              guchar               *data,
-                                              GError              **error);
-static gboolean     pnmsaverow_raw           (PNMRowInfo           *ri,
-                                              guchar               *data,
-                                              GError              **error);
-static gboolean     pnmsaverow_raw_pbm       (PNMRowInfo           *ri,
-                                              guchar               *data,
-                                              GError              **error);
-static gboolean     pnmsaverow_ascii_pbm     (PNMRowInfo           *ri,
-                                              guchar               *data,
-                                              GError              **error);
-static gboolean     pnmsaverow_ascii_indexed (PNMRowInfo           *ri,
-                                              guchar               *data,
-                                              GError              **error);
-static gboolean     pnmsaverow_raw_indexed   (PNMRowInfo           *ri,
-                                              guchar               *data,
-                                              GError              **error);
+static gboolean     pnmsaverow_ascii         (PNMRowInfo             *ri,
+                                              guchar                 *data,
+                                              GError                **error);
+static gboolean     pnmsaverow_raw           (PNMRowInfo             *ri,
+                                              guchar                 *data,
+                                              GError                **error);
+static gboolean     pnmsaverow_raw_pbm       (PNMRowInfo             *ri,
+                                              guchar                 *data,
+                                              GError                **error);
+static gboolean     pnmsaverow_ascii_pbm     (PNMRowInfo             *ri,
+                                              guchar                 *data,
+                                              GError                **error);
+static gboolean     pnmsaverow_ascii_indexed (PNMRowInfo             *ri,
+                                              guchar                 *data,
+                                              GError                **error);
+static gboolean     pnmsaverow_raw_indexed   (PNMRowInfo             *ri,
+                                              guchar                 *data,
+                                              GError                **error);
 
-static PNMScanner * pnmscanner_create        (GInputStream         *input);
-static void         pnmscanner_destroy       (PNMScanner           *s);
-static void         pnmscanner_createbuffer  (PNMScanner           *s,
-                                              gint                  bufsize);
-static void         pnmscanner_getchar       (PNMScanner           *s);
-static void         pnmscanner_eatwhitespace (PNMScanner           *s);
-static void         pnmscanner_eatinnerspace (PNMScanner           *s);
-static void         pnmscanner_gettoken      (PNMScanner           *s,
-                                              gchar                *buf,
-                                              gint                  bufsize);
-static void         pnmscanner_getsmalltoken (PNMScanner           *s,
-                                              gchar                *buf);
-static void         pnmscanner_getheaderline (PNMScanner           *s,
-                                              gchar                *buf,
-                                              gint                  bufsize);
+static PNMScanner * pnmscanner_create        (GInputStream           *input);
+static void         pnmscanner_destroy       (PNMScanner             *s);
+static void         pnmscanner_createbuffer  (PNMScanner             *s,
+                                              gint                    bufsize);
+static void         pnmscanner_getchar       (PNMScanner             *s);
+static void         pnmscanner_eatwhitespace (PNMScanner             *s);
+static void         pnmscanner_eatinnerspace (PNMScanner             *s);
+static void         pnmscanner_gettoken      (PNMScanner             *s,
+                                              gchar                  *buf,
+                                              gint                    bufsize);
+static void         pnmscanner_getsmalltoken (PNMScanner             *s,
+                                              gchar                  *buf);
+static void         pnmscanner_getheaderline (PNMScanner             *s,
+                                              gchar                  *buf,
+                                              gint                    bufsize);
 
 #define pnmscanner_eof(s)   ((s)->eof)
 #define pnmscanner_input(s) ((s)->input)
@@ -339,7 +342,7 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pnm_save,
+                                           FALSE, pnm_save,
                                            GINT_TO_POINTER (FILE_TYPE_PNM),
                                            NULL);
 
@@ -376,7 +379,7 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pnm_save,
+                                           FALSE, pnm_save,
                                            GINT_TO_POINTER (FILE_TYPE_PBM),
                                            NULL);
 
@@ -412,7 +415,7 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pnm_save,
+                                           FALSE, pnm_save,
                                            GINT_TO_POINTER (FILE_TYPE_PGM),
                                            NULL);
 
@@ -448,7 +451,7 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pnm_save,
+                                           FALSE, pnm_save,
                                            GINT_TO_POINTER (FILE_TYPE_PPM),
                                            NULL);
 
@@ -484,7 +487,7 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pnm_save,
+                                           FALSE, pnm_save,
                                            GINT_TO_POINTER (FILE_TYPE_PAM),
                                            NULL);
 
@@ -513,7 +516,7 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           pnm_save,
+                                           FALSE, pnm_save,
                                            GINT_TO_POINTER (FILE_TYPE_PFM),
                                            NULL);
 
@@ -543,11 +546,13 @@ pnm_create_procedure (GimpPlugIn  *plug_in,
 }
 
 static GimpValueArray *
-pnm_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+pnm_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -578,21 +583,18 @@ pnm_save (GimpProcedure        *procedure,
           gint                  n_drawables,
           GimpDrawable        **drawables,
           GFile                *file,
-          const GimpValueArray *args,
+          GimpMetadata         *metadata,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  FileType             file_type   = GPOINTER_TO_INT (run_data);
-  GimpPDBStatusType    status      = GIMP_PDB_SUCCESS;
-  GimpExportReturn     export      = GIMP_EXPORT_CANCEL;
-  const gchar         *format_name = NULL;
-  GError              *error       = NULL;
+  FileType           file_type   = GPOINTER_TO_INT (run_data);
+  GimpPDBStatusType  status      = GIMP_PDB_SUCCESS;
+  GimpExportReturn   export      = GIMP_EXPORT_CANCEL;
+  const gchar       *format_name = NULL;
+  GError            *error       = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_export (config, image, run_mode, args, NULL);
-
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
@@ -683,9 +685,6 @@ pnm_save (GimpProcedure        *procedure,
         }
     }
 
-  gimp_procedure_config_end_export (config, image, file, status);
-  g_object_unref (config);
-
   if (export == GIMP_EXPORT_EXPORT)
     {
       gimp_image_delete (image);
@@ -941,7 +940,7 @@ process_pam_header (PNMScanner *scan,
         {
           /* skip unknown headers but recognize xv's thumbnail format */
           CHECK_FOR_ERROR (g_ascii_isdigit (*buf), pnminfo->jmpbuf,
-                           _("PAM: Unsupported inofficial PNM variant."));
+                           _("PAM: Unsupported unofficial PNM variant."));
         }
     }
     CHECK_FOR_ERROR (pnmscanner_eof (scan), pnminfo->jmpbuf,
