@@ -48,28 +48,30 @@ struct _WbmpClass
 };
 
 #define WBMP_TYPE  (wbmp_get_type ())
-#define WBMP (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WBMP_TYPE, Wbmp))
+#define WBMP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WBMP_TYPE, Wbmp))
 
 GType                   wbmp_get_type         (void) G_GNUC_CONST;
 
-static GList          * wbmp_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * wbmp_create_procedure (GimpPlugIn           *plug_in,
-                                               const gchar          *name);
+static GList          * wbmp_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * wbmp_create_procedure (GimpPlugIn            *plug_in,
+                                               const gchar           *name);
 
-static GimpValueArray * wbmp_load             (GimpProcedure        *procedure,
-                                               GimpRunMode           run_mode,
-                                               GFile                *file,
-                                               const GimpValueArray *args,
-                                               gpointer              run_data);
+static GimpValueArray * wbmp_load             (GimpProcedure         *procedure,
+                                               GimpRunMode            run_mode,
+                                               GFile                 *file,
+                                               GimpMetadata          *metadata,
+                                               GimpMetadataLoadFlags *flags,
+                                               GimpProcedureConfig   *config,
+                                               gpointer               run_data);
 
-GimpImage             * load_image            (GFile                *file,
-                                               GError              **error);
+GimpImage             * load_image            (GFile                 *file,
+                                               GError               **error);
 
-static GimpImage      * read_image            (FILE                 *fd,
-                                               GFile                *file,
-                                               gint                  width,
-                                               gint                  height,
-                                               GError              **error);
+static GimpImage      * read_image            (FILE                  *fd,
+                                               GFile                 *file,
+                                               gint                   width,
+                                               gint                   height,
+                                               GError               **error);
 
 G_DEFINE_TYPE (Wbmp, wbmp, GIMP_TYPE_PLUG_IN)
 
@@ -134,11 +136,13 @@ wbmp_create_procedure (GimpPlugIn  *plug_in,
 }
 
 static GimpValueArray *
-wbmp_load (GimpProcedure        *procedure,
-           GimpRunMode           run_mode,
-           GFile                *file,
-           const GimpValueArray *args,
-           gpointer              run_data)
+wbmp_load (GimpProcedure         *procedure,
+           GimpRunMode            run_mode,
+           GFile                 *file,
+           GimpMetadata          *metadata,
+           GimpMetadataLoadFlags *flags,
+           GimpProcedureConfig   *config,
+           gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
