@@ -96,42 +96,45 @@ struct _XpmClass
 
 
 #define XPM_TYPE  (xpm_get_type ())
-#define XPM (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), XPM_TYPE, Xpm))
+#define XPM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), XPM_TYPE, Xpm))
 
 GType                   xpm_get_type         (void) G_GNUC_CONST;
 
-static GList          * xpm_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * xpm_create_procedure (GimpPlugIn           *plug_in,
-                                              const gchar          *name);
+static GList          * xpm_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * xpm_create_procedure (GimpPlugIn            *plug_in,
+                                              const gchar           *name);
 
-static GimpValueArray * xpm_load             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
-static GimpValueArray * xpm_save             (GimpProcedure        *procedure,
-                                              GimpRunMode           run_mode,
-                                              GimpImage            *image,
-                                              gint                  n_drawables,
-                                              GimpDrawable        **drawables,
-                                              GFile                *file,
-                                              const GimpValueArray *args,
-                                              gpointer              run_data);
+static GimpValueArray * xpm_load             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpMetadataLoadFlags *flags,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
+static GimpValueArray * xpm_save             (GimpProcedure         *procedure,
+                                              GimpRunMode            run_mode,
+                                              GimpImage             *image,
+                                              gint                   n_drawables,
+                                              GimpDrawable         **drawables,
+                                              GFile                 *file,
+                                              GimpMetadata          *metadata,
+                                              GimpProcedureConfig   *config,
+                                              gpointer               run_data);
 
-static GimpImage      * load_image           (GFile               *file,
-                                              GError              **error);
-static guchar         * parse_colors         (XpmImage             *xpm_image);
-static void             parse_image          (GimpImage            *image,
-                                              XpmImage             *xpm_image,
-                                              guchar               *cmap);
-static gboolean         save_image           (GFile                *file,
-                                              GimpImage            *image,
-                                              GimpDrawable         *drawable,
-                                              GObject              *config,
-                                              GError              **error);
-static gboolean         save_dialog          (GimpImage            *image,
-                                              GimpProcedure        *procedure,
-                                              GObject              *config);
+static GimpImage      * load_image           (GFile                 *file,
+                                              GError                **error);
+static guchar         * parse_colors         (XpmImage               *xpm_image);
+static void             parse_image          (GimpImage              *image,
+                                              XpmImage               *xpm_image,
+                                              guchar                 *cmap);
+static gboolean         save_image           (GFile                  *file,
+                                              GimpImage              *image,
+                                              GimpDrawable           *drawable,
+                                              GObject                *config,
+                                              GError                **error);
+static gboolean         save_dialog          (GimpImage              *image,
+                                              GimpProcedure          *procedure,
+                                              GObject                *config);
 
 
 G_DEFINE_TYPE (Xpm, xpm, GIMP_TYPE_PLUG_IN)
@@ -224,7 +227,7 @@ xpm_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           xpm_save, NULL, NULL);
+                                           FALSE, xpm_save, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "*");
 
@@ -266,11 +269,13 @@ xpm_create_procedure (GimpPlugIn  *plug_in,
 }
 
 static GimpValueArray *
-xpm_load (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GFile                *file,
-          const GimpValueArray *args,
-          gpointer              run_data)
+xpm_load (GimpProcedure         *procedure,
+          GimpRunMode            run_mode,
+          GFile                 *file,
+          GimpMetadata          *metadata,
+          GimpMetadataLoadFlags *flags,
+          GimpProcedureConfig   *config,
+          gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -301,19 +306,16 @@ xpm_save (GimpProcedure        *procedure,
           gint                  n_drawables,
           GimpDrawable        **drawables,
           GFile                *file,
-          const GimpValueArray *args,
+          GimpMetadata         *metadata,
+          GimpProcedureConfig  *config,
           gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpPDBStatusType    status = GIMP_PDB_SUCCESS;
-  GimpExportReturn     export = GIMP_EXPORT_CANCEL;
-  GError              *error = NULL;
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
+  GError            *error = NULL;
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, image, run_mode, args);
-
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
@@ -362,9 +364,6 @@ xpm_save (GimpProcedure        *procedure,
         }
     }
 
-  gimp_procedure_config_end_run (config, status);
-  g_object_unref (config);
-
   if (export == GIMP_EXPORT_EXPORT)
     {
       gimp_image_delete (image);
@@ -386,7 +385,8 @@ load_image (GFile   *file,
                              gimp_file_get_utf8_name (file));
 
   /* read the raw file */
-  switch (XpmReadFileToXpmImage (g_file_peek_path (file), &xpm_image, NULL))
+  switch (XpmReadFileToXpmImage ((char *) g_file_peek_path (file),
+                                 &xpm_image, NULL))
     {
     case XpmSuccess:
       break;
@@ -843,7 +843,8 @@ save_image (GFile         *file,
   xpm_image->data       = ibuff;
 
   /* do the save */
-  switch (XpmWriteFileFromXpmImage (g_file_peek_path (file), xpm_image, NULL))
+  switch (XpmWriteFileFromXpmImage ((char *) g_file_peek_path (file),
+                                    xpm_image, NULL))
     {
     case XpmSuccess:
       success = TRUE;
