@@ -45,7 +45,6 @@ enum
   PLURAL,
   UNIT,
   USER_UNIT,
-  BG_COLOR,
   NUM_COLUMNS
 };
 
@@ -76,7 +75,7 @@ static GimpProcedure  * editor_create_procedure (GimpPlugIn           *plug_in,
                                                  const gchar          *name);
 
 static GimpValueArray * editor_run              (GimpProcedure        *procedure,
-                                                 const GimpValueArray *args,
+                                                 GimpProcedureConfig  *config,
                                                  gpointer              run_data);
 
 static GimpUnit new_unit_dialog                 (GtkWindow             *main_window,
@@ -174,7 +173,7 @@ editor_create_procedure (GimpPlugIn  *plug_in,
 
       gimp_procedure_set_menu_label (procedure, _("U_nits"));
       gimp_procedure_set_icon_name (procedure, GIMP_ICON_TOOL_MEASURE);
-      gimp_procedure_add_menu_path (procedure, "<Image>/Edit");
+      gimp_procedure_add_menu_path (procedure, "<Image>/Edit/[Preferences]");
 
       gimp_procedure_set_documentation (procedure,
                                         _("Create or alter units used in GIMP"),
@@ -220,8 +219,7 @@ on_app_activate (GApplication *gapp, gpointer user_data)
                                    G_TYPE_STRING,    /*  SINGULAR      */
                                    G_TYPE_STRING,    /*  PLURAL        */
                                    GIMP_TYPE_UNIT,   /*  UNIT          */
-                                   G_TYPE_BOOLEAN,   /*  USER_UNIT     */
-                                   GDK_TYPE_RGBA);   /*  BG_COLOR      */
+                                   G_TYPE_BOOLEAN);  /*  USER_UNIT     */
 
   self->tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
   g_object_unref (list_store);
@@ -318,9 +316,8 @@ on_app_activate (GApplication *gapp, gpointer user_data)
   col =
     gtk_tree_view_column_new_with_attributes (gettext (columns[SAVE].title),
                                               rend,
-                                              "active",               SAVE,
-                                              "activatable",          USER_UNIT,
-                                              "cell-background-rgba", BG_COLOR,
+                                              "active",      SAVE,
+                                              "activatable", USER_UNIT,
                                               NULL);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (self->tv), col);
@@ -347,8 +344,7 @@ on_app_activate (GApplication *gapp, gpointer user_data)
       col =
         gtk_tree_view_column_new_with_attributes (gettext (columns[i].title),
                                                   gtk_cell_renderer_text_new (),
-                                                  "text",                 i,
-                                                  "cell-background-rgba", BG_COLOR,
+                                                  "text", i,
                                                   NULL);
 
       gtk_tree_view_append_column (GTK_TREE_VIEW (self->tv), col);
@@ -392,7 +388,7 @@ unit_editor_help_clicked (GtkWidget *window)
 
 static GimpValueArray *
 editor_run (GimpProcedure        *procedure,
-            const GimpValueArray *args,
+            GimpProcedureConfig  *config,
             gpointer              run_data)
 {
   GimpUnitEditor *editor = GIMP_UNIT_EDITOR (run_data);
@@ -732,7 +728,6 @@ unit_list_init (GtkTreeView *tv)
   GtkTreeIter   iter;
   gint          num_units;
   GimpUnit      unit;
-  GdkRGBA       color;
 
   list_store = GTK_LIST_STORE (gtk_tree_view_get_model (tv));
 
@@ -740,11 +735,6 @@ unit_list_init (GtkTreeView *tv)
 
   num_units = gimp_unit_get_number_of_units ();
 
-  color.red   = 0.87;
-  color.green = 0.87;
-  color.blue  = 1;
-  color.alpha = 1;
-
   for (unit = GIMP_UNIT_INCH; unit < num_units; unit++)
     {
       gboolean user_unit = (unit >= gimp_unit_get_number_of_built_in_units ());
@@ -761,9 +751,6 @@ unit_list_init (GtkTreeView *tv)
                           PLURAL,       gimp_unit_get_plural (unit),
                           UNIT,         unit,
                           USER_UNIT,    user_unit,
-
-                          user_unit ? -1 : BG_COLOR, &color,
-
                           -1);
     }
 
