@@ -58,7 +58,7 @@ struct _BrowserClass
 
 
 #define BROWSER_TYPE  (browser_get_type ())
-#define BROWSER (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BROWSER_TYPE, Browser))
+#define BROWSER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BROWSER_TYPE, Browser))
 
 GType                   browser_get_type         (void) G_GNUC_CONST;
 
@@ -67,7 +67,7 @@ static GimpProcedure  * browser_create_procedure (GimpPlugIn           *plug_in,
                                                   const gchar          *name);
 
 static GimpValueArray * browser_run              (GimpProcedure        *procedure,
-                                                  const GimpValueArray *args,
+                                                  GimpProcedureConfig  *config,
                                                   gpointer              run_data);
 
 static gboolean         browser_open_url         (GtkWindow            *window,
@@ -128,7 +128,7 @@ browser_create_procedure (GimpPlugIn  *plug_in,
       GIMP_PROC_ARG_STRING (procedure, "url",
                             "URL",
                             "URL to open",
-                            "http://www.gimp.org/",
+                            "https://www.gimp.org/",
                             G_PARAM_READWRITE);
     }
 
@@ -137,19 +137,22 @@ browser_create_procedure (GimpPlugIn  *plug_in,
 
 static GimpValueArray *
 browser_run (GimpProcedure        *procedure,
-             const GimpValueArray *args,
+             GimpProcedureConfig  *config,
              gpointer              run_data)
 {
   GError *error = NULL;
+  gchar  *url   = NULL;
 
-  if (! browser_open_url (NULL, GIMP_VALUES_GET_STRING (args, 0),
-                          &error))
+  g_object_get (config, "url", &url, NULL);
+  if (! browser_open_url (NULL, url, &error))
     {
+      g_free (url);
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_EXECUTION_ERROR,
                                                error);
     }
 
+  g_free (url);
   return gimp_procedure_new_return_values (procedure,
                                            GIMP_PDB_SUCCESS,
                                            NULL);
