@@ -50,32 +50,35 @@ struct _JpegClass
 
 
 #define JPEG_TYPE  (jpeg_get_type ())
-#define JPEG (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), JPEG_TYPE, Jpeg))
+#define JPEG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), JPEG_TYPE, Jpeg))
 
 GType                   jpeg_get_type         (void) G_GNUC_CONST;
 
-static GList          * jpeg_query_procedures (GimpPlugIn           *plug_in);
-static GimpProcedure  * jpeg_create_procedure (GimpPlugIn           *plug_in,
-                                               const gchar          *name);
+static GList          * jpeg_query_procedures (GimpPlugIn            *plug_in);
+static GimpProcedure  * jpeg_create_procedure (GimpPlugIn            *plug_in,
+                                               const gchar           *name);
 
-static GimpValueArray * jpeg_load             (GimpProcedure        *procedure,
-                                               GimpRunMode           run_mode,
-                                               GFile                *file,
-                                               const GimpValueArray *args,
-                                               gpointer              run_data);
-static GimpValueArray * jpeg_load_thumb       (GimpProcedure        *procedure,
-                                               GFile                *file,
-                                               gint                  size,
-                                               const GimpValueArray *args,
-                                               gpointer              run_data);
-static GimpValueArray * jpeg_save             (GimpProcedure        *procedure,
-                                               GimpRunMode           run_mode,
-                                               GimpImage            *image,
-                                               gint                  n_drawables,
-                                               GimpDrawable        **drawables,
-                                               GFile                *file,
-                                               const GimpValueArray *args,
-                                               gpointer              run_data);
+static GimpValueArray * jpeg_load             (GimpProcedure         *procedure,
+                                               GimpRunMode            run_mode,
+                                               GFile                 *file,
+                                               GimpMetadata          *metadata,
+                                               GimpMetadataLoadFlags *flags,
+                                               GimpProcedureConfig   *config,
+                                               gpointer               run_data);
+static GimpValueArray * jpeg_load_thumb       (GimpProcedure         *procedure,
+                                               GFile                 *file,
+                                               gint                   size,
+                                               GimpProcedureConfig   *config,
+                                               gpointer               run_data);
+static GimpValueArray * jpeg_save             (GimpProcedure         *procedure,
+                                               GimpRunMode            run_mode,
+                                               GimpImage             *image,
+                                               gint                   n_drawables,
+                                               GimpDrawable         **drawables,
+                                               GFile                 *file,
+                                               GimpMetadata          *metadata,
+                                               GimpProcedureConfig   *config,
+                                               gpointer               run_data);
 
 
 G_DEFINE_TYPE (Jpeg, jpeg, GIMP_TYPE_PLUG_IN)
@@ -173,7 +176,7 @@ jpeg_create_procedure (GimpPlugIn  *plug_in,
     {
       procedure = gimp_save_procedure_new (plug_in, name,
                                            GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           jpeg_save, NULL, NULL);
+                                           TRUE, jpeg_save, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "RGB*, GRAY*");
 
@@ -200,75 +203,75 @@ jpeg_create_procedure (GimpPlugIn  *plug_in,
        * settings
        */
       GIMP_PROC_ARG_DOUBLE (procedure, "quality",
-                            "_Quality",
-                            "Quality of exported image",
+                            _("_Quality"),
+                            _("Quality of exported image"),
                             0.0, 1.0, 0.9,
                             G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_DOUBLE (procedure, "smoothing",
-                            "S_moothing",
-                            "Smoothing factor for exported image",
+                            _("S_moothing"),
+                            _("Smoothing factor for exported image"),
                             0.0, 1.0, 0.0,
                             G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "optimize",
-                             "Optimi_ze",
-                             "Use optimized tables during Huffman coding",
+                             _("Optimi_ze"),
+                             _("Use optimized tables during Huffman coding"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "progressive",
-                             "_Progressive",
-                             "Create progressive JPEG images",
+                             _("_Progressive"),
+                             _("Create progressive JPEG images"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "cmyk",
-                             "Export as CM_YK",
-                             "Create a CMYK JPEG image using the soft-proofing color profile",
+                             _("Export as CM_YK"),
+                             _("Create a CMYK JPEG image using the soft-proofing color profile"),
                              FALSE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "sub-sampling",
                          _("Su_bsampling"),
-                         "Sub-sampling type { 0 == 4:2:0 (chroma quartered), "
-                         "1 == 4:2:2 (chroma halved horizontally), "
-                         "2 == 4:4:4 (best quality), "
-                         "3 == 4:4:0 (chroma halved vertically)",
+                         _("Sub-sampling type { 0 == 4:2:0 (chroma quartered), "
+                           "1 == 4:2:2 (chroma halved horizontally), "
+                           "2 == 4:4:4 (best quality), "
+                           "3 == 4:4:0 (chroma halved vertically)"),
                          JPEG_SUBSAMPLING_2x2_1x1_1x1,
                          JPEG_SUBSAMPLING_1x2_1x1_1x1,
                          JPEG_SUBSAMPLING_1x1_1x1_1x1,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_BOOLEAN (procedure, "baseline",
-                             "Baseline",
-                             "Force creation of a baseline JPEG "
-                             "(non-baseline JPEGs can't be read by all decoders)",
+                             _("Baseline"),
+                             _("Force creation of a baseline JPEG "
+                               "(non-baseline JPEGs can't be read by all decoders)"),
                              TRUE,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "restart",
                          _("Inter_val (MCU rows):"),
-                         "Interval of restart markers "
-                         "(in MCU rows, 0 = no restart markers)",
+                         _("Interval of restart markers "
+                           "(in MCU rows, 0 = no restart markers)"),
                          0, 64, 0,
                          G_PARAM_READWRITE);
 
       GIMP_PROC_ARG_INT (procedure, "dct",
                          _("_DCT method"),
-                         "DCT method to use { INTEGER (0), FIXED (1), "
-                         "FLOAT (2) }",
+                         _("DCT method to use { INTEGER (0), FIXED (1), "
+                           "FLOAT (2) }"),
                          0, 2, 0,
                          G_PARAM_READWRITE);
 
       /* Some auxiliary arguments mostly for interactive usage. */
 
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "use-original-quality",
-                                 "_Use quality settings from original image",
-                                 "If the original image was loaded from a JPEG "
-                                 "file using non-standard quality settings "
-                                 "(quantization tables), enable this option to "
-                                 "get almost the same quality and file size.",
+                                 _("_Use quality settings from original image"),
+                                 _("If the original image was loaded from a JPEG "
+                                   "file using non-standard quality settings "
+                                   "(quantization tables), enable this option to "
+                                   "get almost the same quality and file size."),
                                  FALSE,
                                  G_PARAM_READWRITE);
       GIMP_PROC_AUX_ARG_INT (procedure, "original-quality",
@@ -287,12 +290,12 @@ jpeg_create_procedure (GimpPlugIn  *plug_in,
                              G_PARAM_READWRITE);
 
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "show-preview",
-                                 "Sho_w preview in image window",
-                                 "Creates a temporary layer with an export preview",
+                                 _("Sho_w preview in image window"),
+                                 _("Creates a temporary layer with an export preview"),
                                  FALSE,
                                  G_PARAM_READWRITE);
       GIMP_PROC_AUX_ARG_BOOLEAN (procedure, "use-arithmetic-coding",
-                                 "Use _arithmetic coding",
+                                 _("Use _arithmetic coding"),
                                  _("Older software may have trouble opening "
                                    "arithmetic-coded images"),
                                  FALSE,
@@ -314,11 +317,13 @@ jpeg_create_procedure (GimpPlugIn  *plug_in,
 }
 
 static GimpValueArray *
-jpeg_load (GimpProcedure        *procedure,
-           GimpRunMode           run_mode,
-           GFile                *file,
-           const GimpValueArray *args,
-           gpointer              run_data)
+jpeg_load (GimpProcedure         *procedure,
+           GimpRunMode            run_mode,
+           GFile                 *file,
+           GimpMetadata          *metadata,
+           GimpMetadataLoadFlags *flags,
+           GimpProcedureConfig   *config,
+           gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -347,23 +352,8 @@ jpeg_load (GimpProcedure        *procedure,
 
   if (image)
     {
-      GimpMetadata *metadata;
-
-      metadata = gimp_image_metadata_load_prepare (image, "image/jpeg",
-                                                   file, NULL);
-
-      if (metadata)
-        {
-          GimpMetadataLoadFlags flags = GIMP_METADATA_LOAD_ALL;
-
-          if (resolution_loaded)
-            flags &= ~GIMP_METADATA_LOAD_RESOLUTION;
-
-          gimp_image_metadata_load_finish (image, "image/jpeg",
-                                           metadata, flags);
-
-          g_object_unref (metadata);
-        }
+      if (resolution_loaded)
+        *flags &= ~GIMP_METADATA_LOAD_RESOLUTION;
     }
 
   if (! image)
@@ -381,11 +371,11 @@ jpeg_load (GimpProcedure        *procedure,
 }
 
 static GimpValueArray *
-jpeg_load_thumb (GimpProcedure        *procedure,
-                 GFile                *file,
-                 gint                  size,
-                 const GimpValueArray *args,
-                 gpointer              run_data)
+jpeg_load_thumb (GimpProcedure       *procedure,
+                 GFile               *file,
+                 gint                 size,
+                 GimpProcedureConfig *config,
+                 gpointer             run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -428,15 +418,14 @@ jpeg_save (GimpProcedure        *procedure,
            gint                  n_drawables,
            GimpDrawable        **drawables,
            GFile                *file,
-           const GimpValueArray *args,
+           GimpMetadata         *metadata,
+           GimpProcedureConfig  *config,
            gpointer              run_data)
 {
-  GimpPDBStatusType      status = GIMP_PDB_SUCCESS;
-  GimpProcedureConfig   *config;
-  GimpMetadata          *metadata;
-  GimpImage             *orig_image;
-  GimpExportReturn       export = GIMP_EXPORT_CANCEL;
-  GError                *error  = NULL;
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  GimpImage         *orig_image;
+  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
+  GError            *error  = NULL;
 
   gint                   orig_num_quant_tables = -1;
   gint                   orig_quality          = -1;
@@ -444,9 +433,6 @@ jpeg_save (GimpProcedure        *procedure,
 
   gegl_init (NULL, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  metadata = gimp_procedure_config_begin_export (config, image, run_mode,
-                                                 args, "image/jpeg");
   preview_image = NULL;
   preview_layer = NULL;
 
@@ -610,9 +596,6 @@ jpeg_save (GimpProcedure        *procedure,
         gimp_metadata_set_bits_per_sample (metadata, 8);
     }
 
-  gimp_procedure_config_end_export (config, image, file, status);
-  g_object_unref (config);
-
   if (export == GIMP_EXPORT_EXPORT)
     {
       /* If the image was exported, delete the new display. This also
