@@ -81,11 +81,11 @@ static const LayerModeMapping layer_mode_map[] =
   { "Difference",    "diff", GIMP_LAYER_MODE_DIFFERENCE_LEGACY,     TRUE },
 
   /* Linear Dodge (cs2) */
-  { "Linear Dodge",  "lddg", GIMP_LAYER_MODE_ADDITION,              TRUE },
+  { "Linear Dodge",  "lddg", GIMP_LAYER_MODE_ADDITION,              FALSE },
   { "Linear Dodge",  "lddg", GIMP_LAYER_MODE_ADDITION_LEGACY,       TRUE },
 
   /* Subtract (??) */
-  { "Subtract",      "fsub", GIMP_LAYER_MODE_SUBTRACT,              TRUE },
+  { "Subtract",      "fsub", GIMP_LAYER_MODE_SUBTRACT,              FALSE },
   { "Subtract",      "fsub", GIMP_LAYER_MODE_SUBTRACT_LEGACY,       TRUE },
 
   /* Darken (ps3) */
@@ -118,15 +118,15 @@ static const LayerModeMapping layer_mode_map[] =
   { "Luminosity",    "lum ", GIMP_LAYER_MODE_LUMINANCE,             FALSE },
 
   /* Divide (??) */
-  { "Divide",        "fdiv", GIMP_LAYER_MODE_DIVIDE,                TRUE },
+  { "Divide",        "fdiv", GIMP_LAYER_MODE_DIVIDE,                FALSE },
   { "Divide",        "fdiv", GIMP_LAYER_MODE_DIVIDE_LEGACY,         TRUE },
 
   /* Color Dodge (ps6) */
-  { "Color Dodge",   "div ", GIMP_LAYER_MODE_DODGE,                 TRUE },
+  { "Color Dodge",   "div ", GIMP_LAYER_MODE_DODGE,                 FALSE },
   { "Color Dodge",   "div ", GIMP_LAYER_MODE_DODGE_LEGACY,          TRUE },
 
   /* Color Burn (ps6) */
-  { "Color Burn",    "idiv", GIMP_LAYER_MODE_BURN,                  TRUE },
+  { "Color Burn",    "idiv", GIMP_LAYER_MODE_BURN,                  FALSE },
   { "Color Burn",    "idiv", GIMP_LAYER_MODE_BURN_LEGACY,           TRUE },
 
   /* Hard Light (ps3) */
@@ -145,16 +145,16 @@ static const LayerModeMapping layer_mode_map[] =
   { "Pin Light",     "pLit", GIMP_LAYER_MODE_PIN_LIGHT,             TRUE },
 
   /* Linear Light (ps7)*/
-  { "Linear Light",  "lLit", GIMP_LAYER_MODE_LINEAR_LIGHT,          TRUE },
+  { "Linear Light",  "lLit", GIMP_LAYER_MODE_LINEAR_LIGHT,          FALSE },
 
   /* Hard Mix (CS)*/
-  { "Hard Mix",      "hMix", GIMP_LAYER_MODE_HARD_MIX,              TRUE },
+  { "Hard Mix",      "hMix", GIMP_LAYER_MODE_HARD_MIX,              FALSE },
 
   /* Exclusion (ps6) */
   { "Exclusion",     "smud", GIMP_LAYER_MODE_EXCLUSION,             TRUE },
 
   /* Linear Burn (ps7)*/
-  { "Linear Burn",   "lbrn", GIMP_LAYER_MODE_LINEAR_BURN,           TRUE },
+  { "Linear Burn",   "lbrn", GIMP_LAYER_MODE_LINEAR_BURN,           FALSE },
 
   /* Darker Color (??)*/
   { "Darker Color",  "dkCl", GIMP_LAYER_MODE_LUMA_DARKEN_ONLY,      FALSE },
@@ -583,6 +583,7 @@ decode_packbits (const gchar *src,
             {
               IFDBG(2) g_debug ("Overrun in packbits replicate of %d chars", n - unpack_left);
               error_code = 2;
+              break;
             }
           memset (dst, *src, n);
           src++;
