@@ -50,33 +50,35 @@ struct _DarktableClass
 
 
 #define DARKTABLE_TYPE  (darktable_get_type ())
-#define DARKTABLE (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DARKTABLE_TYPE, Darktable))
+#define DARKTABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DARKTABLE_TYPE, Darktable))
 
 GType                   darktable_get_type         (void) G_GNUC_CONST;
 
-static GList          * darktable_init_procedures  (GimpPlugIn           *plug_in);
-static GimpProcedure  * darktable_create_procedure (GimpPlugIn           *plug_in,
-                                                    const gchar          *name);
+static GList          * darktable_init_procedures  (GimpPlugIn            *plug_in);
+static GimpProcedure  * darktable_create_procedure (GimpPlugIn            *plug_in,
+                                                    const gchar           *name);
 
-static GimpValueArray * darktable_load             (GimpProcedure        *procedure,
-                                                    GimpRunMode           run_mode,
-                                                    GFile                *file,
-                                                    const GimpValueArray *args,
-                                                    gpointer              run_data);
-static GimpValueArray * darktable_load_thumb       (GimpProcedure        *procedure,
-                                                    GFile                *file,
-                                                    gint                  size,
-                                                    const GimpValueArray *args,
-                                                    gpointer              run_data);
+static GimpValueArray * darktable_load             (GimpProcedure         *procedure,
+                                                    GimpRunMode            run_mode,
+                                                    GFile                 *file,
+                                                    GimpMetadata          *metadata,
+                                                    GimpMetadataLoadFlags *flags,
+                                                    GimpProcedureConfig   *config,
+                                                    gpointer               run_data);
+static GimpValueArray * darktable_load_thumb       (GimpProcedure         *procedure,
+                                                    GFile                 *file,
+                                                    gint                   size,
+                                                    GimpProcedureConfig   *config,
+                                                    gpointer               run_data);
 
-static GimpImage      * load_image                 (GFile                *file,
-                                                    GimpRunMode           run_mode,
-                                                    GError              **error);
-static GimpImage      * load_thumbnail_image       (GFile                *file,
-                                                    gint                  thumb_size,
-                                                    gint                 *width,
-                                                    gint                 *height,
-                                                    GError              **error);
+static GimpImage      * load_image                 (GFile                 *file,
+                                                    GimpRunMode            run_mode,
+                                                    GError               **error);
+static GimpImage      * load_thumbnail_image       (GFile                 *file,
+                                                    gint                   thumb_size,
+                                                    gint                  *width,
+                                                    gint                  *height,
+                                                    GError               **error);
 
 
 G_DEFINE_TYPE (Darktable, darktable, GIMP_TYPE_PLUG_IN)
@@ -302,11 +304,13 @@ darktable_create_procedure (GimpPlugIn  *plug_in,
 }
 
 static GimpValueArray *
-darktable_load (GimpProcedure        *procedure,
-                GimpRunMode           run_mode,
-                GFile                *file,
-                const GimpValueArray *args,
-                gpointer              run_data)
+darktable_load (GimpProcedure         *procedure,
+                GimpRunMode            run_mode,
+                GFile                 *file,
+                GimpMetadata          *metadata,
+                GimpMetadataLoadFlags *flags,
+                GimpProcedureConfig   *config,
+                gpointer               run_data)
 {
   GimpValueArray *return_vals;
   GimpImage      *image;
@@ -329,11 +333,11 @@ darktable_load (GimpProcedure        *procedure,
 }
 
 static GimpValueArray *
-darktable_load_thumb (GimpProcedure        *procedure,
-                      GFile                *file,
-                      gint                  size,
-                      const GimpValueArray *args,
-                      gpointer              run_data)
+darktable_load_thumb (GimpProcedure       *procedure,
+                      GFile               *file,
+                      gint                 size,
+                      GimpProcedureConfig *config,
+                      gpointer             run_data)
 {
   GimpValueArray *return_vals;
   gint            width;
@@ -513,7 +517,7 @@ load_thumbnail_image (GFile   *file,
   gimp_progress_init_printf (_("Opening thumbnail for '%s'"),
                              gimp_file_get_utf8_name (file));
 
-  *width = *height = thumb_size;
+  *width = *height = 0;
 
   if (g_spawn_sync (NULL,
                     argv,
