@@ -490,25 +490,23 @@ gfig_dialog (GimpGfig *gfig)
                     vbox);
 
   /* foreground color button in Stroke frame*/
-  gfig_context->fg_color = g_new0 (GimpRGB, 1);
-  gfig_context->fg_color_button = gimp_color_button_new ("Foreground",
-                                                    SEL_BUTTON_WIDTH,
-                                                    SEL_BUTTON_HEIGHT,
-                                                    gfig_context->fg_color,
-                                                    GIMP_COLOR_AREA_SMALL_CHECKS);
+  gfig_context->fg_color = gegl_color_duplicate (gfig_context->default_style.foreground);
+  gfig_context->fg_color_button = gimp_color_button_new (_("Foreground"),
+                                                         SEL_BUTTON_WIDTH,
+                                                         SEL_BUTTON_HEIGHT,
+                                                         gfig_context->fg_color,
+                                                         GIMP_COLOR_AREA_SMALL_CHECKS);
   g_signal_connect (gfig_context->fg_color_button, "color-changed",
                     G_CALLBACK (set_foreground_callback),
                     gfig_context->fg_color);
-  gimp_color_button_set_color (GIMP_COLOR_BUTTON (gfig_context->fg_color_button),
-                               &gfig_context->default_style.foreground);
   gtk_box_pack_start (GTK_BOX (vbox), gfig_context->fg_color_button,
                       FALSE, FALSE, 0);
   gtk_widget_show (gfig_context->fg_color_button);
 
   /* brush selector in Stroke frame */
   gfig_context->brush_select
-    = gimp_brush_select_button_new ("Brush",
-                                    GIMP_RESOURCE (gfig_context->default_style.brush));
+    = gimp_brush_chooser_new (_("Brush"), _("Brush"),
+                              GIMP_RESOURCE (gfig_context->default_style.brush));
   g_signal_connect (gfig_context->brush_select, "resource-set",
                     G_CALLBACK (gfig_brush_changed_callback), NULL);
   gtk_box_pack_start (GTK_BOX (vbox), gfig_context->brush_select,
@@ -557,24 +555,22 @@ gfig_dialog (GimpGfig *gfig)
                             empty_label, NULL);
 
   /* A page for the fill color button */
-  gfig_context->bg_color = g_new0 (GimpRGB, 1);
-  gfig_context->bg_color_button = gimp_color_button_new ("Background",
-                                           SEL_BUTTON_WIDTH, SEL_BUTTON_HEIGHT,
-                                           gfig_context->bg_color,
-                                           GIMP_COLOR_AREA_SMALL_CHECKS);
+  gfig_context->bg_color = gegl_color_duplicate (gfig_context->default_style.background);
+  gfig_context->bg_color_button = gimp_color_button_new (_("Background"),
+                                                         SEL_BUTTON_WIDTH, SEL_BUTTON_HEIGHT,
+                                                         gfig_context->bg_color,
+                                                         GIMP_COLOR_AREA_SMALL_CHECKS);
   g_signal_connect (gfig_context->bg_color_button, "color-changed",
                     G_CALLBACK (set_background_callback),
                     gfig_context->bg_color);
-  gimp_color_button_set_color (GIMP_COLOR_BUTTON (gfig_context->bg_color_button),
-                               &gfig_context->default_style.background);
   gtk_widget_show (gfig_context->bg_color_button);
   gtk_notebook_append_page (GTK_NOTEBOOK (fill_type_notebook),
                             gfig_context->bg_color_button, NULL);
 
   /* A page for the pattern selector */
   gfig_context->pattern_select
-    = gimp_pattern_select_button_new ("Pattern",
-                                      GIMP_RESOURCE (gfig_context->default_style.pattern));
+    = gimp_pattern_chooser_new (_("Pattern"), _("Pattern"),
+                                GIMP_RESOURCE (gfig_context->default_style.pattern));
   g_signal_connect (gfig_context->pattern_select, "resource-set",
                     G_CALLBACK (gfig_pattern_changed_callback), NULL);
   gtk_widget_show (gfig_context->pattern_select);
@@ -583,8 +579,8 @@ gfig_dialog (GimpGfig *gfig)
 
   /* A page for the gradient selector */
   gfig_context->gradient_select
-    = gimp_gradient_select_button_new ("Gradient",
-                                       GIMP_RESOURCE (gfig_context->default_style.gradient));
+    = gimp_gradient_chooser_new (_("Gradient"), _("Gradient"),
+                                 GIMP_RESOURCE (gfig_context->default_style.gradient));
   g_signal_connect (gfig_context->gradient_select, "resource-set",
                     G_CALLBACK (gfig_gradient_changed_callback), NULL);
   gtk_widget_show (gfig_context->gradient_select);
