@@ -84,7 +84,7 @@ struct _MetadataClass
 
 
 #define METADATA_TYPE  (metadata_get_type ())
-#define METADATA (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), METADATA_TYPE, Metadata))
+#define METADATA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), METADATA_TYPE, Metadata))
 
 GType                   metadata_get_type         (void) G_GNUC_CONST;
 
@@ -93,7 +93,11 @@ static GimpProcedure  * metadata_create_procedure (GimpPlugIn           *plug_in
                                                    const gchar          *name);
 
 static GimpValueArray * metadata_run              (GimpProcedure        *procedure,
-                                                   const GimpValueArray *args,
+                                                   GimpRunMode           run_mode,
+                                                   GimpImage            *image,
+                                                   gint                  n_drawables,
+                                                   GimpDrawable        **drawables,
+                                                   GimpProcedureConfig  *config,
                                                    gpointer              run_data);
 
 static gboolean  metadata_viewer_dialog           (GimpImage            *image,
@@ -168,9 +172,9 @@ metadata_create_procedure (GimpPlugIn  *plug_in,
 
   if (! strcmp (name, PLUG_IN_PROC))
     {
-      procedure = gimp_procedure_new (plug_in, name,
-                                      GIMP_PDB_PROC_TYPE_PLUGIN,
-                                      metadata_run, NULL, NULL);
+      procedure = gimp_image_procedure_new (plug_in, name,
+                                            GIMP_PDB_PROC_TYPE_PLUGIN,
+                                            metadata_run, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "*");
 
@@ -190,19 +194,6 @@ metadata_create_procedure (GimpPlugIn  *plug_in,
                                       "Hartmut Kuhse, Michael Natterer, "
                                       "Ben Touchette",
                                       "2013, 2017");
-
-      GIMP_PROC_ARG_ENUM (procedure, "run-mode",
-                          "Run mode",
-                          "The run mode",
-                          GIMP_TYPE_RUN_MODE,
-                          GIMP_RUN_INTERACTIVE,
-                          G_PARAM_READWRITE);
-
-      GIMP_PROC_ARG_IMAGE (procedure, "image",
-                           "Image",
-                           "The input image",
-                           FALSE,
-                           G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -210,17 +201,18 @@ metadata_create_procedure (GimpPlugIn  *plug_in,
 
 static GimpValueArray *
 metadata_run (GimpProcedure        *procedure,
-              const GimpValueArray *args,
+              GimpRunMode           run_mode,
+              GimpImage            *image,
+              gint                  n_drawables,
+              GimpDrawable        **drawables,
+              GimpProcedureConfig  *config,
               gpointer              run_data)
 {
-  GimpImage    *image;
   GimpMetadata *metadata;
   GError       *error  = NULL;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
-  image = GIMP_VALUES_GET_IMAGE (args, 1);
-
   metadata = gimp_image_get_metadata (image);
 
   /* Always show metadata dialog so we can add appropriate iptc data
@@ -314,7 +306,7 @@ metadata_viewer_dialog (GimpImage     *image,
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), col);
 
   rend = gtk_cell_renderer_text_new ();
-  col = gtk_tree_view_column_new_with_attributes (_("Value"),
+  col = gtk_tree_view_column_new_with_attributes (C_("A tag value", "Value"),
                                                   rend,
                                                   "text", C_EXIF_VALUE,
                                                   NULL);
@@ -352,7 +344,7 @@ metadata_viewer_dialog (GimpImage     *image,
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), col);
 
   rend = gtk_cell_renderer_text_new ();
-  col = gtk_tree_view_column_new_with_attributes (_("Value"),
+  col = gtk_tree_view_column_new_with_attributes (C_("A tag value", "Value"),
                                                   rend,
                                                   "text", C_XMP_VALUE,
                                                   NULL);
@@ -390,7 +382,7 @@ metadata_viewer_dialog (GimpImage     *image,
   gtk_tree_view_append_column (GTK_TREE_VIEW (list_view), col);
 
   rend = gtk_cell_renderer_text_new ();
-  col = gtk_tree_view_column_new_with_attributes (_("Value"),
+  col = gtk_tree_view_column_new_with_attributes (C_("A tag value", "Value"),
                                                   rend,
                                                   "text", C_IPTC_VALUE,
                                                   NULL);
