@@ -46,6 +46,7 @@
 
 #include "scheme-wrapper.h"
 #include "scheme-marshal.h"
+#include "scheme-marshal-return.h"
 
 
 #undef cons
@@ -97,7 +98,8 @@ static const NamedConstant script_constants[] =
   /* Useful values from libgimpbase/gimplimits.h */
   { "MIN-IMAGE-SIZE", GIMP_MIN_IMAGE_SIZE },
   { "MAX-IMAGE-SIZE", GIMP_MAX_IMAGE_SIZE },
-  { "MIN-RESOLUTION", GIMP_MIN_RESOLUTION },
+  /* Note conversion to int, loss of precision: 0.005=>0 */
+  { "MIN-RESOLUTION", (gint) GIMP_MIN_RESOLUTION },
   { "MAX-RESOLUTION", GIMP_MAX_RESOLUTION },
 
   /* Useful misc stuff */
@@ -637,18 +639,17 @@ script_fu_marshal_procedure_call (scheme   *sc,
                                   gboolean  permissive,
                                   gboolean  deprecated)
 {
-  GimpProcedure   *procedure;
-  GimpValueArray  *args;
-  GimpValueArray  *values = NULL;
-  gchar           *proc_name;
-  GParamSpec     **arg_specs;
-  gint             n_arg_specs;
-  gint             actual_arg_count;
-  gint             consumed_arg_count = 0;
-  gchar            error_str[1024];
-  gint             i;
-  pointer          return_val = sc->NIL;
-
+  GimpProcedure        *procedure;
+  GimpProcedureConfig  *config;
+  GimpValueArray       *values = NULL;
+  gchar                *proc_name;
+  GParamSpec          **arg_specs;
+  gint                  n_arg_specs;
+  gint                  actual_arg_count;
+  gint                  consumed_arg_count = 0;
+  gchar                 error_str[1024];
+  gint                  i;
+  pointer               return_val = sc->NIL;
 
   g_debug ("In %s()", G_STRFUNC);
 
@@ -687,6 +688,7 @@ script_fu_marshal_procedure_call (scheme   *sc,
       return script_error (sc, error_str, 0);
     }
 
+  config    = gimp_procedure_create_config (procedure);
   arg_specs = gimp_procedure_get_arguments (procedure, &n_arg_specs);
   actual_arg_count = sc->vptr->list_length (sc, a) - 1;
 
@@ -719,8 +721,6 @@ script_fu_marshal_procedure_call (scheme   *sc,
   }
 
   /*  Marshall the supplied arguments  */
-  args = gimp_value_array_new (n_arg_specs);
-
   for (i = 0; i < n_arg_specs; i++)
     {
       GParamSpec *arg_spec = arg_specs[i];
@@ -751,34 +751,98 @@ script_fu_marshal_procedure_call (scheme   *sc,
       if (G_VALUE_HOLDS_INT (&value))
         {
           if (! sc->vptr->is_number (sc->vptr->pair_car (a)))
-            return script_type_error (sc, "numeric", i, proc_name);
+            {
+              return script_type_error (sc, "numeric", i, proc_name);
+            }
           else
-            g_value_set_int (&value,
-                             sc->vptr->ivalue (sc->vptr->pair_car (a)));
+            {
+              GParamSpecInt *ispec = G_PARAM_SPEC_INT (arg_spec);
+              gint           v     = sc->vptr->ivalue (sc->vptr->pair_car (a));
+
+              if (v < ispec->minimum || v > ispec->maximum)
+                {
+                  gchar error_message[1024];
+
+                  g_snprintf (error_message, sizeof (error_message),
+                              "Invalid value %d for argument %d: expected value between %d and %d",
+                              v, i, ispec->minimum, ispec->maximum);
+                  return script_error (sc, error_message, 0);
+                }
+
+              g_value_set_int (&value, v);
+            }
         }
       else if (G_VALUE_HOLDS_UINT (&value))
         {
           if (! sc->vptr->is_number (sc->vptr->pair_car (a)))
-            return script_type_error (sc, "numeric", i, proc_name);
+            {
+              return script_type_error (sc, "numeric", i, proc_name);
+            }
           else
-            g_value_set_uint (&value,
-                              sc->vptr->ivalue (sc->vptr->pair_car (a)));
+            {
+              GParamSpecUInt *ispec = G_PARAM_SPEC_UINT (arg_spec);
+              gint            v     = sc->vptr->ivalue (sc->vptr->pair_car (a));
+
+              if (v < ispec->minimum || v > ispec->maximum)
+                {
+                  gchar error_message[1024];
+
+                  g_snprintf (error_message, sizeof (error_message),
+                              "Invalid value %d for argument %d: expected value between %d and %d",
+                              v, i, ispec->minimum, ispec->maximum);
+                  return script_error (sc, error_message, 0);
+                }
+
+              g_value_set_uint (&value, v);
+            }
         }
       else if (G_VALUE_HOLDS_UCHAR (&value))
         {
           if (! sc->vptr->is_number (sc->vptr->pair_car (a)))
-            return script_type_error (sc, "numeric", i, proc_name);
+            {
+              return script_type_error (sc, "numeric", i, proc_name);
+            }
           else
-            g_value_set_uchar (&value,
-                               sc->vptr->ivalue (sc->vptr->pair_car (a)));
+            {
+              GParamSpecUChar *cspec = G_PARAM_SPEC_UCHAR (arg_spec);
+              gint             c     = sc->vptr->ivalue (sc->vptr->pair_car (a));
+
+              if (c < cspec->minimum || c > cspec->maximum)
+                {
+                  gchar error_message[1024];
+
+                  g_snprintf (error_message, sizeof (error_message),
+                              "Invalid value %d for argument %d: expected value between %d and %d",
+                              c, i, cspec->minimum, cspec->maximum);
+                  return script_error (sc, error_message, 0);
+                }
+
+              g_value_set_uchar (&value, c);
+            }
         }
       else if (G_VALUE_HOLDS_DOUBLE (&value))
         {
           if (! sc->vptr->is_number (sc->vptr->pair_car (a)))
-            return script_type_error (sc, "numeric", i, proc_name);
+            {
+              return script_type_error (sc, "numeric", i, proc_name);
+            }
           else
-            g_value_set_double (&value,
-                                sc->vptr->rvalue (sc->vptr->pair_car (a)));
+            {
+              GParamSpecDouble *dspec = G_PARAM_SPEC_DOUBLE (arg_spec);
+              gdouble           d     = sc->vptr->rvalue (sc->vptr->pair_car (a));
+
+              if (d < dspec->minimum || d > dspec->maximum)
+                {
+                  gchar error_message[1024];
+
+                  g_snprintf (error_message, sizeof (error_message),
+                              "Invalid value %f for argument %d: expected value between %f and %f",
+                              d, i, dspec->minimum, dspec->maximum);
+                  return script_error (sc, error_message, 0);
+                }
+
+              g_value_set_double (&value, d);
+            }
         }
       else if (G_VALUE_HOLDS_ENUM (&value))
         {
@@ -845,8 +909,7 @@ script_fu_marshal_procedure_call (scheme   *sc,
                   {
                     g_printerr ("     ");
                     for (j = 0; j < count; ++j)
-                      g_printerr (" \"%s\"",
-                                  args[i].data.d_strv[j]);
+                      g_printerr (" \"%s\"", array[j]);
                     g_printerr ("\n");
                   }
               }
@@ -921,7 +984,7 @@ script_fu_marshal_procedure_call (scheme   *sc,
             {
               gint id = sc->vptr->ivalue (sc->vptr->pair_car (a));
 
-              pointer error = marshal_ID_to_drawable(sc, a, id, &value);
+              pointer error = marshal_ID_to_item (sc, a, id, &value);
               if (error)
                 return error;
             }
@@ -987,7 +1050,12 @@ script_fu_marshal_procedure_call (scheme   *sc,
                */
               gint32 *array;
 
-              n_elements = GIMP_VALUES_GET_INT (args, i - 1);
+              if (i == 0)
+                return script_error (sc, "The first argument cannot be an array", a);
+              else if (! g_type_is_a (arg_specs[i - 1]->value_type, G_TYPE_INT))
+                return script_error (sc, "Array arguments must be preceded by an int argument (number of items)", a);
+
+              g_object_get (config, arg_specs[i - 1]->name, &n_elements, NULL);
 
               if (n_elements > sc->vptr->vector_length (vector))
                 return script_length_error_in_vector (sc, i, proc_name, n_elements, vector);
@@ -1053,7 +1121,12 @@ script_fu_marshal_procedure_call (scheme   *sc,
             {
               gdouble *array;
 
-              n_elements = GIMP_VALUES_GET_INT (args, i - 1);
+              if (i == 0)
+                return script_error (sc, "The first argument cannot be an array", a);
+              else if (! g_type_is_a (arg_specs[i - 1]->value_type, G_TYPE_INT))
+                return script_error (sc, "Array arguments must be preceded by an int argument (number of items)", a);
+
+              g_object_get (config, arg_specs[i - 1]->name, &n_elements, NULL);
 
               if (n_elements > sc->vptr->vector_length (vector))
                 return script_length_error_in_vector (sc, i, proc_name, n_elements, vector);
@@ -1084,13 +1157,13 @@ script_fu_marshal_procedure_call (scheme   *sc,
 
           if (sc->vptr->is_string (sc->vptr->pair_car (a)))
             {
-              if (! gimp_rgb_parse_css (&color,
-                                        sc->vptr->string_value (sc->vptr->pair_car (a)),
-                                        -1))
+              gchar *color_string = sc->vptr->string_value (sc->vptr->pair_car (a));
+
+              if (! gimp_rgb_parse_css (&color, color_string, -1))
                 return script_type_error (sc, "color string", i, proc_name);
 
               gimp_rgb_set_alpha (&color, 1.0);
-              g_debug ("(%s)", sc->vptr->string_value (sc->vptr->pair_car (a)));
+              gimp_value_set_rgb (&value, &color);
             }
           else if (sc->vptr->is_list (sc, sc->vptr->pair_car (a)) &&
                    sc->vptr->list_length (sc, sc->vptr->pair_car (a)) == 3)
@@ -1128,6 +1201,58 @@ script_fu_marshal_procedure_call (scheme   *sc,
           else
             return script_type_error (sc, "color string or list", i, proc_name);
         }
+      else if (GIMP_VALUE_HOLDS_COLOR (&value))
+        {
+          GeglColor *color = NULL;
+
+          if (sc->vptr->is_string (sc->vptr->pair_car (a)))
+            {
+              gchar *color_string = sc->vptr->string_value (sc->vptr->pair_car (a));
+
+              if (! (color = gimp_color_parse_css (color_string, -1)))
+                return script_type_error (sc, "color string", i, proc_name);
+            }
+          else if (sc->vptr->is_list (sc, sc->vptr->pair_car (a)) &&
+                   sc->vptr->list_length (sc, sc->vptr->pair_car (a)) == 3)
+            {
+              pointer color_list;
+              guchar  r = 0, g = 0, b = 0;
+
+              color_list = sc->vptr->pair_car (a);
+              if (sc->vptr->is_number (sc->vptr->pair_car (color_list)))
+                r = CLAMP (sc->vptr->ivalue (sc->vptr->pair_car (color_list)),
+                           0, 255);
+              else
+                return script_type_error_in_container (sc, "numeric", i, 0, proc_name, 0);
+
+              color_list = sc->vptr->pair_cdr (color_list);
+              if (sc->vptr->is_number (sc->vptr->pair_car (color_list)))
+                g = CLAMP (sc->vptr->ivalue (sc->vptr->pair_car (color_list)),
+                           0, 255);
+              else
+                return script_type_error_in_container (sc, "numeric", i, 1, proc_name, 0);
+
+              color_list = sc->vptr->pair_cdr (color_list);
+              if (sc->vptr->is_number (sc->vptr->pair_car (color_list)))
+                b = CLAMP (sc->vptr->ivalue (sc->vptr->pair_car (color_list)),
+                           0, 255);
+              else
+                return script_type_error_in_container (sc, "numeric", i, 2, proc_name, 0);
+
+              color = gegl_color_new ("black");
+              gegl_color_set_rgba_with_space (color,
+                                              (gdouble) r / 255.0,
+                                              (gdouble) g / 255.0,
+                                              (gdouble) b / 255.0,
+                                              1.0, NULL);
+            }
+          else
+            {
+              return script_type_error (sc, "color string or list", i, proc_name);
+            }
+
+          g_value_take_object (&value, color);
+        }
       else if (GIMP_VALUE_HOLDS_RGB_ARRAY (&value))
         {
           vector = sc->vptr->pair_car (a);
@@ -1137,7 +1262,12 @@ script_fu_marshal_procedure_call (scheme   *sc,
             {
               GimpRGB *array;
 
-              n_elements = GIMP_VALUES_GET_INT (args, i - 1);
+              if (i == 0)
+                return script_error (sc, "The first argument cannot be an array", a);
+              else if (! g_type_is_a (arg_specs[i - 1]->value_type, G_TYPE_INT))
+                return script_error (sc, "Array arguments must be preceded by an int argument (number of items)", a);
+
+              g_object_get (config, arg_specs[i - 1]->name, &n_elements, NULL);
 
               if (n_elements > sc->vptr->vector_length (vector))
                 return script_length_error_in_vector (sc, i, proc_name, n_elements, vector);
@@ -1230,11 +1360,15 @@ script_fu_marshal_procedure_call (scheme   *sc,
         }
       else if (GIMP_VALUE_HOLDS_OBJECT_ARRAY (&value))
         {
+          /* Now PDB procedures take arrays of Item (Drawable, Vectors, etc.).
+           * When future PDB procedures take arrays of Image, Display, Resource, etc.
+           * this will need changes.
+           */
           vector = sc->vptr->pair_car (a);
 
           if (sc->vptr->is_vector (vector))
             {
-              pointer error = marshal_vector_to_drawable_array (sc, vector, &value);
+              pointer error = marshal_vector_to_item_array (sc, vector, &value);
               if (error)
                 return error;
             }
@@ -1256,16 +1390,30 @@ script_fu_marshal_procedure_call (scheme   *sc,
         }
       else if (GIMP_VALUE_HOLDS_RESOURCE (&value))
         {
-          if (! sc->vptr->is_string (sc->vptr->pair_car (a)))
-            return script_type_error (sc, "string", i, proc_name);
+          if (! sc->vptr->is_integer (sc->vptr->pair_car (a)))
+            return script_type_error (sc, "integer", i, proc_name);
           else
             {
               /* Create new instance of a resource object. */
               GimpResource *resource;
-              GType         type = G_VALUE_TYPE (&value);
-              const gchar  *name = sc->vptr->string_value (sc->vptr->pair_car (a));
-              resource = gimp_resource_get_by_name (type, name);
 
+              gint resource_id = sc->vptr->ivalue (sc->vptr->pair_car (a));
+              /* Resource is abstract superclass. Concrete subclass is e.g. Brush.
+               * The gvalue holds i.e. requires an instance of concrete subclass.
+               * ID's are unique across all instances of Resource.
+               */
+
+              if (! gimp_resource_id_is_valid (resource_id))
+               {
+                  /* Not the ID of any instance of Resource. */
+                  return script_error (sc, "runtime: invalid resource ID", a);
+                }
+              resource = gimp_resource_get_by_id (resource_id);
+              if (! g_value_type_compatible (G_OBJECT_TYPE (resource), G_VALUE_TYPE (&value)))
+                {
+                  /* not the required subclass held by the gvalue */
+                  return script_error (sc, "runtime: resource ID of improper subclass.", a);
+                }
               g_value_set_object (&value, resource);
             }
         }
@@ -1277,7 +1425,18 @@ script_fu_marshal_procedure_call (scheme   *sc,
           return implementation_error (sc, error_str, 0);
         }
       debug_gvalue (&value);
-      gimp_value_array_append (args, &value);
+      if (g_param_value_validate (arg_spec, &value))
+        {
+          gchar error_message[1024];
+
+          g_snprintf (error_message, sizeof (error_message),
+                      "Invalid value for argument %d",
+                      i);
+          g_value_unset (&value);
+
+          return script_error (sc, error_message, 0);
+        }
+      g_object_set_property (G_OBJECT (config), arg_specs[i]->name, &value);
       g_value_unset (&value);
     }
 
@@ -1286,9 +1445,9 @@ script_fu_marshal_procedure_call (scheme   *sc,
       return script_error (sc, "A script cannot refresh scripts", 0);
 
   g_debug ("calling %s", proc_name);
-  values = gimp_pdb_run_procedure_array (gimp_get_pdb (),
-                                         proc_name, args);
+  values = gimp_procedure_run_config (procedure, config);
   g_debug ("done.");
+  g_clear_object (&config);
 
   /*  Check the return status  */
   if (! values)
@@ -1300,380 +1459,19 @@ script_fu_marshal_procedure_call (scheme   *sc,
       return script_error (sc, error_str, 0);
     }
 
-  /* No need to log a string for status, we log the cases, or caller will log it.*/
+  {
+    pointer calling_error;
+    return_val = marshal_PDB_return (sc, values, proc_name, &calling_error);
 
-  switch (GIMP_VALUES_GET_ENUM (values, 0))
-    {
-    case GIMP_PDB_EXECUTION_ERROR:
-      if (gimp_value_array_length (values) > 1 &&
-          G_VALUE_HOLDS_STRING (gimp_value_array_index (values, 1)))
-        {
-          g_snprintf (error_str, sizeof (error_str),
-                      "Procedure execution of %s failed: %s",
-                      proc_name,
-                      GIMP_VALUES_GET_STRING (values, 1));
-        }
-      else
-        {
-          g_snprintf (error_str, sizeof (error_str),
-                      "Procedure execution of %s failed",
-                      proc_name);
-        }
-        /* not language errors, procedure returned error for unknown reason. */
-      return foreign_error (sc, error_str, 0);
-      break;
-
-    case GIMP_PDB_CALLING_ERROR:
-      if (gimp_value_array_length (values) > 1 &&
-          G_VALUE_HOLDS_STRING (gimp_value_array_index (values, 1)))
-        {
-          g_snprintf (error_str, sizeof (error_str),
-                      "Procedure execution of %s failed on invalid input arguments: %s",
-                      proc_name,
-                      GIMP_VALUES_GET_STRING (values, 1));
-        }
-      else
-        {
-          g_snprintf (error_str, sizeof (error_str),
-                      "Procedure execution of %s failed on invalid input arguments",
-                      proc_name);
-        }
-      /* not language errors, GIMP validated the GValueArray
-       * and decided it doesn't match the registered signature
-       * or the procedure decided its preconditions not met (e.g. out of range)
-       */
-      return foreign_error (sc, error_str, 0);
-      break;
-
-    case GIMP_PDB_SUCCESS:
-      g_debug ("Count of non-status values returned: %d", gimp_value_array_length (values) - 1);
-
-      /* Counting down, i.e. traversing in reverse.
-       * i+1 is the current index.  i is the preceding value.
-       * When at the current index is an array, preceding value (at i) is array length.
-       *
-       * The below code for array results is not safe.
-       * It implicitly requires, but does not explicitly check,
-       * that the returned length equals the actual length of the returned array,
-       * and iterates over the returned array assuming it has the returned length.
-       */
-      for (i = gimp_value_array_length (values) - 2; i >= 0; --i)
-        {
-          GValue *value = gimp_value_array_index (values, i + 1);
-          gint    j;
-
-          g_debug ("Return value %d is type %s", i+1, G_VALUE_TYPE_NAME (value));
-
-          /* Order is important.
-           * GFile before other objects.
-           * GIMP resource objects before GIMP Image, Drawable, etc. objects.
-           * Alternatively, more specific tests.
-           */
-          if (G_VALUE_TYPE (value) == G_TYPE_FILE)
-            {
-              gchar *parsed_filepath = marshal_returned_gfile_to_string (value);
-
-              if (parsed_filepath)
-                {
-                  g_debug ("PDB procedure returned GFile '%s'", parsed_filepath);
-                  /* copy string into interpreter state. */
-                  return_val = sc->vptr->cons (sc, sc->vptr->mk_string (sc, parsed_filepath), return_val);
-                  g_free (parsed_filepath);
-                }
-              else
-                {
-                  g_warning ("PDB procedure failed to return a valid GFile");
-                  return_val = sc->vptr->cons (sc, sc->vptr->mk_string (sc, ""), return_val);
-                }
-              /* Ensure return_val holds a string, possibly empty. */
-            }
-          else if (GIMP_VALUE_HOLDS_RESOURCE (value))
-            {
-              /* ScriptFu represents resource objects by ther unique string ID's. */
-              GObject *object = g_value_get_object (value);
-              gchar   *name   = NULL;
-
-              if (object)
-                name = gimp_resource_get_name (GIMP_RESOURCE(object));
-
-              if (! name)
-                g_warning("PDB procedure returned NULL object.");
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_string (sc, name),
-                                           return_val);
-
-              g_free (name);
-            }
-          else if (G_VALUE_HOLDS_OBJECT (value))
-            {
-              /* G_VALUE_HOLDS_OBJECT only ensures value derives from GObject.
-               * Could be a GIMP or a GLib type.
-               * Here we handle GIMP types, which all have an id property.
-               * Resources have a string ID and Images and Drawables etc. have an int ID.
-               */
-              GObject *object = g_value_get_object (value);
-              gint     id     = -1;
-
-              /* expect a GIMP opaque object having an "id" property */
-              if (object)
-                g_object_get (object, "id", &id, NULL);
-
-              /* id is -1 when the gvalue had no GObject*,
-               * or the referenced object had no property "id".
-               * This can be an undetected fault in the called procedure.
-               * It is not necessarily an error in the script.
-               */
-              if (id == -1)
-                g_warning("PDB procedure returned NULL GIMP object.");
-
-              g_debug ("PDB procedure returned object ID: %i", id);
-
-              /* Scriptfu stores object IDs as int. */
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_integer (sc, id),
-                                           return_val);
-            }
-          else if (G_VALUE_HOLDS_INT (value))
-            {
-              gint v = g_value_get_int (value);
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_integer (sc, v),
-                                           return_val);
-            }
-          else if (G_VALUE_HOLDS_UINT (value))
-            {
-              guint v = g_value_get_uint (value);
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_integer (sc, v),
-                                           return_val);
-            }
-          else if (G_VALUE_HOLDS_DOUBLE (value))
-            {
-              gdouble v = g_value_get_double (value);
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_real (sc, v),
-                                           return_val);
-            }
-          else if (G_VALUE_HOLDS_ENUM (value))
-            {
-              gint v = g_value_get_enum (value);
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_integer (sc, v),
-                                           return_val);
-            }
-          else if (G_VALUE_HOLDS_BOOLEAN (value))
-            {
-              gboolean v = g_value_get_boolean (value);
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_integer (sc, v),
-                                           return_val);
-            }
-          else if (G_VALUE_HOLDS_STRING (value))
-            {
-              const gchar *v = g_value_get_string (value);
-
-              if (! v)
-                v = "";
-
-              return_val = sc->vptr->cons (sc, sc->vptr->mk_string (sc, v),
-                                           return_val);
-            }
-          else if (GIMP_VALUE_HOLDS_INT32_ARRAY (value))
-            {
-              gint32        n      = GIMP_VALUES_GET_INT (values, i);
-              const gint32 *v      = gimp_value_get_int32_array (value);
-              pointer       vector = sc->vptr->mk_vector (sc, n);
-
-              for (j = 0; j < n; j++)
-                {
-                  sc->vptr->set_vector_elem (vector, j,
-                                             sc->vptr->mk_integer (sc, v[j]));
-                }
-
-              return_val = sc->vptr->cons (sc, vector, return_val);
-            }
-          else if (G_VALUE_HOLDS (value, G_TYPE_BYTES))
-            {
-              GBytes       *v_bytes  = g_value_get_boxed (value);
-              const guint8 *v        = g_bytes_get_data (v_bytes, NULL);
-              gsize         n        = g_bytes_get_size (v_bytes);
-              pointer       vector   = sc->vptr->mk_vector (sc, n);
-
-              for (j = 0; j < n; j++)
-                {
-                  sc->vptr->set_vector_elem (vector, j,
-                                             sc->vptr->mk_integer (sc, v[j]));
-                }
-
-              return_val = sc->vptr->cons (sc, vector, return_val);
-            }
-          else if (GIMP_VALUE_HOLDS_FLOAT_ARRAY (value))
-            {
-              gint32         n      = GIMP_VALUES_GET_INT (values, i);
-              const gdouble *v      = gimp_value_get_float_array (value);
-              pointer        vector = sc->vptr->mk_vector (sc, n);
-
-              for (j = 0; j < n; j++)
-                {
-                  sc->vptr->set_vector_elem (vector, j,
-                                             sc->vptr->mk_real (sc, v[j]));
-                }
-
-              return_val = sc->vptr->cons (sc, vector, return_val);
-            }
-          else if (G_VALUE_HOLDS (value, G_TYPE_STRV))
-            {
-              gint32         n    = 0;
-              const gchar  **v    = g_value_get_boxed (value);
-              pointer        list = sc->NIL;
-
-              n = (v)? g_strv_length ((char **) v) : 0;
-              for (j = n - 1; j >= 0; j--)
-                {
-                  list = sc->vptr->cons (sc,
-                                         sc->vptr->mk_string (sc,
-                                                              v[j] ?
-                                                              v[j] : ""),
-                                         list);
-                }
-
-              return_val = sc->vptr->cons (sc, list, return_val);
-            }
-          else if (GIMP_VALUE_HOLDS_RGB (value))
-            {
-              GimpRGB  v;
-              guchar   r, g, b;
-              gpointer temp_val;
-
-              gimp_value_get_rgb (value, &v);
-              gimp_rgb_get_uchar (&v, &r, &g, &b);
-
-              temp_val = sc->vptr->cons
-                           (sc,
-                            sc->vptr->mk_integer (sc, r),
-                            sc->vptr->cons
-                              (sc,
-                               sc->vptr->mk_integer (sc, g),
-                               sc->vptr->cons
-                                 (sc,
-                                  sc->vptr->mk_integer (sc, b),
-                                  sc->NIL)));
-
-              return_val = sc->vptr->cons (sc,
-                                           temp_val,
-                                           return_val);
-            }
-          else if (GIMP_VALUE_HOLDS_RGB_ARRAY (value))
-            {
-              gint32          n = GIMP_VALUES_GET_INT (values, i);
-              const GimpRGB  *v = gimp_value_get_rgb_array (value);
-              pointer  vector   = sc->vptr->mk_vector (sc, n);
-
-              for (j = 0; j < n; j++)
-                {
-                  guchar  r, g, b;
-                  pointer temp_val;
-
-                  gimp_rgb_get_uchar (&v[j], &r, &g, &b);
-
-                  temp_val = sc->vptr->cons
-                              (sc,
-                               sc->vptr->mk_integer (sc, r),
-                               sc->vptr->cons
-                                 (sc,
-                                  sc->vptr->mk_integer (sc, g),
-                                  sc->vptr->cons
-                                    (sc,
-                                     sc->vptr->mk_integer (sc, b),
-                                     sc->NIL)));
-                  sc->vptr->set_vector_elem (vector, j, temp_val);
-                }
-
-              return_val = sc->vptr->cons (sc, vector, return_val);
-            }
-          else if (GIMP_VALUE_HOLDS_PARASITE (value))
-            {
-              GimpParasite *v = g_value_get_boxed (value);
-
-              if (v->name == NULL)
-                {
-                  /* Wrongly passed a Parasite that appears to be null, or other error. */
-                  return_val = implementation_error (sc, "Error: null parasite", 0);
-                }
-              else
-                {
-                  gchar   *data = g_strndup (v->data, v->size);
-                  gint     char_cnt = g_utf8_strlen (data, v->size);
-                  pointer  temp_val;
-
-                  /* don't move the mk_foo() calls outside this function call,
-                   * otherwise they might be garbage collected away!
-                   */
-                  temp_val = sc->vptr->cons
-                               (sc,
-                                sc->vptr->mk_string (sc, v->name),
-                                sc->vptr->cons
-                                  (sc,
-                                   sc->vptr->mk_integer (sc, v->flags),
-                                   sc->vptr->cons
-                                     (sc,
-                                      sc->vptr->mk_counted_string (sc,
-                                                                   data,
-                                                                   char_cnt),
-                                      sc->NIL)));
-
-                  return_val = sc->vptr->cons (sc,
-                                               temp_val,
-                                               return_val);
-                  g_free (data);
-
-                  g_debug ("name '%s'", v->name);
-                  g_debug ("flags %d", v->flags);
-                  g_debug ("size %d", v->size);
-                  g_debug ("data '%.*s'", v->size, (gchar *) v->data);
-                }
-            }
-          else if (GIMP_VALUE_HOLDS_OBJECT_ARRAY (value))
-            {
-              pointer vector = marshal_returned_object_array_to_vector (sc, value);
-              return_val = sc->vptr->cons (sc, vector, return_val);
-            }
-          else if (G_VALUE_TYPE (&value) == GIMP_TYPE_PDB_STATUS_TYPE)
-            {
-              /* Called procedure implemented incorrectly. */
-              return implementation_error (sc, "Procedure execution returned multiple status values", 0);
-            }
-          else
-            {
-              /* Missing cases here. */
-              g_snprintf (error_str, sizeof (error_str),
-                          "Unhandled return type %s",
-                          G_VALUE_TYPE_NAME (value));
-              return implementation_error (sc, error_str, 0);
-            }
-        }
-      break;
-
-    case GIMP_PDB_PASS_THROUGH:
-    case GIMP_PDB_CANCEL:   /*  should we do something here?  */
-      g_debug ("Status is PASS_THROUGH or CANCEL");
-      break;
-    }
-
-  /* If we have no non-status return value(s) from PDB call, return
-   * either TRUE or FALSE to indicate if call succeeded.
-   */
-  if (return_val == sc->NIL)
-    {
-      g_debug ("returning with only a status result");
-      if (GIMP_VALUES_GET_ENUM (values, 0) == GIMP_PDB_SUCCESS)
-        return_val = sc->vptr->cons (sc, sc->T, sc->NIL);
-      else
-        return_val = sc->vptr->cons (sc, sc->F, sc->NIL);
-    }
-  else
-    {
-      g_debug ("returning with non-empty result");
-    }
+    /* Now returns error immediately.
+     * Which leaks memory normally freed below.
+     * Most plugins, except extension script-fu, will exit soon anyway.
+     * FUTURE: don't leak.
+     */
+    if (calling_error != NULL)
+      /* calling error is foreign_error or similar. */
+      return calling_error;
+  }
 
   g_free (proc_name);
 
@@ -1681,7 +1479,6 @@ script_fu_marshal_procedure_call (scheme   *sc,
   gimp_value_array_unref (values);
 
   /*  free arguments and values  */
-  gimp_value_array_unref (args);
 
   /* The callback is NULL except for script-fu-server.  See explanation there. */
   if (post_command_callback != NULL)
