@@ -36,7 +36,9 @@
  */
 
 static gboolean script_fu_script_param_init (SFScript             *script,
-                                             const GimpValueArray *args,
+                                             GParamSpec          **pspecs,
+                                             guint                 n_pspecs,
+                                             GimpProcedureConfig  *config,
                                              SFArgType             type,
                                              gint                  n);
 static void     script_fu_script_set_proc_metadata (
@@ -158,12 +160,11 @@ script_fu_script_create_PDB_procedure (GimpPlugIn     *plug_in,
       g_debug ("script_fu_script_create_PDB_procedure: %s, plugin type %i, image_proc",
                script->name, plug_in_type);
 
-      procedure = gimp_image_procedure_new (
-                                      plug_in, script->name,
-                                      plug_in_type,
-                                      (GimpRunImageFunc) script_fu_run_image_procedure,
-                                      script, /* user_data, pointer in extension-script-fu process */
-                                      NULL);
+      procedure = gimp_image_procedure_new (plug_in, script->name,
+                                            plug_in_type,
+                                            (GimpRunImageFunc) script_fu_run_image_procedure,
+                                            script, /* user_data, pointer in extension-script-fu process */
+                                            NULL);
 
       script_fu_script_set_proc_metadata (procedure, script);
 
@@ -266,23 +267,25 @@ script_fu_script_reset (SFScript *script,
 
 gint
 script_fu_script_collect_standard_args (SFScript             *script,
-                                        const GimpValueArray *args)
+                                        GParamSpec          **pspecs,
+                                        guint                 n_pspecs,
+                                        GimpProcedureConfig  *config)
 {
   gint params_consumed = 0;
 
   g_return_val_if_fail (script != NULL, 0);
 
   /*  the first parameter may be a DISPLAY id  */
-  if (script_fu_script_param_init (script,
-                                   args, SF_DISPLAY,
+  if (script_fu_script_param_init (script, pspecs, n_pspecs,
+                                   config, SF_DISPLAY,
                                    params_consumed))
     {
       params_consumed++;
     }
 
   /*  an IMAGE id may come first or after the DISPLAY id  */
-  if (script_fu_script_param_init (script,
-                                   args, SF_IMAGE,
+  if (script_fu_script_param_init (script, pspecs, n_pspecs,
+                                   config, SF_IMAGE,
                                    params_consumed))
     {
       params_consumed++;
@@ -290,17 +293,17 @@ script_fu_script_collect_standard_args (SFScript             *script,
       /*  and may be followed by a DRAWABLE, LAYER, CHANNEL or
        *  VECTORS id
        */
-      if (script_fu_script_param_init (script,
-                                       args, SF_DRAWABLE,
+      if (script_fu_script_param_init (script, pspecs, n_pspecs,
+                                       config, SF_DRAWABLE,
                                        params_consumed) ||
-          script_fu_script_param_init (script,
-                                       args, SF_LAYER,
+          script_fu_script_param_init (script, pspecs, n_pspecs,
+                                       config, SF_LAYER,
                                        params_consumed) ||
-          script_fu_script_param_init (script,
-                                       args, SF_CHANNEL,
+          script_fu_script_param_init (script, pspecs, n_pspecs,
+                                       config, SF_CHANNEL,
                                        params_consumed) ||
-          script_fu_script_param_init (script,
-                                       args, SF_VECTORS,
+          script_fu_script_param_init (script, pspecs, n_pspecs,
+                                       config, SF_VECTORS,
                                        params_consumed))
         {
           params_consumed++;
@@ -339,7 +342,9 @@ script_fu_script_get_command (SFScript *script)
 
 gchar *
 script_fu_script_get_command_from_params (SFScript             *script,
-                                          const GimpValueArray *args)
+                                          GParamSpec          **pspecs,
+                                          guint                 n_pspecs,
+                                          GimpProcedureConfig  *config)
 {
   GString *s;
   gint     i;
@@ -351,13 +356,18 @@ script_fu_script_get_command_from_params (SFScript             *script,
 
   for (i = 0; i < script->n_args; i++)
     {
-      GValue *value = gimp_value_array_index (args, i + 1);
+      GValue      value = G_VALUE_INIT;
+      GParamSpec *pspec = pspecs[i + SF_ARG_TO_CONFIG_OFFSET];
+
+      g_value_init (&value, pspec->value_type);
+      g_object_get_property (G_OBJECT (config), pspec->name, &value);
 
       g_string_append_c (s, ' ');
 
       script_fu_arg_append_repr_from_gvalue (&script->args[i],
                                              s,
-                                             value);
+                                             &value);
+      g_value_unset (&value);
     }
 
   g_string_append_c (s, ')');
@@ -395,11 +405,15 @@ script_fu_script_get_command_for_image_proc (SFScript            *script,
                                              GimpImage            *image,
                                              guint                 n_drawables,
                                              GimpDrawable        **drawables,
-                                             const GimpValueArray *args)
+                                             GimpProcedureConfig  *config)
 {
-  GString *s;
+  GParamSpec **pspecs;
+  guint        n_pspecs;
+  GString     *s;
+  gint         i;
 
   g_return_val_if_fail (script != NULL, NULL);
+  g_return_val_if_fail (GIMP_IS_PROCEDURE_CONFIG (config), NULL);
 
   s = g_string_new ("(");
   g_string_append (s, script->name);
@@ -420,20 +434,28 @@ script_fu_script_get_command_for_image_proc (SFScript            *script,
    */
   script_fu_command_append_drawables (s, n_drawables, drawables);
 
-  /* args contains the "other" args
+  /* config contains the "other" args
    * Iterate over the GimpValueArray.
    * But script->args should be the same length, and types should match.
    */
-  for (guint i = 0; i < gimp_value_array_length (args); i++)
+  pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (config), &n_pspecs);
+
+  /* config will have 1 additional property: "procedure". */
+  for (i = 1; i < n_pspecs; i++)
     {
-      GValue *value = gimp_value_array_index (args, i);
+      GParamSpec *pspec = pspecs[i];
+      GValue      value = G_VALUE_INIT;
+
       g_string_append_c (s, ' ');
-      script_fu_arg_append_repr_from_gvalue (&script->args[i],
-                                             s,
-                                             value);
+
+      g_value_init (&value, pspec->value_type);
+      g_object_get_property (G_OBJECT (config), pspec->name, &value);
+      script_fu_arg_append_repr_from_gvalue (&script->args[i - 1], s, &value);
+      g_value_unset (&value);
     }
 
   g_string_append_c (s, ')');
+  g_free (pspecs);
 
   return g_string_free (s, FALSE);
 }
@@ -462,7 +484,9 @@ script_fu_script_infer_drawable_arity (SFScript *script)
 
 static gboolean
 script_fu_script_param_init (SFScript             *script,
-                             const GimpValueArray *args,
+                             GParamSpec          **pspecs,
+                             guint                 n_pspecs,
+                             GimpProcedureConfig  *config,
                              SFArgType             type,
                              gint                  n)
 {
@@ -470,16 +494,20 @@ script_fu_script_param_init (SFScript             *script,
 
   if (script->n_args > n &&
       arg->type == type  &&
-      gimp_value_array_length (args) > n + 1)
+      /* The first pspec is "procedure", the second is "run-mode". */
+      n_pspecs > n + SF_ARG_TO_CONFIG_OFFSET)
     {
-      GValue *value = gimp_value_array_index (args, n + 1);
+      GValue      value = G_VALUE_INIT;
+      GParamSpec *pspec = pspecs[n + SF_ARG_TO_CONFIG_OFFSET];
 
+      g_value_init (&value, pspec->value_type);
+      g_object_get_property (G_OBJECT (config), pspec->name, &value);
       switch (type)
         {
         case SF_IMAGE:
-          if (GIMP_VALUE_HOLDS_IMAGE (value))
+          if (GIMP_VALUE_HOLDS_IMAGE (&value))
             {
-              GimpImage *image = g_value_get_object (value);
+              GimpImage *image = g_value_get_object (&value);
 
               arg->value.sfa_image = gimp_image_get_id (image);
               return TRUE;
@@ -487,9 +515,9 @@ script_fu_script_param_init (SFScript             *script,
           break;
 
         case SF_DRAWABLE:
-          if (GIMP_VALUE_HOLDS_DRAWABLE (value))
+          if (GIMP_VALUE_HOLDS_DRAWABLE (&value))
             {
-              GimpItem *item = g_value_get_object (value);
+              GimpItem *item = g_value_get_object (&value);
 
               arg->value.sfa_drawable = gimp_item_get_id (item);
               return TRUE;
@@ -497,9 +525,9 @@ script_fu_script_param_init (SFScript             *script,
           break;
 
         case SF_LAYER:
-          if (GIMP_VALUE_HOLDS_LAYER (value))
+          if (GIMP_VALUE_HOLDS_LAYER (&value))
             {
-              GimpItem *item = g_value_get_object (value);
+              GimpItem *item = g_value_get_object (&value);
 
               arg->value.sfa_layer = gimp_item_get_id (item);
               return TRUE;
@@ -507,9 +535,9 @@ script_fu_script_param_init (SFScript             *script,
           break;
 
         case SF_CHANNEL:
-          if (GIMP_VALUE_HOLDS_CHANNEL (value))
+          if (GIMP_VALUE_HOLDS_CHANNEL (&value))
             {
-              GimpItem *item = g_value_get_object (value);
+              GimpItem *item = g_value_get_object (&value);
 
               arg->value.sfa_channel = gimp_item_get_id (item);
               return TRUE;
@@ -517,9 +545,9 @@ script_fu_script_param_init (SFScript             *script,
           break;
 
         case SF_VECTORS:
-          if (GIMP_VALUE_HOLDS_VECTORS (value))
+          if (GIMP_VALUE_HOLDS_VECTORS (&value))
             {
-              GimpItem *item = g_value_get_object (value);
+              GimpItem *item = g_value_get_object (&value);
 
               arg->value.sfa_vectors = gimp_item_get_id (item);
               return TRUE;
@@ -527,9 +555,9 @@ script_fu_script_param_init (SFScript             *script,
           break;
 
         case SF_DISPLAY:
-          if (GIMP_VALUE_HOLDS_DISPLAY (value))
+          if (GIMP_VALUE_HOLDS_DISPLAY (&value))
             {
-              GimpDisplay *display = g_value_get_object (value);
+              GimpDisplay *display = g_value_get_object (&value);
 
               arg->value.sfa_display = gimp_display_get_id (display);
               return TRUE;
@@ -539,6 +567,7 @@ script_fu_script_param_init (SFScript             *script,
         default:
           break;
         }
+      g_value_unset (&value);
     }
 
   return FALSE;
