@@ -50,6 +50,8 @@ typedef struct
   gint   history;
 } SFEnum;
 
+typedef gint32 SFResourceType;
+
 typedef union
 {
   gint32         sfa_image;
@@ -63,11 +65,7 @@ typedef union
   gchar         *sfa_value;
   SFAdjustment   sfa_adjustment;
   SFFilename     sfa_file;
-  gchar         *sfa_font;
-  gchar         *sfa_gradient;
-  gchar         *sfa_palette;
-  gchar         *sfa_pattern;
-  gchar         *sfa_brush;
+  SFResourceType sfa_resource;
   SFOption       sfa_option;
   SFEnum         sfa_enum;
 } SFArgValue;
@@ -75,7 +73,8 @@ typedef union
 typedef struct
 {
   SFArgType   type;
-  gchar      *label;
+  gchar      *label;         /* label on widget in dialog. Not unique. */
+  gchar      *property_name; /* name of property of Procedure. Unique. */
   SFArgValue  default_value;
   SFArgValue  value;
 } SFArg;
@@ -96,6 +95,14 @@ typedef struct
   GType           proc_class; /* GimpProcedure or GimpImageProcedure. */
 } SFScript;
 
+/* ScriptFu keeps array of SFArg, it's private arg specs.
+ * Scripts declare args except run_mode (SF hides it.)
+ * A GimpConfig has two extra leading properties, for procedure and run_mode.
+ * A set of GParamSpecs is derived from a config and also has two extra.
+ * This defines the difference in length between SF's arg spec array and a config.
+ */
+#define SF_ARG_TO_CONFIG_OFFSET 2
+
 typedef struct
 {
   SFScript *script;   /* script which defined this menu path and label */
