@@ -61,21 +61,11 @@
 #include <glib.h>
 
 #ifdef G_OS_WIN32
-#ifdef _WIN32_WINNT
-#undef _WIN32_WINNT
-#endif
-#define _WIN32_WINNT 0x0502
 #include <winsock2.h>
 #include <ws2tcpip.h>
 
 typedef short sa_family_t; /* Not defined by winsock */
 
-#ifndef AI_ADDRCONFIG
-/* Missing from mingw headers, but value is publicly documented
- * on http://msdn.microsoft.com/en-us/library/ms737530%28v=VS.85%29.aspx
- */
-#define AI_ADDRCONFIG 0x0400
-#endif
 #include <libgimpbase/gimpwin32-io.h>
 #else
 #include <sys/socket.h>
@@ -271,18 +261,20 @@ script_fu_server_post_command (void)
 
 GimpValueArray *
 script_fu_server_run (GimpProcedure        *procedure,
-                      const GimpValueArray *args)
+                      GimpProcedureConfig  *config)
 {
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
   GimpRunMode        run_mode;
-  const gchar       *ip;
+  gchar             *ip;
   gint               port;
-  const gchar       *logfile;
+  gchar             *logfile;
 
-  run_mode = GIMP_VALUES_GET_ENUM   (args, 0);
-  ip       = GIMP_VALUES_GET_STRING (args, 1);
-  port     = GIMP_VALUES_GET_INT    (args, 2);
-  logfile  = GIMP_VALUES_GET_STRING (args, 3);
+  g_object_get (config,
+                "run-mode", &run_mode,
+                "ip",       &ip,
+                "port",     &port,
+                "logfile",  &logfile,
+                NULL);
 
   script_fu_set_run_mode (run_mode);
   script_fu_set_print_flag (1);
@@ -312,6 +304,8 @@ script_fu_server_run (GimpProcedure        *procedure,
     default:
       break;
     }
+  g_free (ip);
+  g_free (logfile);
 
   return gimp_procedure_new_return_values (procedure, status, NULL);
 }
